/*
 *  Gary Jennejohn (C) 2003 <gj@denx.de>
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Routines for generic manipulation of the interrupts found on the 
 * AMAZON_SE boards.
 */


#include "amazon_se.h"
#include "irq.h"

void disable_amazon_se_irq(unsigned int irq_nr)
{
	/* have to access the correct register here */
	if (irq_nr <= INT_NUM_IM0_IRL31 && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
		*AMAZON_SE_ICU_IM0_IER &= (~AMAZON_SE_ICU_IM0_IER_IR(irq_nr));
	} 
	else if (irq_nr <= INT_NUM_IM1_IRL31 && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
		*AMAZON_SE_ICU_IM1_IER &= (~AMAZON_SE_ICU_IM1_IER_IR(irq_nr - 32));
	}
	else if (irq_nr <= INT_NUM_IM2_IRL31 && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
		*AMAZON_SE_ICU_IM2_IER &= (~AMAZON_SE_ICU_IM2_IER_IR(irq_nr - 64));
	}else if (irq_nr <= INT_NUM_IM3_IRL31 && irq_nr >= INT_NUM_IM3_IRL0)
	{
		/* access IM3 */
		*AMAZON_SE_ICU_IM3_IER &= (~AMAZON_SE_ICU_IM3_IER_IR((irq_nr - 96)));
	}else if (irq_nr <= INT_NUM_IM4_IRL31 && irq_nr >= INT_NUM_IM4_IRL0)
	{
		/* access IM4 */
		*AMAZON_SE_ICU_IM4_IER &= (~AMAZON_SE_ICU_IM4_IER_IR((irq_nr - 128)));
	}

}
void mask_and_ack_amazon_se_irq(unsigned int irq_nr)
{
	/* have to access the correct register here */
	if (irq_nr <= INT_NUM_IM0_IRL31 && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
		/* mask */
		*AMAZON_SE_ICU_IM0_IER &= ~AMAZON_SE_ICU_IM0_IER_IR(irq_nr);
		/* ack */
		*AMAZON_SE_ICU_IM0_ISR = AMAZON_SE_ICU_IM0_ISR_IR(irq_nr);
	}
	else if (irq_nr <= INT_NUM_IM1_IRL31 && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
		/* mask */
		*AMAZON_SE_ICU_IM1_IER &= ~AMAZON_SE_ICU_IM1_IER_IR(irq_nr - 32);
		/* ack */
		*AMAZON_SE_ICU_IM1_ISR = AMAZON_SE_ICU_IM1_ISR_IR(irq_nr - 32);
	}
	else if (irq_nr <= INT_NUM_IM2_IRL31 && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
		/* mask */
		*AMAZON_SE_ICU_IM2_IER &= ~AMAZON_SE_ICU_IM2_IER_IR(irq_nr - 64);
		/* ack */
		*AMAZON_SE_ICU_IM2_ISR = AMAZON_SE_ICU_IM2_ISR_IR(irq_nr - 64);
	}
	else if (irq_nr <= INT_NUM_IM3_IRL31 && irq_nr >= INT_NUM_IM3_IRL0)
	{
		/* access IM3 */
		/* mask */
		*AMAZON_SE_ICU_IM3_IER &= ~AMAZON_SE_ICU_IM3_IER_IR(irq_nr - 96);
		/* ack */
		*AMAZON_SE_ICU_IM3_ISR = AMAZON_SE_ICU_IM3_ISR_IR(irq_nr - 96);
	}
	else if (irq_nr <= INT_NUM_IM4_IRL31 && irq_nr >= INT_NUM_IM4_IRL0)
	{
		/* access IM4 */
		/* mask */		
		*AMAZON_SE_ICU_IM4_IER &= ~AMAZON_SE_ICU_IM4_IER_IR(irq_nr - 128);
		/* ack */
		*AMAZON_SE_ICU_IM4_ISR = AMAZON_SE_ICU_IM4_ISR_IR(irq_nr - 128);
	}
}
void enable_amazon_se_irq(unsigned int irq_nr)
{
	/* have to access the correct register here */
	if (irq_nr <= INT_NUM_IM0_IRL31 && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
		*AMAZON_SE_ICU_IM0_IER |= AMAZON_SE_ICU_IM0_IER_IR(irq_nr);
	}
	else if (irq_nr <= INT_NUM_IM1_IRL31 && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
		*AMAZON_SE_ICU_IM1_IER |= AMAZON_SE_ICU_IM1_IER_IR(irq_nr - 32);
	}
	else if (irq_nr <= INT_NUM_IM2_IRL31 && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
		*AMAZON_SE_ICU_IM2_IER |= AMAZON_SE_ICU_IM2_IER_IR(irq_nr - 64);
	}
	else if (irq_nr <= INT_NUM_IM3_IRL31 && irq_nr >= INT_NUM_IM3_IRL0)
	{
		/* access IM3 */
		*AMAZON_SE_ICU_IM3_IER |= AMAZON_SE_ICU_IM3_IER_IR((irq_nr - 96));
	}
	else if (irq_nr <= INT_NUM_IM4_IRL31 && irq_nr >= INT_NUM_IM4_IRL0)
	{
		/* access IM4 */
		*AMAZON_SE_ICU_IM4_IER |= AMAZON_SE_ICU_IM4_IER_IR((irq_nr - 128));
	}

}
