/*****************************************************************************
 * DANUBE BootROM
 * Copyright (c) 2005, Infineon Technologies AG, All rights reserved
 * IFAP DC COM SD
 * SFlash AT45DB321C *
 *****************************************************************************/

#include <config.h>
#include "lib.h"
#include "ssc.h"
#include "sflash.h"
#include "ssc_extra.h"
//#include <asc.h>
#include <asm/addrspace.h>
#include <asm/ifx_mips.h>
#include "bstrap.h"
#include <asm/mipsregs.h>
#include "sflash_extra.h"

#define TEST_IMG	EEPROM_CONTENT	 

#undef DEBUG_SFLASH_OUTPUT
//#define DEBUG_SFLASH_OUTPUT

#if defined(DEBUG_SFLASH_OUTPUT)
#define DEBUG_SFLASH(string)	asc_puts(string)
#define DEBUG_SFLASH_U8(string)	print_u8(string)
#define DEBUG_SFLASH_U32(string)	print_u32(string)
#else
#define DEBUG_SFLASH(string)
#define DEBUG_SFLASH_U8(string)
#define DEBUG_SFLASH_U32(string)
#endif

/* dupm sflash data out */
#undef DUMP_SFLASH_DATA

//#define PAGE_SIZE_BIG		1056
#define PAGE_SIZE_SMALL		256

//#define SECTOR_SIZE		0x10000  /*IFX board  Commented by Manamohan */
#define SECTOR_SIZE		0x1000

#define MAIN_MEM_PAGE_READ	0xD2

#define CONT_ARRAY_READ		0x03
#define FAST_READ		0x0B

#define BUFFER1_WRITE		0x84
#define BUFFER2_WRITE		0x87
#define BUFFER1_TO_PAGE_MEM	0x88
#define BUFFER2_TO_PAGE_MEM	0x89

//#define PAGE_ERASE		0x81
//#define SECTOR_ERASE		0xD8 /*Commented by Manamohan */
#define SECTOR_ERASE		0x20
//#define BLOCK_ERASE		0x52
#define CHIP_ERASE		0x60

#define WREN			0x06

#define PAGE_PROGRAM		0x02

#define STATUS_OPCODE		0x05 // yclee: RDSR
#define STATUS_RDBY		0x01 // yclee: WIP

#if 0
typedef struct 
{
	u8 manuf_id;
	u8 device_id1;
	u8 device_id2;
   u32 sector_size;
   u8 sector_erase;
   u8 chip_erase;
   u32 page_size;
   u8 page_write;
} flash_type;
#endif

flash_type flash_found;

//move to sflash_extra.h, Leon 2008/11/6
#if 0
#define RDID                        0x9F
#define EWSR                        0x50
#define WRSR                        0x01
#define RDSR                        0x05

#define MXIC_MANUF_ID               0xC2
#define MXIC_SECTOR_SIZE            0x1000
//#define MXIC_D_SECTOR_SIZE            0x1000   //MX25L1605D, MX25L3205D
//#define MXIC_A_SECTOR_SIZE            0x10000   //MX25L3205A
#define MXIC_SECTOR_ERASE_CMD       0x20     
#define MXIC_CHIP_ERASE_CMD         0x60
#define MXIC_PAGE_SIZE              256
#define MXIC_PAGE_WRITE_CMD         0x02

#define WINBOND_MANUF_ID           0xEF
#define WINBOND_SECTOR_SIZE        0x1000
#define WINBOND_SECTOR_ERASE_CMD   0x20
#define WINBOND_CHIP_ERASE_CMD     0x60
#define WINBOND_PAGE_SIZE          256
#define WINBOND_PAGE_WRITE_CMD     0x02
#endif


//#define BYTE_ADDR_BIG(addr)		(addr%PAGE_SIZE_BIG)
//#define PAGE_ADDR_BIG(addr)		(addr/PAGE_SIZE_BIG)
//#define BYTE_ADDR_SMALL(addr)		(addr%PAGE_SIZE_SMALL)
//#define PAGE_ADDR_SMALL(addr)		(addr/PAGE_SIZE_SMALL)

spi_dev_t spi_sflash;
typedef void (*convert_addr_t)(u32 address, u8 * addr);
void at45db321c_convert_addr(u32 address, u8* addr);
//int  _program_flash(u8 * wr_buf, u32 data_len,u8 addr_cycles, u32 page_size, convert_addr_t convert,int verify,int offset);

int _program_flash(u8 * wr_buf, u32 data_len,u8 addr_cycles, u32 page_size,
                convert_addr_t convert,int verify,u32 start_addr);
void _dump_sflash(u8 addr_cycles,u32 page_size, u32 page_no, convert_addr_t convert);
void _sflash_init(void);

void sflash_sync(void)	
{
	volatile u32 temp;
	int ready=0;
	while(ready==0){
		spi_session(&spi_sflash,0x05,0,3,0,0,(u32*)&temp,1);

#if 0		
		asc_puts("\ntemp = ");
		print_u32(temp);
		asc_puts("\n\n\n\n");
#endif
		
		if((temp&STATUS_RDBY)==0){
			ready=1;
		}
	}
}

void flash_chip_erase(void)
{
	spi_session(&spi_sflash,WREN,0,0,0,0,0,0); // write_enable
        //printf("\n inside flash_chip_erase"); 
	//spi_session(&spi_sflash,CHIP_ERASE,0,0,0,0,0,0);
	spi_session(&spi_sflash,flash_found.chip_erase,0,0,0,0,0,0);
	sflash_sync();
	return;
}


void flash_sector_erase(u8 * addr)
{
	spi_session(&spi_sflash,WREN,0,0,0,0,0,0); // write_enable

	//spi_session(&spi_sflash,SECTOR_ERASE,addr,0,0,0,0,0);
	spi_session(&spi_sflash,flash_found.sector_erase,addr,0,0,0,0,0);
	sflash_sync();
	return;
}

void flash_read_identification(void)
{
   volatile u32 buf = 0x0;
   volatile u32 status=0x0;

	spi_session(&spi_sflash,RDID,0,0,0,0,&buf,4); // write_enable
	sflash_sync();
   flash_found.manuf_id = (buf >> 24) & 0xFF;
	flash_found.device_id1 = (buf >>16) & 0xFF;
	flash_found.device_id2 = (buf >>8) & 0xFF;
   //printf("\n%s,%d,buf=0x%x\n",__FUNCTION__,__LINE__,buf);
   switch(flash_found.manuf_id) 
   {
           case MXIC_MANUF_ID:
               flash_found.sector_size = MXIC_SECTOR_SIZE;
               flash_found.sector_erase = MXIC_SECTOR_ERASE_CMD;
               flash_found.chip_erase = MXIC_CHIP_ERASE_CMD;
               flash_found.page_size = MXIC_PAGE_SIZE;
               flash_found.page_write = MXIC_PAGE_WRITE_CMD;
		
		if(flash_found.device_id2==0x15)
			printf("\nFlash: 2MB ");
		else if(flash_found.device_id2==0x16)
			printf("\nFlash: 4MB ");
		else
			printf("\nFlash: unknown ");
		printf("\nMXIC_MANUF_ID found, memory type=%x, memory density=%x\n", flash_found.device_id1, flash_found.device_id2);
               break;
           case WINBOND_MANUF_ID:
               flash_found.sector_size = WINBOND_SECTOR_SIZE;
               flash_found.sector_erase = WINBOND_SECTOR_ERASE_CMD;
               flash_found.chip_erase = WINBOND_CHIP_ERASE_CMD;
               flash_found.page_size = WINBOND_PAGE_SIZE;
               flash_found.page_write = WINBOND_PAGE_WRITE_CMD;

		 printf("\nFlash: 2MB ");
		
		printf("\nWINBOND_MANUF_ID found, memory type=%x, memory density=%x\n", flash_found.device_id1, flash_found.device_id2);
	       break;
           

              

          default:
		flash_found.sector_size = MXIC_SECTOR_SIZE;
               	flash_found.sector_erase = MXIC_SECTOR_ERASE_CMD;
               	flash_found.chip_erase = MXIC_CHIP_ERASE_CMD;
               	flash_found.page_size = MXIC_PAGE_SIZE;
               	flash_found.page_write = MXIC_PAGE_WRITE_CMD;
	
               printf("\nEntered the default section in the switch case\n");
               break;
   }

	#if 0	
   printf ("flash_found.manuf_id = %x\n", flash_found.manuf_id);
   printf ("flash_found.sector_size = %d\n", flash_found.sector_size);
   printf ("flash_found.sector_erase = %x\n", flash_found.sector_erase);
   printf ("flash_found.chip_erase = %x\n", flash_found.chip_erase);
   printf ("flash_found.page_size = %d\n", flash_found.page_size);
   printf ("flash_found.page_write = %x\n", flash_found.page_write);
	#endif

	return;
}
u8 manufacture_id(void)
{
  return(flash_found.manuf_id);
}
void page_write(u8 *addr, u8 *buf,u32 len)
{
	spi_session(&spi_sflash,WREN,0,0,0,0,0,0); // write_enable

	//spi_session(&spi_sflash,PAGE_PROGRAM,addr,0,buf,len,0,0);
	spi_session(&spi_sflash,flash_found.page_write,addr,0,buf,len,0,0);
	sflash_sync();
	return;
}

void flash_read(u8 *addr, u32 *buf, u32 len)
{
#if 0
	spi_session(&spi_sflash,CONT_ARRAY_READ,addr,0,0,0,buf,len);
#else	
	spi_session(&spi_sflash,FAST_READ,addr,1,0,0,buf,len);
#endif
	
	return;
}

int compare_data(u8* src, u8* dst,u32 size)
{
	int i=0;
	int result=1;
	for(i=0;i<size;i++)
	{
		if(*(src+i)!=*(dst+i))
		{
         printf("\nCompare data falied\n");
			result=0;
			asc_puts("diff: [");
			print_u32((u32)((src+i)));
			asc_puts("]= ");
			print_u8((u32)(*(src+i)));
			asc_puts("--- [");
			print_u32((u32)((dst+i)));
			asc_puts("]= ");
			print_u8((u32)(*(dst+i)));
			asc_puts("\n");
		}
	}
	return result;

}

/*	Brief:	convert address from u32 to u8 array for AT45DB321C
 *	Parameter:
 *		address: the address, maximum 32 bit
 *		addr:	array that holds the results, maximum 4 elemets
 *		addr_cycles: number of real data bytes=, maximum 4
 */
void at45db321c_convert_addr(u32 address, u8* addr)
{
	addr[0] = (u8) ((address>>16) & 0xff);
	addr[1] = (u8) ((address>>8) & 0xff);
	addr[2] = (u8) (address & 0xff);
}

#define MAX_ADDRESS_NUM		5

//static u32 g_sflash_content[] = TEST_IMG;

#if 0 /*Commented by Manamohan */

/*  Brief: read certain number of page
 *  Parameter:
 *    addr_cycles: number of address (eg, 3 bytes)
 *    page_size: eg, 512 bytes
 *    page_no: number of pages to be dump
 */
void _dump_sflash(u8 addr_cycles,u32 page_size, u32 page_no, convert_addr_t convert)
{
	u8 addr[MAX_ADDRESS_NUM];
	u32 address=0;
	u32 rd_buf[PAGE_SIZE_SMALL/4];
	int i,j;
	u32 eff_len,eff_len_in_page;

	spi_sflash.addr_cycles = addr_cycles;
	sflash_sync();
	asc_puts("page no: ");print_u32(page_no); asc_puts("\n");
	(*convert)(address,addr);
	for(i=0;i<page_no;i++){
                eff_len = page_size;
		for(;eff_len>0;){
			eff_len_in_page=(eff_len>PAGE_SIZE_SMALL)?(PAGE_SIZE_SMALL):(eff_len);
			flash_read(addr,rd_buf,eff_len_in_page);
			asc_puts("\n");
			for(j=0;j<eff_len_in_page/4;j++){
				print_u32(rd_buf[j]); asc_puts(" ");
                                if ( j % 8 == 7) asc_puts("\n");
			}
			asc_puts("\n");
			address+=eff_len_in_page;
			(*convert)(address,addr);
			eff_len-=eff_len_in_page;
		}
	}
#endif

int amazon_se_sflash_read(u8* buffer,u32 cnt,u32 offset)
{
   int err=0;
	u8 addr[MAX_ADDRESS_NUM];
   u32 rem=0;
#if 0
   //_sflash_init();
   //printf("\nspi_sflash.addr_cycles = %x\n", spi_sflash.addr_cycles);
  //printf("\ncnt=0x%x,offset=0x%x,buffer[0]=0x%x",cnt,offset,buffer[0]);
   err=sflash_read_block((u32 *)buffer,cnt,offset);
   if(err < 0)
      return err;
   else
      return cnt; 
#endif
   
   //_sflash_init();
#if 1
   rem = cnt % 4;
   if(rem !=0)
   {
   if(rem == 1)
           rem +=3;
   else
      if(rem ==2)
              rem+=2;
      else
              rem+=1;
   }
#endif
   

	sflash_sync();
   spi_sflash.addr_cycles = 3;
   at45db321c_convert_addr(offset,addr);
   flash_read(addr,(u32*)buffer,cnt+rem);
   return cnt;
}
        
int amazon_se_sflash_write (u8* buffer,u32 cnt,u32 offset)
{
#if 1
   int err=0;

   _sflash_init();
   err=_program_flash(buffer,cnt,3,PAGE_SIZE_SMALL,at45db321c_convert_addr,0,offset);
   if(err)
      return err;
   else
      return cnt;
#endif
//return(amazon_se_sflash_partial_write (buffer, cnt,offset));

}

int amazon_se_sflash_partial_write (u8* buffer,u32 cnt,u32 offset)
{
   int err=0;

   _sflash_init();
   err=_program_partial_flash(buffer,cnt,3,PAGE_SIZE_SMALL,at45db321c_convert_addr,0,offset);
   if(err)
      return err;
   else
      return cnt;
}

int _program_partial_flash(u8 * wr_buf, u32 data_len,u8 addr_cycles, u32 page_size, convert_addr_t convert,int verify,u32 start_addr)
{
	u8 addr[MAX_ADDRESS_NUM];
//	u32 start_addr = 0;
	u32 address;
	int i=0,j=0,k=0,len=0;
	u32 page_no=0;
	u32 sector_no=0;
	u32 eff_len,eff_len_in_page;
	int err=0;
   u32 sect_number=0;
   u32 sect_end_addr=0;
   u8* read_buffer,*read_buffer_temp;
   u32 start_addr_temp;
   u32 first_sector_addr;

	spi_sflash.addr_cycles = addr_cycles;
	sflash_sync();
	// SECTOR_SIZE is different from page program size!!!!!		               
	page_no=(data_len%(flash_found.page_size))==0?(data_len/flash_found.page_size):(data_len/(flash_found.page_size)+1);
   
   if(start_addr / flash_found.sector_size == 0)
           first_sector_addr = 0;
   else
           first_sector_addr = (start_addr / flash_found.sector_size) * flash_found.sector_size;
   //printf("\nfirst_sector_addr=0x%x\n",first_sector_addr);

   if(start_addr % flash_found.sector_size == 0)
   {
	   sector_no=(data_len%flash_found.sector_size)==0?(data_len/flash_found.sector_size):(data_len/flash_found.sector_size+1);

   //printf("\nEnterd if  part of number of sector cal\n");
   }
   else
   {
       //printf("\nEnterd else part of number of sector cal\n");
/* If the start address is not aligned   */
       len = data_len; 
       start_addr_temp = start_addr;
       do {
            sect_number = start_addr_temp / flash_found.sector_size;
            sect_end_addr = (sect_number * flash_found.sector_size) + flash_found.sector_size - 1;
            len = len - (sect_end_addr - start_addr_temp) + 1;
            start_addr_temp = sect_end_addr + 1;
            sector_no++;
      }while(len > 0);

   }

/* Read the each sector and modify the data and then erase and write back */
   read_buffer = (u8*)malloc (sizeof(u8) * flash_found.sector_size);
   read_buffer_temp = read_buffer; 
   start_addr_temp = start_addr;
   for(i=0; i < sector_no; i++)
   {
      read_buffer = read_buffer_temp;
#if 1
      _sflash_init(); 
      if(amazon_se_sflash_read(read_buffer,flash_found.sector_size,first_sector_addr) !=
                     flash_found.sector_size)
      {
              printf("amazon_se_sflash_read fail\n");
              return -1;
      }
#endif

   //printf("\nfirst_sector_addr=0x%x\n",first_sector_addr);
/* Erase the readout sector */

#if 1
     (*convert)(first_sector_addr,addr);
		sflash_sync();
		flash_sector_erase(addr);
#endif

    
/* Update the new data in the buffer */
      read_buffer += (start_addr_temp - first_sector_addr);
      len = first_sector_addr + flash_found.sector_size - start_addr_temp;
      if(len > data_len)
      {
              len = data_len;
      }
      data_len -= len;
      for(j=0; j<len ; j++)
      {
         *(read_buffer + j) = *wr_buf++;
         //printf("\nread_buffer =0x%x,wr_buf=0x%x\n",*(read_buffer + j),*wr_buf);
      }
/* Write back the whole sector  */
      page_no = flash_found.sector_size /flash_found.page_size;
      read_buffer = read_buffer_temp;
      address = first_sector_addr;
	   for(k=0;k<page_no;k++)
      {
		   (*convert)(address,addr);
		   page_write(addr,read_buffer,flash_found.page_size);
         read_buffer += flash_found.page_size;
         address += flash_found.page_size;
	   }
      //printf("\nCompleted Page Write\n");
      first_sector_addr += flash_found.sector_size;
      start_addr_temp = first_sector_addr;

   }
   free(read_buffer_temp);
   return err;
}

/*  Brief:  program a serial flash and check [optional]
 *  Parameter:
 *    addr_cycles: number of address cycles
 *    page_size
 *    convert_addr_t: convert address
 *    verify: read back and compare
 *  Return:
 *    0 no err
 *    1 otherwise
 */
int _program_flash(u8 * wr_buf, u32 data_len,u8 addr_cycles, u32 page_size,
                convert_addr_t convert,int verify,u32 start_addr)
{
	u8 addr[MAX_ADDRESS_NUM];
//	u32 start_addr = 0;
	u32 address;
	u32 rd_buf[PAGE_SIZE_SMALL/4];
	int i;
	u32 page_no;
	u32 sector_no;
	u32 eff_len,eff_len_in_page;
	int err=0;

	spi_sflash.addr_cycles = addr_cycles;
	sflash_sync();

	// SECTOR_SIZE is different from page program size!!!!!		               
	page_no=(data_len%page_size)==0?(data_len/page_size):(data_len/page_size+1);
	sector_no=(data_len%SECTOR_SIZE)==0?(data_len/SECTOR_SIZE):(data_len/SECTOR_SIZE+1);


	// let's erase enough flash first
#if 1
	address = start_addr;
	for(i=0;i<sector_no;i++){
	        (*convert)(address,addr);
		sflash_sync();
		flash_sector_erase(addr);
		address += SECTOR_SIZE;
        }
#else
        asc_puts("\nDoing flash_chip_erase. This will take a while ...\n");
	flash_chip_erase();
#endif

	// now is time for programming flash
	address = start_addr;
	for(i=0;i<page_no;i++){
		(*convert)(address,addr);
		eff_len=(data_len>page_size)?page_size:data_len;
		page_write(addr,wr_buf,eff_len);
		for(;eff_len>0 && err==0;){
			eff_len_in_page=(eff_len>PAGE_SIZE_SMALL)?(PAGE_SIZE_SMALL):(eff_len);

                        if (verify) {
                            flash_read(addr,rd_buf,eff_len_in_page);
                            if (!compare_data(wr_buf,(u8 *)rd_buf,eff_len_in_page) ){
                               err=1;
                            }
                        }
			
			address+=eff_len_in_page;
			eff_len-=eff_len_in_page;
			wr_buf+=eff_len_in_page;
			
                        //asc_puts(".");
                          printf(".");
		}
		
		data_len-=page_size;
	}
        return err;
}

void _sflash_init(void)
{
   static int i=0;
  
  ssc_init((u32)SFLASH_BAUDRATE);
	spi_sflash.read_cmd    = SFLASH_READ;
  spi_sflash.cs = 0;
spi_sflash.addr_cycles = SPI_ADDR_CYC_MAX;
   if(i == 0)
   {
      flash_read_identification();
      i = 1;
   }

}

static inline u32 get_cycles (void)
{
          return read_c0_count();
}

#if 0
#define TEST_PAT_BYTESIZE	40000
u32 g_sflash_test_pattern[TEST_PAT_BYTESIZE/4];
#endif

#define COMBINED_IMAGE_HACK

#if 0

void test_sflash(void)
{
	u8 * wr_buf = (u8 *) g_sflash_content;
	u32 data_len=sizeof(g_sflash_content);


#ifdef COMBINED_IMAGE_HACK
   g_sflash_content[28] = 0x0004000 - 0x20;
#endif
#if 0
	u8 * wr_buf2 = (u8 *) g_sflash_test_pattern;
	u32 data_len2=sizeof(g_sflash_test_pattern);
#endif

  int err=0;

  u32 start_cycle, end_cycle;

#if 0
   int i;

   for (i = 0; i < TEST_PAT_BYTESIZE; i++)
   {
       g_sflash_test_pattern[i] = 0x5678abcd + i;
   } 
#endif

  _sflash_init();

start_cycle = get_cycles();      

#if 0
asc_puts("\n\n");
print_u32(data_len2);
asc_puts("\n\n");

  /* burn in dummy image first */
  err=_program_flash(wr_buf2,data_len2,3,PAGE_SIZE_SMALL,at45db321c_convert_addr,1);
#endif

asc_puts("\n\n");
print_u32(data_len);
asc_puts("\n\n");

  /* burn in actual desired image */
  err=_program_flash(wr_buf,data_len,3,PAGE_SIZE_SMALL,at45db321c_convert_addr,1);

end_cycle = get_cycles();

asc_puts("\ncycle start ....... \n");
print_u32(start_cycle);

asc_puts("\ncycle end ....... \n");
print_u32(end_cycle);

	if (err){
		asc_puts("\nSFLASH READ/WRITE TEST Fails\n");
	}else{
		asc_puts("\nSFLASH READ/WRITE TEST Pass\n");
	}
}
#endif


#if 0
void dump_at45_sflash(u32 page_no)
{
  /* for AT45DB321C */
  _dump_sflash(3,PAGE_SIZE_SMALL,page_no, at45db321c_convert_addr);
}

void test_write_sflash(void)
{
    u8 buf[8];
    u8 len = 4;
    u8 addr[4];
    u8 buf0[PAGE_SIZE_SMALL];
    u8 buf2[PAGE_SIZE_SMALL];
    int i;

    addr[0] = 0;
    addr[1] = 0;
    addr[2] = 0;
    addr[3] = 0;

    for (i = 0; i < PAGE_SIZE_SMALL; i++)
    {
        buf0[i] = 0x12 + i;
    }

    buf2[0] = 0x2a;
    buf2[1] = 0x4b;
    buf2[2] = 0x6c;
    buf2[3] = 0x8d;

   _sflash_init();

    sflash_sync();
    spi_session(&spi_sflash,0x9f,0,3,0,0,buf,len);
    asc_puts("RDID:\n");
    print_u8(buf[0]);
    print_u8(buf[1]);
    print_u8(buf[2]);    
    asc_puts("\n\n");

    spi_sflash.addr_cycles = 3;

    flash_sector_erase(addr);
    sflash_sync();

#if 0
    spi_session(&spi_sflash,0x9f,0,3,0,0,buf,len);
    asc_puts("RDID3:\n");
    print_u8(buf[0]);
    print_u8(buf[1]);
    print_u8(buf[2]);
    asc_puts("\n\n");
#endif

    page_write(addr, buf0, PAGE_SIZE_SMALL);
    sflash_sync();

#if 0
    spi_session(&spi_sflash,0x9f,0,3,0,0,buf,len);
    asc_puts("RDID2:\n");
    print_u8(buf[0]);
    print_u8(buf[1]);
    print_u8(buf[2]);
    asc_puts("\n\n");
#endif

    spi_session(&spi_sflash,CONT_ARRAY_READ,addr,0,0,0,buf2,PAGE_SIZE_SMALL);

    for (i = 0; i < PAGE_SIZE_SMALL; i++)
    {

        print_u8(buf0[i]);
        asc_puts(" -> ");
        print_u8(buf2[i]);
        asc_puts("\n\n");
    }
}

#endif
/*------------------------------------------------------------------------------*/
/* Function Name       : addr_dataflash 					*/
/* Object              : Test if address is valid				*/
/*------------------------------------------------------------------------------*/
int addr_serialflash (unsigned long addr)
{
	int addr_valid = 0;

	if ((((int) addr) & 0xF0000000) == 0) 
   {
			addr_valid = 1;
	}

	return addr_valid;
}

const char ascii[16] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a',
        'b', 'c', 'd', 'e', 'f' };

void print_u8(u8 data)
{
           char string[3]={0,0,0};

             string[0]=ascii[(data&0xf0)>>4];
               string[1]=ascii[data&0xf];

                  asc_puts(string);
}

void print_u32(u32 data)
{
           print_u8( (u8) (data>>24)&0xff);
              print_u8( (u8) (data>>16)&0xff);
                 print_u8( (u8) (data>>8)&0xff);
                    print_u8( (u8) (data>>0)&0xff);
}



/*
    Decrypt DSP FW
      data - pointer to encrypted firmware (32 bit aligned)
        n    - number of bytes in firmware (multiple of 32)
         */
void decrypt(u32 addr, int n)
{
       u32 * data = (u32*)addr;
       *data ^= 0x86B75CA1;
       while (n > 0)
       {
          *(data + 1) ^= *data;
          data++;
          n -= 4;
       }
}



void asc_puts (const char *s)
{
    while (*s)
    {
       putc (*s++);
    }
}

