/* ============================================================================
 * Copyright (C) 2003[- 2004] ? Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon 
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 * 
 * ============================================================================
 *
 * File Name: ifx_cfg.h
 * Author : Mars Lin (mars.lin@infineon.com)
 * Date: 
 *
 * ===========================================================================
 *
 * Project:
 * Block:
 *
 * ===========================================================================
 * Contents:  This file contains the data structures and definitions used 
 * 	      by the core iptables and the sip alg modules. 
 * ===========================================================================
 * References:
 */

/*
 * This file contains the configuration parameters for the IFX board.
 */
#ifndef _IFX_CFG_H_
#define _IFX_CFG_H_

/*-----------------------------------------------------------------------
 * U-Boot/Kernel configurations
 */
#define IFX_CFG_UBOOT_DEFAULT_CFG_IPADDR		"192.168.1.1"
#define IFX_CFG_UBOOT_DEFAULT_CFG_SERVERIP		"192.168.1.2"
#define IFX_CFG_UBOOT_DEFAULT_CFG_ETHADDR		"00:03:19:11:11:11"
#define IFX_CFG_UBOOT_DEFAULT_CFG_NETDEV		"eth0"
#define IFX_CFG_UBOOT_DEFAULT_CFG_BAUDRATE		"115200"
#define IFX_CFG_UBOOT_DEFAULT_CFG_TTY			"ttyS0"
#ifdef IFX_CONFIG_FLASH_TYPE_SPI
	#define IFX_CFG_UBOOT_DEFAULT_CFG_FLASH_ADDRESS		"0x0"
#else
	#define IFX_CFG_UBOOT_DEFAULT_CFG_FLASH_ADDRESS		"B0000000"
#endif

/* End of U-Boot/Kernel configurations
 *-----------------------------------------------------------------------
 */

/*-----------------------------------------------------------------------
 * Board specific configurations
 */
#ifdef IFX_CONFIG_MEMORY_SIZE
	#define IFX_CFG_MEM_SIZE	IFX_CONFIG_MEMORY_SIZE
#else
	#error "ERROR!! Define memory size first!"
#endif

#ifdef IFX_CONFIG_FLASH_TYPE_SPI
#if (IFX_CONFIG_FLASH_SIZE == 2)
#if (IFX_ADSL_FIRMWARE_IN_ROOTFS == 1)
#define IFX_CFG_FLASH_PARTITIONS_INFO                                   \
        "part0_begin=0x00000000\0"                                      \
        "part1_begin=0x00010000\0"                                      \
        "total_part=2\0"

#define IFX_CFG_FLASH_DATA_BLOCKS_INFO					\
	"data_block0=" IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME "\0"	\
	"data_block1=" IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME "\0"	\
	"data_block2=" IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME "\0"	\
	"data_block3=" IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME "\0"		\
	"data_block4=" IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME "\0"		\
	"data_block5=" IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME "\0"	\
	"total_db=6\0"

	#define IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME		"uboot"
	#define IFX_CFG_FLASH_UBOOT_IMAGE_START_ADDR		0x00000000
	#define IFX_CFG_FLASH_UBOOT_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_UBOOT_IMAGE_MTDBLOCK_NAME		"/dev/mtdblock0"

	#define IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME		"rootfs"
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_START_ADDR           0x00010000
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock1"

	#define IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME		"kernel"
	#define IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR		0x001FD1FF
	#define IFX_CFG_FLASH_KERNEL_IMAGE_SIZE			0

	#define IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME		"sysconfig"
	#define IFX_CFG_FLASH_SYSTEM_CFG_START_ADDR		0x001FD200
	#define IFX_CFG_FLASH_SYSTEM_CFG_SIZE			0
	#define IFX_CFG_FLASH_SYSTEM_CFG_END_ADDR		0xB01FF1FF

	#define IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME		"ubootconfig"
	#define IFX_CFG_FLASH_UBOOT_CFG_START_ADDR		0x001FF200
	#define IFX_CFG_FLASH_UBOOT_CFG_SIZE                    0x0C00
	#define IFX_CFG_FLASH_UBOOT_CFG_END_ADDR		0xB01FFDFF

	#define IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME		"fwdiag"
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_START_ADDR		0x001FFE00
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_SIZE		0x0200
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_END_ADDR		0xB01FFFFF

	#define IFX_CFG_FLASH_END_ADDR				0x001FFFFF
#else
#define IFX_CFG_FLASH_PARTITIONS_INFO                                   \
        "part0_begin=0x00000000\0"                                      \
        "part1_begin=0x00010000\0"                                      \
        "part2_begin=0x00040000\0"                                      \
        "total_part=3\0"

#define IFX_CFG_FLASH_DATA_BLOCKS_INFO					\
	"data_block0=" IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME "\0"	\
	"data_block1=" IFX_CFG_FLASH_FIRMWARE_IMAGE_BLOCK_NAME "\0"	\
	"data_block2=" IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME "\0"	\
	"data_block3=" IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME "\0"	\
	"data_block4=" IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME "\0"		\
	"data_block5=" IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME "\0"		\
	"data_block6=" IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME "\0"	\
	"total_db=7\0"

	#define IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME		"uboot"
	#define IFX_CFG_FLASH_UBOOT_IMAGE_START_ADDR		0x00000000
	#define IFX_CFG_FLASH_UBOOT_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_UBOOT_IMAGE_MTDBLOCK_NAME		"/dev/mtdblock0"

	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_BLOCK_NAME		"firmware"
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_START_ADDR         0x00010000
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_SIZE		0
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock1"

	#define IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME		"rootfs"
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_START_ADDR           0x00040000
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock1"

	#define IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME		"kernel"
	#define IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR		0x001FD1FF
	#define IFX_CFG_FLASH_KERNEL_IMAGE_SIZE			0

	#define IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME		"sysconfig"
	#define IFX_CFG_FLASH_SYSTEM_CFG_START_ADDR		0x001FD200
	#define IFX_CFG_FLASH_SYSTEM_CFG_SIZE			0
	#define IFX_CFG_FLASH_SYSTEM_CFG_END_ADDR		0xB01FF1FF

	#define IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME		"ubootconfig"
	#define IFX_CFG_FLASH_UBOOT_CFG_START_ADDR		0x001FF200
	#define IFX_CFG_FLASH_UBOOT_CFG_SIZE                    0x0C00
	#define IFX_CFG_FLASH_UBOOT_CFG_END_ADDR		0xB01FFDFF

	#define IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME		"fwdiag"
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_START_ADDR		0x001FFE00
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_SIZE		0x0200
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_END_ADDR		0xB01FFFFF

	#define IFX_CFG_FLASH_END_ADDR				0x001FFFFF


#endif
//4MB flash partition
#elif (IFX_CONFIG_FLASH_SIZE == 4)
#define IFX_CFG_FLASH_PARTITIONS_INFO                                   \
        "part0_begin=0x00000000\0"                                      \
        "part1_begin=0x00010000\0"                                      \
        "part2_begin=0x00040000\0"                                      \
        "total_part=3\0"

#ifdef IFX_CONFIG_WIRELESS_IFX1350 
#define IFX_CFG_FLASH_DATA_BLOCKS_INFO                                  \
        "data_block0=" IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME "\0"        \
        "data_block1=" IFX_CFG_FLASH_FIRMWARE_IMAGE_BLOCK_NAME "\0"     \
        "data_block2=" IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME "\0"       \
        "data_block3=" IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME "\0"       \
        "data_block4=" IFX_CFG_FLASH_VOIP_CFG_BLOCK_NAME "\0"	\
        "data_block5=" IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME "\0"         \
	"data_block6=" IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME "\0"          \
        "data_block7=" IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME "\0"	\
	"total_db=8\0"
#else
#define IFX_CFG_FLASH_DATA_BLOCKS_INFO                                  \
        "data_block0=" IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME "\0"        \
        "data_block1=" IFX_CFG_FLASH_FIRMWARE_IMAGE_BLOCK_NAME "\0"     \
        "data_block2=" IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME "\0"       \
        "data_block3=" IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME "\0"       \
        "data_block4=" IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME "\0"         \
	"data_block5=" IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME "\0"          \
        "data_block6=" IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME "\0"	\
	"total_db=7\0"
#endif

	#define IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME		"uboot"
	#define IFX_CFG_FLASH_UBOOT_IMAGE_START_ADDR		0x00000000
	#define IFX_CFG_FLASH_UBOOT_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_UBOOT_IMAGE_MTDBLOCK_NAME		"/dev/mtdblock0"

	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_BLOCK_NAME		"firmware"
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_START_ADDR         0x00010000
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_SIZE		0
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock1"

	#define IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME		"rootfs"
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_START_ADDR           0x00040000
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock2"

	#define IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME		"kernel"
	//#define IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR		0x00270000
	#define IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR         0x00340000
	#define IFX_CFG_FLASH_KERNEL_IMAGE_SIZE			0

#ifdef IFX_CONFIG_WIRELESS_IFX1350 
	#define IFX_CFG_FLASH_VOIP_CFG_BLOCK_NAME               "voip"
	#define IFX_CFG_FLASH_VOIP_CFG_START_ADDR               0x003DD200
	#define IFX_CFG_FLASH_VOIP_CFG_SIZE                     0
	#define IFX_CFG_FLASH_VOIP_CFG_END_ADDR                 0x003FD1FF
#endif
	#define IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME		"sysconfig"
	#define IFX_CFG_FLASH_SYSTEM_CFG_START_ADDR		0x003FD200
	#define IFX_CFG_FLASH_SYSTEM_CFG_SIZE			0
	#define IFX_CFG_FLASH_SYSTEM_CFG_END_ADDR		0x003FF1FF

	#define IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME		"ubootconfig"
	#define IFX_CFG_FLASH_UBOOT_CFG_START_ADDR		0x003FF200
	#define IFX_CFG_FLASH_UBOOT_CFG_SIZE			0x0C00
	#define IFX_CFG_FLASH_UBOOT_CFG_END_ADDR		0x003FFDFF

	#define IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME		"fwdiag"
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_START_ADDR		0x003FFE00
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_SIZE		0x0200
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_END_ADDR		0x003FFFFF

	#define IFX_CFG_FLASH_END_ADDR				0x003FFFFF
#elif (IFX_CONFIG_FLASH_SIZE == 512)
#ifndef CONFIG_SPI
#define IFX_CFG_FLASH_PARTITIONS_INFO                                   \
        "part0_begin=0xB0000000\0"                                      \
        "part1_begin=0xB0010000\0"                                      \
        "part2_begin=0xB0040000\0"                                      \
        "total_part=3\0"

#define IFX_CFG_FLASH_DATA_BLOCKS_INFO                                  \
        "data_block0=" IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME "\0"        \
        "data_block1=" IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME "\0"     \
	"data_block2=" IFX_CFG_FLASH_APPL_CFG_BLOCK_NAME "\0"          \
	"data_block3=" IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME "\0"          \
	"total_db=3\0"

	#define IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME		"uboot"
	#define IFX_CFG_FLASH_UBOOT_IMAGE_START_ADDR		0xB0000000
	#define IFX_CFG_FLASH_UBOOT_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_UBOOT_IMAGE_END_ADDR		0xB000FFFF

	#define IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME		"eCos firmware"
	#define IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR		0xB0010000
	#define IFX_CFG_FLASH_KERNEL_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_KERNEL_IMAGE_END_ADDR		0xB007DFFF

	#define IFX_CFG_FLASH_APPL_CFG_BLOCK_NAME		"Application Config Structure"
	#define IFX_CFG_FLASH_APPL_CFG_START_ADDR		0xB007E000
	#define IFX_CFG_FLASH_APPL_CFG_SIZE			0x1000
	#define IFX_CFG_FLASH_APPL_CFG_END_ADDR			0xB007EFFF

	#define IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME		"ubootconfig"
	#define IFX_CFG_FLASH_UBOOT_CFG_START_ADDR		0xB007F000 
	#define IFX_CFG_FLASH_UBOOT_CFG_SIZE                    0x1000
	#define IFX_CFG_FLASH_UBOOT_CFG_END_ADDR		0xB007FFFF

	#define IFX_CFG_FLASH_END_ADDR				0xB007FFFF
/* end tatha */
#endif //CONFIG_SPI
#ifdef CONFIG_SPI
#define IFX_CFG_FLASH_PARTITIONS_INFO                                   \
        "part0_begin=0x00000000\0"                                      \
        "part1_begin=0x00010000\0"                                      \
        "part2_begin=0x00040000\0"                                      \
        "total_part=3\0"

#define IFX_CFG_FLASH_DATA_BLOCKS_INFO                                  \
        "data_block0=" IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME "\0"        \
        "data_block1=" IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME "\0"     \
	"data_block2=" IFX_CFG_FLASH_APPL_CFG_BLOCK_NAME "\0"          \
	"data_block3=" IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME "\0"          \
	"total_db=3\0"

	#define IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME		"uboot"
	#define IFX_CFG_FLASH_UBOOT_IMAGE_START_ADDR		0x00000000
	#define IFX_CFG_FLASH_UBOOT_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_UBOOT_IMAGE_END_ADDR		0x0000FFFF

	#define IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME		"eCos firmware"
	#define IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR		0x00010000
	#define IFX_CFG_FLASH_KERNEL_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_KERNEL_IMAGE_END_ADDR		0x0007DFFF

	#define IFX_CFG_FLASH_APPL_CFG_BLOCK_NAME		"Application Config Structure"
	#define IFX_CFG_FLASH_APPL_CFG_START_ADDR		0x0007E000
	#define IFX_CFG_FLASH_APPL_CFG_SIZE			0x1000
	#define IFX_CFG_FLASH_APPL_CFG_END_ADDR			0x0007EFFF

	#define IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME		"ubootconfig"
	#define IFX_CFG_FLASH_UBOOT_CFG_START_ADDR		0x0007F000 
	//#define IFX_CFG_FLASH_UBOOT_CFG_SIZE                    0x1000
	#define IFX_CFG_FLASH_UBOOT_CFG_SIZE                    0x0c00
	#define IFX_CFG_FLASH_UBOOT_CFG_END_ADDR		0x0007FFFF

	#define IFX_CFG_FLASH_END_ADDR				0x0007FFFF
/* end tatha */
#endif //CONFIG_SPI
#else
	#error "ERROR!! Define flash size first!"
#endif


#else //IFX_CONFIG_FLASH_TYPE_SPI
//2MB flash partition
#if (IFX_CONFIG_FLASH_SIZE == 2)
#if (IFX_ADSL_FIRMWARE_IN_ROOTFS == 1)
#define IFX_CFG_FLASH_PARTITIONS_INFO                                   \
        "part0_begin=0xB0000000\0"                                      \
        "part1_begin=0xB0010000\0"                                      \
        "total_part=2\0"

#define IFX_CFG_FLASH_DATA_BLOCKS_INFO					\
	"data_block0=" IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME "\0"	\
	"data_block1=" IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME "\0"	\
	"data_block2=" IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME "\0"	\
	"data_block3=" IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME "\0"		\
	"data_block4=" IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME "\0"		\
	"data_block5=" IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME "\0"	\
	"total_db=6\0"

	#define IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME		"uboot"
	#define IFX_CFG_FLASH_UBOOT_IMAGE_START_ADDR		0xB0000000
	#define IFX_CFG_FLASH_UBOOT_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_UBOOT_IMAGE_MTDBLOCK_NAME		"/dev/mtdblock0"

	#define IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME		"rootfs"
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_START_ADDR           0xB0010000
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock1"

	#define IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME		"kernel"
	#define IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR		0xB01FD1FF
	#define IFX_CFG_FLASH_KERNEL_IMAGE_SIZE			0

	#define IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME		"sysconfig"
	#define IFX_CFG_FLASH_SYSTEM_CFG_START_ADDR		0xB01FD200
	#define IFX_CFG_FLASH_SYSTEM_CFG_SIZE			0
	#define IFX_CFG_FLASH_SYSTEM_CFG_END_ADDR		0xB01FF1FF

	#define IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME		"ubootconfig"
	#define IFX_CFG_FLASH_UBOOT_CFG_START_ADDR		0xB01FF200
	#define IFX_CFG_FLASH_UBOOT_CFG_SIZE                    0x0C00
	#define IFX_CFG_FLASH_UBOOT_CFG_END_ADDR		0xB01FFDFF

	#define IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME		"fwdiag"
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_START_ADDR		0xB01FFE00
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_SIZE		0x0200
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_END_ADDR		0xB01FFFFF

	#define IFX_CFG_FLASH_END_ADDR				0xB01FFFFF
#else
/* ADSL Firmware exits as a seperate MTD partition */
#define IFX_CFG_FLASH_PARTITIONS_INFO                                   \
        "part0_begin=0xB0000000\0"                                      \
        "part1_begin=0xB0010000\0"                                      \
        "part2_begin=0xB0040000\0"                                      \
        "total_part=3\0"

#define IFX_CFG_FLASH_DATA_BLOCKS_INFO					\
	"data_block0=" IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME "\0"	\
	"data_block1=" IFX_CFG_FLASH_FIRMWARE_IMAGE_BLOCK_NAME "\0"	\
	"data_block2=" IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME "\0"	\
	"data_block3=" IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME "\0"	\
	"data_block4=" IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME "\0"		\
	"data_block5=" IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME "\0"		\
	"data_block6=" IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME "\0"	\
	"total_db=7\0"

	#define IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME		"uboot"
	#define IFX_CFG_FLASH_UBOOT_IMAGE_START_ADDR		0xB0000000
	#define IFX_CFG_FLASH_UBOOT_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_UBOOT_IMAGE_MTDBLOCK_NAME		"/dev/mtdblock0"

	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_BLOCK_NAME		"firmware"
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_START_ADDR         0xB0010000
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_SIZE		0
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock1"

	#define IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME		"rootfs"
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_START_ADDR           0xB0040000
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock2"

	#define IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME		"kernel"
	#define IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR		0xB01FD1FF
	#define IFX_CFG_FLASH_KERNEL_IMAGE_SIZE			0

	#define IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME		"sysconfig"
	#define IFX_CFG_FLASH_SYSTEM_CFG_START_ADDR		0xB01FD200
	#define IFX_CFG_FLASH_SYSTEM_CFG_SIZE			0
	#define IFX_CFG_FLASH_SYSTEM_CFG_END_ADDR		0xB01FF1FF

	#define IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME		"ubootconfig"
	#define IFX_CFG_FLASH_UBOOT_CFG_START_ADDR		0xB01FF200
	#define IFX_CFG_FLASH_UBOOT_CFG_SIZE                    0x0C00
	#define IFX_CFG_FLASH_UBOOT_CFG_END_ADDR		0xB01FFDFF

	#define IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME		"fwdiag"
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_START_ADDR		0xB01FFE00
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_SIZE		0x0200
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_END_ADDR		0xB01FFFFF

	#define IFX_CFG_FLASH_END_ADDR				0xB01FFFFF

#endif // IFX_ADSL_FIRMWARE_IN_ROOTFS 

//4MB flash partition
#elif (IFX_CONFIG_FLASH_SIZE == 4)
#if (IFX_ADSL_FIRMWARE_IN_ROOTFS == 1)

#define IFX_CFG_FLASH_PARTITIONS_INFO                                   \
        "part0_begin=0xB0000000\0"                                      \
        "part1_begin=0xB0010000\0"                                      \
        "total_part=2\0"

#define IFX_CFG_FLASH_DATA_BLOCKS_INFO                                  \
        "data_block0=" IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME "\0"        \
        "data_block1=" IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME "\0"       \
        "data_block2=" IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME "\0"       \
        "data_block3=" IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME "\0"         \
	"data_block4=" IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME "\0"          \
        "data_block5=" IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME "\0"	\
	"total_db=6\0"

	#define IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME		"uboot"
	#define IFX_CFG_FLASH_UBOOT_IMAGE_START_ADDR		0xB0000000
	#define IFX_CFG_FLASH_UBOOT_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_UBOOT_IMAGE_MTDBLOCK_NAME		"/dev/mtdblock0"

	#define IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME		"rootfs"
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_START_ADDR           0xB0010000
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock1"

	#define IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME		"kernel"
	#define IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR		0xB03FB1FF
	#define IFX_CFG_FLASH_KERNEL_IMAGE_SIZE			0

	#define IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME		"sysconfig"
	#define IFX_CFG_FLASH_SYSTEM_CFG_START_ADDR		0xB03FB200
	#define IFX_CFG_FLASH_SYSTEM_CFG_SIZE			0
	#define IFX_CFG_FLASH_SYSTEM_CFG_END_ADDR		0xB03FF1FF

	#define IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME		"ubootconfig"
	#define IFX_CFG_FLASH_UBOOT_CFG_START_ADDR		0xB03FF200
	#define IFX_CFG_FLASH_UBOOT_CFG_SIZE			0x0C00
	#define IFX_CFG_FLASH_UBOOT_CFG_END_ADDR		0xB03FFDFF

	#define IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME		"fwdiag"
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_START_ADDR		0xB03FFE00
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_SIZE		0x0200
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_END_ADDR		0xB03FFFFF

	#define IFX_CFG_FLASH_END_ADDR				0xB03FFFFF
#else
/* ADSL Firmware exits as a seperate MTD partition */
#define IFX_CFG_FLASH_PARTITIONS_INFO                                   \
        "part0_begin=0xB0000000\0"                                      \
        "part1_begin=0xB0010000\0"                                      \
        "part2_begin=0xB0040000\0"                                      \
        "total_part=3\0"

#define IFX_CFG_FLASH_DATA_BLOCKS_INFO                                  \
        "data_block0=" IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME "\0"        \
        "data_block1=" IFX_CFG_FLASH_FIRMWARE_IMAGE_BLOCK_NAME "\0"     \
        "data_block2=" IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME "\0"       \
        "data_block3=" IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME "\0"       \
	"data_block4=" IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME "\0"         \
	"data_block5=" IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME "\0"          \
        "data_block6=" IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME "\0"	\
	"total_db=7\0"

	#define IFX_CFG_FLASH_UBOOT_IMAGE_BLOCK_NAME		"uboot"
	#define IFX_CFG_FLASH_UBOOT_IMAGE_START_ADDR		0xB0000000
	#define IFX_CFG_FLASH_UBOOT_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_UBOOT_IMAGE_MTDBLOCK_NAME		"/dev/mtdblock0"

	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_BLOCK_NAME		"firmware"
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_START_ADDR         0xB0010000
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_SIZE		0
	#define IFX_CFG_FLASH_FIRMWARE_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock1"

	#define IFX_CFG_FLASH_ROOTFS_IMAGE_BLOCK_NAME		"rootfs"
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_START_ADDR           0xB0040000
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_SIZE			0
	#define IFX_CFG_FLASH_ROOTFS_IMAGE_MTDBLOCK_NAME	"/dev/mtdblock2"

	#define IFX_CFG_FLASH_KERNEL_IMAGE_BLOCK_NAME		"kernel"
	#define IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR		0xB03FB1FF
	#define IFX_CFG_FLASH_KERNEL_IMAGE_SIZE			0


	#define IFX_CFG_FLASH_SYSTEM_CFG_BLOCK_NAME		"sysconfig"
	#define IFX_CFG_FLASH_SYSTEM_CFG_START_ADDR		0xB03FB200
	#define IFX_CFG_FLASH_SYSTEM_CFG_SIZE			0
	#define IFX_CFG_FLASH_SYSTEM_CFG_END_ADDR		0xB03FF1FF

	#define IFX_CFG_FLASH_UBOOT_CFG_BLOCK_NAME		"ubootconfig"
	#define IFX_CFG_FLASH_UBOOT_CFG_START_ADDR		0xB03FF200
	#define IFX_CFG_FLASH_UBOOT_CFG_SIZE			0x0C00
	#define IFX_CFG_FLASH_UBOOT_CFG_END_ADDR		0xB03FFDFF

	#define IFX_CFG_FLASH_FIRMWARE_DIAG_BLOCK_NAME		"fwdiag"
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_START_ADDR		0xB03FFE00
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_SIZE		0x0200
	#define IFX_CFG_FLASH_FIRMWARE_DIAG_END_ADDR		0xB03FFFFF

	#define IFX_CFG_FLASH_END_ADDR				0xB03FFFFF

#endif	// IFX_ADSL_FIRMWARE_IN_ROOTFS 

#else
	#error "ERROR!! Define flash size first!"
#endif

#endif /* IFX_CONFIG_FLASH_TYPE_SPI */
/* End of Board specific configurations
 *-----------------------------------------------------------------------
 */

#endif
