#include <config.h>
#include <asm/lib.h>
#include <asm/addrspace.h>
#include <asm/ifx_mips.h>

#define ppe_mem_acc(addr,data) *((volatile u32*)(addr)) = data
#define fuse_addr  ((volatile u32*)(AMAZON_SE_MPS + 0x0354))
#define cdm_0_addr_bit_mask   0xFFF
#define sb_0_addr_bit_mask    0xFFF
#define sb_1_addr_bit_mask    0xFFF
#define PPE32_BASE  0xBE180000
#define PPE32_SB_RAM_B0               PPE32_BASE + (0x8200 * 4)
#define PPE32_SB_RAM_B1               PPE32_BASE + (0x8C00 * 4)
#define PPE32_CDM_CODE_MEM_B0         PPE32_BASE + (0x1000 * 4)
#define PPE32_DATA_MEM_MAP_REG_BASE   PPE32_BASE + (0x4000 * 4)
#define PPE32_CDM_CFG                 PPE32_DATA_MEM_MAP_REG_BASE + (0x0100 * 4)

#define NO_OF_FUSES 	4

void rar_init(void)
{
  /* Set rar_val of CDM RAM block 0 to '0' */
  ppe_mem_acc(PPE32_CDM_CODE_MEM_B0, 0);
  ppe_mem_acc(PPE32_CDM_CODE_MEM_B0+4, 0);
  /* Set rar_val of SB RAM block 0 to '0' */
  ppe_mem_acc(PPE32_SB_RAM_B0, 0);
  ppe_mem_acc(PPE32_SB_RAM_B0+4, 0);
  /* Set rar_val of SB RAM block 1 to '0' */
  ppe_mem_acc(PPE32_SB_RAM_B1, 0);
  ppe_mem_acc(PPE32_SB_RAM_B1+4, 0);
};

int fuse_prg(void)
{
  u32 fuse_value;
  int i=0;
  /* fuse programming except ARC memories*/
  /* wait fuse reading is finished */
  while ((*(AMAZON_SE_RCU_RST_REQ)&0x10000000)==0){
    //add a watchdog
    i++;
    /* 0x4000 translate to  about 16 ms@111M, so should be enough */
    if (i == 0x4000)return;
  }
  // STEP a: Prepare memory for external accesses
  // Write fuse_en bit27
  *(AMAZON_SE_RCU_RST_REQ) = (*(AMAZON_SE_RCU_RST_REQ) | 0x08000000);

  // Code memory preparation
  //ppe_mem_acc(PPE32_CDM_CFG,0x4);
  ppe_mem_acc(PPE32_CDM_CFG,0x0); // cdm1 as DMEM (reset value); cdm0 as CMEM

  /* Init rar_val of CDM RAM 0 SB RAM 0, 1 */
  rar_init();

  // STEP b: Write values into the memory addresses
  //         2 RAR's per memory
  for (i=0; i<NO_OF_FUSES; i++){
    fuse_value = *(fuse_addr+i);
    switch(fuse_value&0xF0000) {
      case 0x20000:
        ppe_mem_acc((PPE32_CDM_CODE_MEM_B0), ((fuse_value & cdm_0_addr_bit_mask) | (cdm_0_addr_bit_mask + 0x1)));
        break;
      case 0x30000:
        ppe_mem_acc((PPE32_CDM_CODE_MEM_B0+0x4), ((fuse_value & cdm_0_addr_bit_mask) | (cdm_0_addr_bit_mask + 0x1)));
        break;
      case 0x40000:
        ppe_mem_acc((PPE32_SB_RAM_B0), ((fuse_value & sb_0_addr_bit_mask) | (sb_0_addr_bit_mask + 0x1)));
        break;
      case 0x50000:
        ppe_mem_acc((PPE32_SB_RAM_B0+0x4), ((fuse_value & sb_0_addr_bit_mask) | (sb_0_addr_bit_mask + 0x1)));
        break;
      case 0x60000:
        ppe_mem_acc((PPE32_SB_RAM_B1), ((fuse_value & sb_1_addr_bit_mask) | (sb_1_addr_bit_mask + 0x1)));
        break;
      case 0x70000:
        ppe_mem_acc((PPE32_SB_RAM_B1+0x4), ((fuse_value & sb_1_addr_bit_mask) | (sb_1_addr_bit_mask + 0x1)));
        break;
      default :
        // ADSL related
        break;
    };
  };

  // STEP c: Memory read/write check
  //  Disable Fuse_en (bit 27)
  *(AMAZON_SE_RCU_RST_REQ) = (*(AMAZON_SE_RCU_RST_REQ) & 0xF7FFFFFF);
  return 0;
};
