#!/bin/sh

APPS_NAME="uboot"
if [ -e sysconfig.sh ]; then
	. sysconfig.sh
	. config.sh
	. model_config.sh
else
        echo "Application "$APPS_NAME" not configured"
        exit 0
fi                                                                                                                                       
display_info "----------------------------------------------------------------------"
display_info "                      build ${IFX_PLATFORM_NAME} u-boot               "
display_info "----------------------------------------------------------------------"

parse_args $@

UBOOT_CFLAGS="-DCONFIG_IFX_MIPS -DCONFIG_LZMA -DIFX_CONFIG_MEMORY_SIZE=${IFX_CONFIG_MEMORY_SIZE} -DIFX_CONFIG_FLASH_SIZE=${IFX_CONFIG_FLASH_SIZE}"

UBOOT_EXTRA_CFLAGS=

if [ "A${IFX_CONFIG_KERNEL_MEMORY_SIZE}" != "A" ]; then
	UBOOT_EXTRA_CFLAGS="-DIFX_CONFIG_KERNEL_MEMORY_SIZE=${IFX_CONFIG_KERNEL_MEMORY_SIZE}"
fi
if [ "A${IFX_CONFIG_CPU1_SUPPORT}" != "A" -a "${IFX_CONFIG_CPU1_SUPPORT}" = "1" ] ;then
	UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_CPU1_SUPPORT"
fi
if [ "${IFX_ADSL_FIRMWARE_IN_ROOTFS}" = "1" ]; then
        UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_ADSL_FIRMWARE_IN_ROOTFS=1"
else
        UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_ADSL_FIRMWARE_IN_ROOTFS=0"
fi
if [ "A${IFX_CONFIG_ETHERNET_TYPE}" != "A" ]; then
	if [ "${IFX_CONFIG_ETHERNET_TYPE}" == "RMII" ]; then
		UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_CONFIG_ETHERNET_TYPE_RMII"
	elif [ "${IFX_CONFIG_ETHERNET_TYPE}" == "EPHY" ]; then
		UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_CONFIG_ETHERNET_TYPE_EPHY"
	fi
fi
if [ "A${IFX_CONFIG_LED_TYPE}" != "A" ]; then
	if [ "${IFX_CONFIG_LED_TYPE}" == "GPIO" ]; then
		UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_CONFIG_LED_TYPE_GPIO"
	elif [ "${IFX_CONFIG_LED_TYPE}" == "LEDC" ]; then
		UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_CONFIG_LED_TYPE_LEDC"
	fi
fi
if [ "A${IFX_CONFIG_WIRELESS_TSC}" != "A" ]; then
	if [ "${IFX_CONFIG_WIRELESS_TSC}" == "1" ]; then
		UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_CONFIG_WIRELESS_TSC"
	fi	
fi
if [ "A${IFX_CONFIG_CHINA_MKT_1}" != "A" ]; then
	UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_CONFIG_CHINA_MKT_1"
fi
if [ "A${IFX_CONFIG_FLASH_TYPE}" != "A" ]; then
	if [ "${IFX_CONFIG_FLASH_TYPE}" == "SPI" ]; then
		UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_CONFIG_FLASH_TYPE_SPI -DCONFIG_SPI -DBOOTROM_IMP -DCONFIG_SPI_IFX"
	fi
fi

if [ "A${IFX_CONFIG_WIRELESS_IFX1350}" != "A" ]; then
        UBOOT_EXTRA_CFLAGS="${UBOOT_EXTRA_CFLAGS} -DIFX_CONFIG_WIRELESS_IFX1350"
fi

UBOOT_CFLAGS="${UBOOT_CFLAGS} ${UBOOT_EXTRA_CFLAGS}"

UBOOT_COMPRESS=lzma

if [ $BUILD_CLEAN -eq 1 ]; then
        rm -f .config_ok
	CROSS_COMPILE="mips-linux-" CROSS_COMPILE_UCLIBC=1 COMPRESS=${UBOOT_COMPRESS} PLATFORM_CPU=4kc IFX_CFLAGS="${UBOOT_CFLAGS}" make HEAD_FLASH_TEXT_BASE=${IFX_HEAD_FLASH_TEXT_BASE} UBOOT_RAM_TEXT_BASE=${IFX_UBOOT_RAM_TEXT_BASE} IN_PAD_LEN=${IFX_IN_PAD_LEN} CPU_TYPE=${IFX_CONFIG_CPU} distclean
        [ $BUILD_CONFIGURE -ne 1 ] && exit 0
fi

if [ $BUILD_CONFIGURE -eq 1 -o ! -f .config_ok ] ;then
	CROSS_COMPILE="mips-linux-" CROSS_COMPILE_UCLIBC=1 COMPRESS=${UBOOT_COMPRESS} PLATFORM_CPU=4kc IFX_CFLAGS="${UBOOT_CFLAGS}" make HEAD_FLASH_TEXT_BASE=${IFX_HEAD_FLASH_TEXT_BASE} UBOOT_RAM_TEXT_BASE=${IFX_UBOOT_RAM_TEXT_BASE} IN_PAD_LEN=${IFX_IN_PAD_LEN} CPU_TYPE=${IFX_CONFIG_CPU} ifx_mips_config
        echo -n > .config_ok
fi

CROSS_COMPILE="mips-linux-" CROSS_COMPILE_UCLIBC=1 COMPRESS=${UBOOT_COMPRESS} PLATFORM_CPU=4kc IFX_CFLAGS="${UBOOT_CFLAGS}" make HEAD_FLASH_TEXT_BASE=${IFX_HEAD_FLASH_TEXT_BASE} UBOOT_RAM_TEXT_BASE=${IFX_UBOOT_RAM_TEXT_BASE} IN_PAD_LEN=${IFX_IN_PAD_LEN} CPU_TYPE=${IFX_CONFIG_CPU} ifx_all

if [ "A${IFX_CONFIG_FLASH_TYPE}" == "ASPI" ]; then
	. build-pro.sh		
	cp -f u-boot-spi.bin "$BUILD_DIR"
	if [ "$COPY_TFTP" = "y" ];then
		cp -f u-boot-spi.bin "$TFTP_DIR"
	fi
else
	cp -f u-boot.ifx "$BUILD_DIR"
	if [ "$COPY_TFTP" = "y" ];then
		cp -f u-boot.ifx "$TFTP_DIR"
	fi
fi

