/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Cache support: switch on or off, get status
 */
#include <common.h>
#include <command.h>
#include <cmd_upgrade.h>

#if defined(BUILD_FROM_IFX_UTILITIES)
#include "crc32.h"
#include <syslog.h>
#define getenv(x)		get_env(x)
#define simple_strtoul(a,b,c)	strtoul(a,b,c)
#define setenv(x,y)		set_env(x,y)
#define uchar			unsigned char
#define ulong			unsigned long
#define uint32_t		unsigned int
#define uint8_t			unsigned char
//#define printf(format, args...)	syslog(LOG_INFO, format, ##args)
#endif //BUILD_FROM_IFX_UTILITIES

#include "image.h"

#if !defined(BUILD_FROM_IFX_UTILITIES)
int do_upgrade( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]) {
	ulong srcAddr=0;
	int srcLen;
	ulong srcHeader=0;
	int curParSize=0, dir, pad;
	image_header_t *pimg_header = NULL;
	char name[16], strimg_crc[32], buf[32];;

	if(argc != 3) {
		printf("Usage :\n%s\n",cmdtp->usage);
		return 1;
	}

	srcAddr = simple_strtoul(argv[1], NULL, 16);
	srcLen = simple_strtoul(argv[2], NULL, 16);

	do {
		srcHeader = (!srcHeader ? srcAddr : srcHeader + curParSize);
		pimg_header = (image_header_t *)srcHeader;		
		curParSize = sizeof(image_header_t) + pimg_header->ih_size;
		pad = (16 - (curParSize % 16)) % 16;

		switch(pimg_header->ih_type) {
			case IH_TYPE_MULTI:
				curParSize = sizeof(image_header_t) + 8;
				continue;
			case IH_TYPE_FILESYSTEM:
				sprintf(name, "rootfs");
				dir = 0;
				break;
			case IH_TYPE_KERNEL:
				sprintf(name, "kernel");
				dir = 1;
				break;
			case IH_TYPE_FIRMWARE:
				sprintf(name, "firmware");
				dir = 0;
				break;
			default:
				printf("Unknown image type!!\n");
				continue;
		}

		if(upgrade_img(srcHeader, curParSize, name, dir, 0)) {
			printf("Can not upgrade the image %s\n", name);
		} else {
			sprintf(strimg_crc, "f_%s_crc", name);
			sprintf(buf, "%lX", ntohl(pimg_header->ih_dcrc));
			setenv(strimg_crc, buf);
			saveenv();
#ifdef UBOOT_ENV_COPY
			saveenv_copy();
#endif
		}
		curParSize = curParSize + pad;
	} while(srcLen > (srcHeader - srcAddr) + curParSize);

	return 0;
}

#else

int do_upgrade( int file_fd , int srcLen) {
	ulong srcHeader=0;
	uint32_t curParSize=0, dir, pad;
	char name[16], strimg_crc[32], buf[32];;
	image_header_t xImgHeader;
	char *xHeader = NULL;
	uint32_t iFileReadSize =0, iTotalFileReadSize = 0;
	int iRet = 0;
	int Is_Multi=0; 

	do {

		//first read only the header to figure our the image total size
		iRet = read(file_fd, (void *)&xImgHeader , (size_t)sizeof(image_header_t)); 
		if(iRet < sizeof(image_header_t)){
			printf("Failure : could read %d out of %d bytes from image\n", iRet, sizeof(image_header_t));
			return 1; //failure
		}

		printf("\nImage Header --> Data Size = %d\n Image Name = %s\n",xImgHeader.ih_size , xImgHeader.ih_name);

                if(xImgHeader.ih_type == IH_TYPE_MULTI){
			Is_Multi = 1;	
                        curParSize = sizeof(image_header_t) + 8;
                        iTotalFileReadSize += curParSize;
                        if( lseek(file_fd, iTotalFileReadSize, SEEK_SET) == -1){
                                printf("Can not move the offset for image to upgrade\n");
                                return 1;  //failure
                        }

                        continue;
                }


		curParSize = sizeof(image_header_t) + xImgHeader.ih_size;
		if (Is_Multi)
			pad = (16 - (curParSize % 16)) % 16;
		else
			pad = 0;

		// reposition the file read offset to the start of image
		if( lseek(file_fd, iTotalFileReadSize, SEEK_SET) == -1){
			printf("Can not move the offset for image to upgrade\n");
		        return 1;  //failure	
		}

		// then read the total size into a buffer to be passed to upgrade_img
		iFileReadSize = curParSize + pad;
	        xHeader = (char *)malloc(iFileReadSize);			

		iRet = read(file_fd, xHeader , (size_t)iFileReadSize);
       		if(iRet < (iFileReadSize)){
			printf("Failure : could read %d out of %d bytes from image\n", iRet, iFileReadSize);
			return 1; //failure
		}
        

		switch(xImgHeader.ih_type) {
			case IH_TYPE_MULTI:
				curParSize = sizeof(image_header_t) + 8;
				continue;
			case IH_TYPE_FILESYSTEM:
				sprintf(name, "rootfs");
				dir = 0;
				break;
			case IH_TYPE_KERNEL:
				sprintf(name, "kernel");
				dir = 1;
				break;
			case IH_TYPE_FIRMWARE:
				sprintf(name, "firmware");
				dir = 0;
				break;
			default:
				printf("Unknown image type!!\n");
				continue;
		}

		if(upgrade_img(xHeader, curParSize, name, dir, 0)) {
			printf("Can not upgrade the image %s\n", name);
		} else {
			sprintf(strimg_crc, "f_%s_crc", name);
			sprintf(buf, "%lX", ntohl(xImgHeader.ih_dcrc));
			setenv(strimg_crc, buf);
			saveenv();
#ifdef UBOOT_ENV_COPY
			saveenv_copy();
#endif
		}
		iTotalFileReadSize += iFileReadSize;

		if(xHeader != NULL){
			free(xHeader);
			xHeader = NULL;
		}

	} while(srcLen > iTotalFileReadSize );

	return 0;
}
#endif


#if !defined(BUILD_FROM_IFX_UTILITIES)
U_BOOT_CMD(                                       
	upgrade,	3,	1,	do_upgrade,
	"upgrade - forward/backward copy memory to pre-defined flash location\n",
	"upgrade <source addr> <size> \n	- upgrade image to pre-define partition.\n"
);
#endif




