#include "common.h"
#include "command.h"
#include "cmd_upgrade.h"
#include "flash.h"

#if defined(BUILD_FROM_IFX_UTILITIES)
#include "crc32.h"
#define getenv(x)					get_env(x)
#define simple_strtoul(a,b,c)				strtoul(a,b,c)
#define setenv(x,y)					set_env(x,y)
#define uchar						unsigned char
#define ulong						unsigned long
#define uint32_t					unsigned int					
#define uint8_t						unsigned char					
#endif

#include "image.h"

#ifdef DEBUG_UPGRADE_MECHANISM
#	ifdef BUILD_FROM_IFX_UTILITIES
#		define upgrade_debug_printf(...)	printf
#	else
#		define upgrade_debug_printf		debug
#	endif
#else
#	ifdef BUILD_FROM_IFX_UTILITIES
#		define upgrade_debug_printf(...)
#	else
#		define upgrade_debug_printf(...)
#	endif
#endif

int program_img(ulong srcAddr,int srcLen,ulong destAddr)
{
	upgrade_debug_printf("program_img : the srcAddr is 0x%08lx, length is %d while destAddr is 0x%08lx\n",srcAddr,srcLen,destAddr);

#ifdef CONFIG_SPI
	//amazon_se_sflash_write((char*)srcAddr,srcLen,destAddr);
	amazon_se_sflash_partial_write((char*)srcAddr,srcLen,destAddr);
	puts ("Writing to Flash... ");
#else
	flash_sect_protect(0,destAddr,destAddr + srcLen-1);
	upgrade_debug_printf("Erase Flash from 0x%08lx to 0x%08lx\n", destAddr, destAddr + srcLen-1);
	if(flash_sect_erase(destAddr,destAddr + srcLen-1,1)) {
		return 1;
	}
	puts ("Writing to Flash... ");

	if(flash_write(srcAddr,destAddr,srcLen)) {
		return 1;
	}
	upgrade_debug_printf("Image at 0x%08lx with size %d is programmed at 0x%08lx successfully\n",srcAddr,srcLen,destAddr);
	printf ("Saving Environment ...\n");
	flash_sect_protect(1,destAddr,destAddr + srcLen-1);
#endif
	return 0;
}

int FindPartBoundary(ulong img_addr,ulong *curpart_begin,ulong *nextpart_begin)
{
	char strPartName[16];
	ulong part_begin_addr[MAX_PARTITION];
	int nPartNo,i;

	nPartNo = simple_strtoul((char *)getenv("total_part"),NULL,10);
	if(nPartNo <= 0 || nPartNo >= MAX_PARTITION){
		printf("Total no. of current partitions [%d] is out of limit (0,%d)\n",MAX_PARTITION);
		return 1;
	}

	for(i = 0; i < nPartNo; i++){
		memset(strPartName,0x00,sizeof(strPartName));
		sprintf(strPartName,"part%d_begin",i);
		part_begin_addr[i] = simple_strtoul((char *)getenv(strPartName),NULL,16);
	}

	part_begin_addr[i] = simple_strtoul((char *)getenv("flash_end"),NULL,16) + 1;

	for(i = 0; i < nPartNo; i++){
		if(img_addr >= part_begin_addr[i] && img_addr < part_begin_addr[i+1]){
			*curpart_begin = part_begin_addr[i];
			*nextpart_begin = part_begin_addr[i+1];
			return 0;
		}
	}

	printf("The begining of the image to be programmed [0x%08lx] is not within current patition boundary\n",img_addr);
	return 1;
}

int FindNPImgLoc(ulong img_addr,ulong *nextStartAddr,ulong *preEndAddr)
{
	ulong Img_startAddr[MAX_DATABLOCK];
	ulong Img_size[MAX_DATABLOCK];
	ulong nDBNo;
	char strDBName[MAX_DATABLOCK][32];
	char strTemp[32];
	char *strT;
	ulong curpart_begin,nextpart_begin;
	int i;

	nDBNo = simple_strtoul((char *)getenv("total_db"),NULL,10);
	if(nDBNo <= 0 || nDBNo >= MAX_DATABLOCK){
		printf("Total no. of current data blocks [%d] is out of limit (0,%d)\n",nDBNo,MAX_PARTITION);
		return 1;
	}
	
	if(FindPartBoundary(img_addr,&curpart_begin,&nextpart_begin))
		return 1;

	upgrade_debug_printf("For the image address 0x%08lx, partition boundary is found as [0x%08lx,0x%08lx]\n",img_addr,curpart_begin,nextpart_begin);
	*nextStartAddr = nextpart_begin;
	*preEndAddr = curpart_begin;

	for(i = 0; i < nDBNo; i++){
		memset(strDBName[i],0x00,sizeof(strDBName[i]));
		memset(strTemp,0x00,sizeof(strTemp));
		sprintf(strTemp,"data_block%d",i);
		strcpy(strDBName[i],(char *)getenv(strTemp));
		if(strcmp(strDBName[i],"") == 0){
			printf("Variable %s is not set\n",strTemp);
			return 1;
		}
	}
	
	for(i = 0; i < nDBNo; i++){
		memset(strTemp,0x00,sizeof(strTemp));
		strT = NULL;
		sprintf(strTemp,"f_%s_addr",strDBName[i]);
		strT = (char *)getenv(strTemp);
		if(strT == NULL){
			printf("Variable %s is not set\n",strTemp);
			return 1;
		}
		Img_startAddr[i] = simple_strtoul((char *)strT,NULL,16);

		memset(strTemp,0x00,sizeof(strTemp));
		strT = NULL;
		sprintf(strTemp,"f_%s_size",strDBName[i]);
		strT = (char *)getenv(strTemp);
		if(strT == NULL){
			printf("Variable %s is not set\n",strTemp);
			return 1;
		}
		Img_size[i] = simple_strtoul((char *)strT,NULL,16);
	}

	for(i = 0; i < nDBNo; i++){
		if(Img_startAddr[i] > img_addr && Img_startAddr[i] < *nextStartAddr)
			*nextStartAddr = Img_startAddr[i];
		if(Img_startAddr[i] + Img_size[i] < img_addr && Img_startAddr[i] + Img_size[i] > *preEndAddr)
			*preEndAddr = Img_startAddr[i] + Img_size[i];
	}
	printf("For img_addr 0x%08lx, nextStartAddr 0x%08lx and preEndAddr 0x%08lx\n",img_addr,*nextStartAddr,*preEndAddr);
	return 0;
}

int upgrade_img(ulong srcAddr, ulong srcLen, char *imgName, enum ExpandDir dir, int bSaveEnv)
{
	ulong img_addr,img_size,img_endaddr;
	char newimg_para[32];
	char strimg_addr[32],strimg_size[32];
	ulong nextStartAddr,preEndAddr;
	image_header_t *pimg_header = NULL;
	char *srcData_Copy=NULL;

	memset(strimg_addr,0x00,sizeof(strimg_addr));
	sprintf(strimg_addr,"f_%s_addr",imgName);
	sprintf(strimg_size,"f_%s_size",imgName);
	img_addr = simple_strtoul((char *)getenv(strimg_addr),NULL,16);
	if (img_addr == 0) {
		printf("The environment variable %s not found\n",strimg_addr);
		return 1;
	}

	if (FindNPImgLoc(img_addr,&nextStartAddr,&preEndAddr))
		return 1;
	pimg_header = (image_header_t *)srcAddr;
	if (pimg_header->ih_magic == IH_MAGIC) {
		printf("Image contains header with name [%s]\n",pimg_header->ih_name);
		if(pimg_header->ih_type != IH_TYPE_KERNEL) {
			printf("This is not kernel image and so removing header\n");
			srcAddr += sizeof(*pimg_header);
			srcLen -= sizeof(*pimg_header);
		}
		img_size = simple_strtoul((char *)getenv(strimg_size),NULL,16); //509061:tc.chen
	} else if (!strcmp(imgName,"uboot")) {
		img_size = simple_strtoul((char *)getenv(strimg_size),NULL,16); //509061:tc.chen
	} else {
		struct conf_header *header;
#if !defined(BUILD_FROM_IFX_UTILITIES)
		srcData_Copy = srcAddr;
		memmove(srcData_Copy + sizeof(struct conf_header) ,(void *)srcAddr,srcLen);
#else
		srcData_Copy = malloc(srcLen + sizeof(struct conf_header));
		memcpy(srcData_Copy+sizeof(struct conf_header),srcAddr,srcLen);
#endif
		header = (struct conf_header *)srcData_Copy;
		header->size = srcLen;
#if defined(BUILD_FROM_IFX_UTILITIES)
		header->crc = 0 ^ 0xffffffffL;
#else
		header->crc = 0;
#endif
		header->crc = crc32(header->crc,srcData_Copy+sizeof(struct conf_header),srcLen);
#if defined(BUILD_FROM_IFX_UTILITIES)
		header->crc ^= 0xffffffffL;
#endif
	}	

	if (dir == FORWARD) {
		if (img_addr + srcLen > nextStartAddr) {
			printf("Cannot upgrade image %s.\n Error : From start address 0x%08lx, the new size %d is bigger than begining of next image/end of the partition 0x%08lx\n",strimg_addr,img_addr,srcLen,nextStartAddr);
			return 1;
		}
		upgrade_debug_printf("Programming for FORWARD direction\n");
	} else if (dir == BACKWARD) {
		img_endaddr = nextStartAddr - 1;
		if(img_endaddr - srcLen < preEndAddr) {
			printf("Cannot upgrade image %s.\n Error : From end address 0x%08lx, the new size %d is bigger than end of previous image/begining of the partition 0x%08lx\n",strimg_addr,img_endaddr,srcLen,preEndAddr);
			return 1;
		}

		img_addr = img_endaddr - srcLen + 1;
		img_addr = (img_addr/16)*16;

		upgrade_debug_printf("Programming for BACKWARD direction\n");
	} else {
		printf("The expansion direction [%d] is invalid\n",dir);
		return 1;
	}
	if (srcData_Copy) {
		if (program_img(srcData_Copy,srcLen+sizeof(struct conf_header),img_addr)) {
#if defined(BUILD_FROM_IFX_UTILITIES)
			free(srcData_Copy);
#endif
			return 1;
		}
	} else {
		if (program_img(srcAddr,srcLen,img_addr)) {
			return 1;
		}
	}
	
#if !defined(BUILD_FROM_IFX_UTILITIES)
	if (strcmp(strimg_addr,"f_uboot_addr") == 0) {
		ulong erase_addr1=0, erase_addr2=0;
		erase_addr1 = simple_strtoul((char *)getenv("f_firmware_addr"),NULL,16);
		erase_addr2 = simple_strtoul((char *)getenv("flash_end"),NULL,16);
#ifndef CONFIG_SPI
		flash_sect_erase(erase_addr1, erase_addr2, 1);
#endif
		do_reset (NULL, 0, 0, NULL);
		return 0;
	}
#endif
	memset(newimg_para,0x00,sizeof(newimg_para));
	sprintf(newimg_para,"0x%08lx",srcLen);
	setenv(strimg_size,newimg_para);
	upgrade_debug_printf("New variables %s = %s set\n",strimg_size,newimg_para);

	memset(newimg_para,0x00,sizeof(newimg_para));
	sprintf(newimg_para,"0x%08lx",img_addr);
	setenv(strimg_addr,newimg_para);
	upgrade_debug_printf("New variables %s = %s set\n",strimg_addr,newimg_para);

	if (strcmp(strimg_addr,"f_kernel_addr") == 0) {
		setenv("kernel_addr",newimg_para);
		upgrade_debug_printf("New variables kernel_addr = %s set\n",newimg_para);
	}

	if (strcmp(strimg_addr,"f_rootfs_addr") == 0) {
		memset(newimg_para,0x00,sizeof(newimg_para));
		sprintf(newimg_para,"0x%08lx",img_addr + srcLen);
		setenv("f_rootfs_end",newimg_para);
		upgrade_debug_printf("New variables kernel_addr = %s set\n",newimg_para);
	}

	if (bSaveEnv) {
		saveenv();
#ifdef UBOOT_ENV_COPY
		saveenv_copy();
#endif //UBOOT_ENV_COPY
	}
#if defined(BUILD_FROM_IFX_UTILITIES)
	if (srcData_Copy)
		free(srcData_Copy);
#endif
	return 0;
}

