/*
 * (C) Copyright 2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <malloc.h>
#include <devices.h>
#include <version.h>
#include <net.h>
#include <environment.h>

#ifndef CFG_BOOTSTRAP_CODE
#include "efuse.h"
#endif //CFG_BOOTSTRAP_CODE

#ifdef CFG_BOOTSTRAP_CODE
#include <asm/ifx_mips.h>
#include <configs/ifx_mips.h>
#undef CONFIG_MICROBZIP2

#ifdef CONFIG_BZIP2
#include <bzlib.h>
#endif

#ifdef CONFIG_MICROBZIP2
#include <micro_bzlib.h>
#endif

#ifdef CONFIG_LZMA
#include <LzmaWrapper.h>
#endif
#endif //CFG_BOOTSTRAP_CODE

DECLARE_GLOBAL_DATA_PTR;

#if ( ((CFG_ENV_ADDR+CFG_ENV_SIZE) < BOOTSTRAP_CFG_MONITOR_BASE) || \
	(CFG_ENV_ADDR >= (BOOTSTRAP_CFG_MONITOR_BASE + CFG_MONITOR_LEN)) ) || \
    defined(CFG_ENV_IS_IN_NVRAM) && defined(CFG_BOOTSTRAP_CODE)
#define	TOTAL_MALLOC_LEN	(CFG_MALLOC_LEN + CFG_ENV_SIZE)
#elif ( ((CFG_ENV_ADDR+CFG_ENV_SIZE) < CFG_MONITOR_BASE) || \
	(CFG_ENV_ADDR >= (CFG_MONITOR_BASE + CFG_MONITOR_LEN)) ) || \
    defined(CFG_ENV_IS_IN_NVRAM)
#define	TOTAL_MALLOC_LEN	(CFG_MALLOC_LEN + CFG_ENV_SIZE)
#else
#define	TOTAL_MALLOC_LEN	CFG_MALLOC_LEN
#endif

#ifdef CONFIG_IFX_MIPS
extern int gpio_init(void);
#endif

extern int timer_init(void);
extern int incaip_set_cpuclk(void);

#ifdef CONFIG_SPI
#ifndef BOOTROM_IMP
extern  amazon_se_spi_init(void); 
#endif
#endif

#ifdef CFG_BOOTSTRAP_CODE
extern ulong uboot_end_data_bootstrap;
extern ulong uboot_end_bootstrap;
#else //CFG_BOOTSTRAP_CODE
extern ulong uboot_end_data;
extern ulong uboot_end;
#endif //CFG_BOOTSTRAP_CODE
extern int gStat;

ulong monitor_flash_len;

const char version_string[] = U_BOOT_VERSION" (" __DATE__ " - " __TIME__ ")";

#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
static char *failed = "*** failed ***\n";
#endif

/*
 * Begin and End of memory area for malloc(), and current "brk"
 */
static ulong mem_malloc_start;
static ulong mem_malloc_end;
static ulong mem_malloc_brk;

/*
 * The Malloc area is immediately below the monitor copy in DRAM
 */
#ifdef CFG_BOOTSTRAP_CODE
static void mem_malloc_init (ulong dest_addr) {
#else //CFG_BOOTSTRAP_CODE
static void mem_malloc_init (void) {
	ulong dest_addr = CFG_MONITOR_BASE + gd->reloc_off;
#endif //CFG_BOOTSTRAP_CODE
	mem_malloc_end = dest_addr;
	mem_malloc_start = dest_addr - TOTAL_MALLOC_LEN;
	mem_malloc_brk = mem_malloc_start;

	memset ((void *) mem_malloc_start,
		0,
		mem_malloc_end - mem_malloc_start);
}

#ifdef CFG_BOOTSTRAP_CODE
void *malloc(unsigned int size) {
	if(size < (mem_malloc_end - mem_malloc_start)) {
		mem_malloc_start += size;
		debug ("malloc : size required = 0x%08lx and pointer = 0x%08lx\n",size,mem_malloc_start - size);
		return (void *)(mem_malloc_start - size);
	}
	return NULL;
}

void *realloc(void *src,unsigned int size) {
	return NULL;
}

void free(void *src) {
	return;
}
#endif //CFG_BOOTSTRAP_CODE

void *sbrk (ptrdiff_t increment)
{
	ulong old = mem_malloc_brk;
	ulong new = old + increment;

	if ((new < mem_malloc_start) || (new > mem_malloc_end)) {
		return (NULL);
	}
	mem_malloc_brk = new;
	return ((void *) old);
}


static int init_func_ram (void)
{
#ifdef	CONFIG_BOARD_TYPES
	int board_type = gd->board_type;
#else
	int board_type = 0;	/* use dummy arg */
#endif

#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
#ifdef CONFIG_USE_DDR_RAM
	puts ("DDR-DRAM:  ");
#else
	puts ("DRAM:  ");
#endif
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
	if ((gd->ram_size = initdram (board_type)) > 0) {
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
		print_size (gd->ram_size, "\n");
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
		return (0);
	}
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
	puts (failed);
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
	return (1);
}

#if !defined(CFG_BOOTSTRAP_CODE) || defined(DEBUG_ENABLE_BOOTSTRAP_PRINTF)
static int display_banner(void)
{
	printf ("\n\n%s\n\n", version_string);
	return (0);
}
#endif

#ifndef CFG_BOOTSTRAP_CODE
static void display_flash_config(ulong size)
{
	puts ("Flash: ");
	print_size (size, "\n");
}
#endif //CFG_BOOTSTRAP_CODE

#if !defined(CFG_BOOTSTRAP_CODE) || defined(DEBUG_ENABLE_BOOTSTRAP_PRINTF)
static int init_baudrate (void)
{
#ifndef CFG_BOOTSTRAP_CODE
	char tmp[64];	/* long enough for environment variables */
	int i = getenv_r ("baudrate", tmp, sizeof (tmp));

	gd->baudrate = (i > 0)
			? (int) simple_strtoul (tmp, NULL, 10)
			: CONFIG_BAUDRATE;
#else //CFG_BOOTSTRAP_CODE
	gd->baudrate = CONFIG_BAUDRATE; 
#endif //CFG_BOOTSTRAP_CODE
	return (0);
}
#endif

/*
 * Breath some life into the board...
 *
 * The first part of initialization is running from Flash memory;
 * its main purpose is to initialize the RAM so that we
 * can relocate the monitor code to RAM.
 */

/*
 * All attempts to come up with a "common" initialization sequence
 * that works for all boards and architectures failed: some of the
 * requirements are just _too_ different. To get rid of the resulting
 * mess of board dependend #ifdef'ed code we now make the whole
 * initialization sequence configurable to the user.
 *
 * The requirements for any new initalization function is simple: it
 * receives a pointer to the "global data" structure as it's only
 * argument, and returns an integer return code, where 0 means
 * "continue" and != 0 means "fatal error, hang the system".
 */
typedef int (init_fnc_t) (void);

init_fnc_t *init_sequence[] = {
#ifdef CFG_BOOTSTRAP_CODE
	//fuse_prg,
	//timer_init,
	//env_init,		/* initialize environment */
#ifdef CONFIG_INCA_IP
	incaip_set_cpuclk,	/* set cpu clock according to environment variable */
#endif
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
	init_baudrate,		/* initialze baudrate settings */
	serial_init,		/* serial communications setup */
	console_init_f,
	display_banner,		/* say that we are here */
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
	//checkboard,
	init_func_ram,
	NULL,
#else //CFG_BOOTSTRAP_CODE
	fuse_prg,
#ifdef CONFIG_IFX_MIPS
	gpio_init,
#endif
	timer_init,
	env_init,		/* initialize environment */
#ifdef CONFIG_INCA_IP
	incaip_set_cpuclk,	/* set cpu clock according to environment variable */
#endif
	init_baudrate,		/* initialze baudrate settings */
	serial_init,		/* serial communications setup */
	console_init_f,
	display_banner,		/* say that we are here */
	checkboard,
	init_func_ram,
	NULL,
#endif //CFG_BOOTSTRAP_CODE
};

#ifdef CFG_BOOTSTRAP_CODE
extern void bootstrap_relocate_code(ulong addr_sp, gd_t *id, ulong addr);

void bootstrap_board_init_f(ulong bootflag)
#else
void board_init_f(ulong bootflag)
#endif
{
	gd_t gd_data, *id;
	bd_t *bd;
	init_fnc_t **init_fnc_ptr;
#ifdef CFG_BOOTSTRAP_CODE
	ulong addr, addr_sp, len = (ulong)&uboot_end_bootstrap - BOOTSTRAP_CFG_MONITOR_BASE;
	ulong lzmaImageaddr = 0;
#else //CFG_BOOTSTRAP_CODE
	ulong addr, addr_sp, len = CFG_MONITOR_LEN;
#endif //CFG_BOOTSTRAP_CODE
	ulong *s;
#ifdef CONFIG_PURPLE
	void copy_code (ulong);
#endif

	/* Pointer is writable since we allocated a register for it.
	 */
	gd = &gd_data;
	/* compiler optimization barrier needed for GCC >= 3.4 */
	__asm__ __volatile__("": : :"memory");

	memset ((void *)gd, 0, sizeof (gd_t));

	for (init_fnc_ptr = init_sequence; *init_fnc_ptr; ++init_fnc_ptr) {
		if ((*init_fnc_ptr)() != 0) {
			hang ();
		}
	}

	/*
	 * Now that we have DRAM mapped and working, we can
	 * relocate the code and continue running from DRAM.
	 */
	addr = CFG_SDRAM_BASE + gd->ram_size;

	/* We can reserve some RAM "on top" here.
	 */

	/* round down to next 4 kB limit.
	 */
	addr &= ~(4096 - 1);
	debug ("Top of RAM usable for U-Boot at: %08lx\n", addr);

	/* Reserve memory for U-Boot code, data & bss
	 * round down to next 16 kB limit
	 */
	addr -= len;
	addr &= ~(16 * 1024 - 1);

	debug ("Reserving %d Bytes for U-Boot at: %08lx\n", len, addr);

	 /* Reserve memory for malloc() arena.
	 */
	addr_sp = addr - TOTAL_MALLOC_LEN;
	debug ("Reserving %d Bytes for malloc() at: %08lx\n", TOTAL_MALLOC_LEN, addr_sp);

	/*
	 * (permanently) allocate a Board Info struct
	 * and a permanent copy of the "global" data
	 */
	addr_sp -= sizeof(bd_t);
	bd = (bd_t *)addr_sp;
	gd->bd = bd;
	debug ("Reserving %d Bytes for Board Info at: %08lx\n",	sizeof(bd_t), addr_sp);

	addr_sp -= sizeof(gd_t);
	id = (gd_t *)addr_sp;
	debug ("Reserving %d Bytes for Global Data at: %08lx\n", sizeof (gd_t), addr_sp);

 	/* Reserve memory for boot params.
	 */
	addr_sp -= CFG_BOOTPARAMS_LEN;
	bd->bi_boot_params = addr_sp;
	debug ("Reserving %dk for boot params() at: %08lx\n", CFG_BOOTPARAMS_LEN >> 10, addr_sp);

	/*
	 * Finally, we set up a new (bigger) stack.
	 *
	 * Leave some safety gap for SP, force alignment on 16 byte boundary
	 * Clear initial stack frame
	 */
	addr_sp -= 16;
	addr_sp &= ~0xF;
	s = (ulong *)addr_sp;
	*s-- = 0;
	*s-- = 0;
	addr_sp = (ulong)s;
	debug ("Stack Pointer at: %08lx\n", addr_sp);

	/*
	 * Save local variables to board info struct
	 */
	bd->bi_memstart	= CFG_SDRAM_BASE;	/* start of  DRAM memory */
	bd->bi_memsize	= gd->ram_size;		/* size  of  DRAM memory in bytes */
	bd->bi_baudrate	= gd->baudrate;		/* Console Baudrate */

	memcpy (id, (void *)gd, sizeof (gd_t));

	/* On the purple board we copy the code in a special way
	 * in order to solve flash problems
	 */
#ifdef CONFIG_PURPLE
	copy_code(addr);
#endif

#ifdef CFG_BOOTSTRAP_CODE
	lzmaImageaddr = (ulong)&uboot_end_data_bootstrap;
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
	puts("\n HEAD: relocate_code start");
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
	bootstrap_relocate_code (addr_sp, id, addr);
#else //CFG_BOOTSTRAP_CODE
	puts("\n relocate_code start");
	relocate_code (addr_sp, id, addr);
#endif //CFG_BOOTSTRAP_CODE

	/* NOTREACHED - relocate_code() does not return */
}
/************************************************************************
 *
 * This is the next part if the initialization sequence: we are now
 * running from RAM and have a "normal" C environment, i. e. global
 * data can be written, BSS has been cleared, the stack size in not
 * that critical any more, etc.
 *
 ************************************************************************
 */
#ifdef CFG_BOOTSTRAP_CODE

#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
extern void print_image_hdr (image_header_t *hdr);
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
//extern void jump_unconditional (ulong addr);

void bootstrap_board_init_r (gd_t *id, ulong dest_addr) {
	int i;
	ulong	addr;
	ulong	data, len, checksum;
	ulong  *len_ptr;
	image_header_t header;
	image_header_t *hdr = &header;
	unsigned int destLen;
	int (*fn)();

#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
	puts("\n HEAD: relocate_code finish.\n");
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF

	/* initialize malloc() area */
	mem_malloc_init(dest_addr);

	addr = (char *)(BOOTSTRAP_CFG_MONITOR_BASE + ((ulong)&uboot_end_data_bootstrap - dest_addr));
	memmove (&header, (char *)addr, sizeof(image_header_t));

	if (ntohl(hdr->ih_magic) != IH_MAGIC) {
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
		printf ("Bad Magic Number at address 0x%08lx\n",addr);
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
		return;
	}

	data = (ulong)&header;
	len  = sizeof(image_header_t);

	checksum = ntohl(hdr->ih_hcrc);
	hdr->ih_hcrc = 0;
	if (crc32 (0, (char *)data, len) != checksum) {
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
		printf ("Bad Header Checksum\n");
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
		return;
	}

#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
	print_image_hdr (hdr);
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF

	data = addr + sizeof(image_header_t);
	len  = ntohl(hdr->ih_size);
	len_ptr = (ulong *)data;

#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
	debug ("Disabling all the interrupts\n");
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
	disable_interrupts();

#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
	debug ("   Uncompressing UBoot Image ... \n" );
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
	/*
	 * If we've got less than 4 MB of malloc() space,
	 * use slower decompression algorithm which requires
	 * at most 2300 KB of memory.
	 */
	destLen = 0x0;

#ifdef CONFIG_BZIP2
	i = BZ2_bzBuffToBuffDecompress ((char*)ntohl(hdr->ih_load),
					0x400000, (char *)data, len,
					CFG_MALLOC_LEN < (4096 * 1024), 0);
	if (i != BZ_OK) {
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
			printf ("BUNZIP2 ERROR %d - must RESET board to recover\n", i);
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
			return;
	}
#elif CONFIG_MICROBZIP2
	i = micro_bzBuffToBuffDecompress ((char*)ntohl(hdr->ih_load),
					&destLen, (char *)data, len,
					CFG_MALLOC_LEN < (4096 * 1024), 0);
	if (i != RETVAL_OK) {
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
		printf ("MICRO_BUNZIP2 ERROR %d - must RESET board to recover\n", i);
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
		return;
	}
#elif CONFIG_LZMA
	i = lzma_inflate ((char *)data, len, (char*)ntohl(hdr->ih_load), &destLen);
	if (i != LZMA_RESULT_OK) {
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
		printf ("LZMA ERROR %d - must RESET board to recover\n", i);
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
		return;
	}
#else
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
	printf ("NONE Compressing u-boot body!!\n");
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
	memmove ((void *)ntohl(hdr->ih_load), (uchar *)data, len);
	destLen = len;
#endif
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
	debug ("   Uncompression completed successfully with destLen %d.\n ",destLen );
	debug ("Head: Jumping to u-boot in the ram at 0x%08lx\n", CFG_MONITOR_BASE);
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF

	//jump_unconditional(CFG_MONITOR_BASE);
	fn = ntohl(hdr->ih_load);
	(*fn)();
	hang ();
}
#else //CFG_BOOTSTRAP_CODE
void board_init_r (gd_t *id, ulong dest_addr)
{
	cmd_tbl_t *cmdtp;
	ulong size;
	extern void malloc_bin_reloc (void);
#ifndef CFG_ENV_IS_NOWHERE
	extern char * env_name_spec;
#endif
	char *s, *e;
	bd_t *bd;
	int i;

	puts("\n relocate_code finish.\n");

	gd = id;
	gd->flags |= GD_FLG_RELOC;	/* tell others: relocation done */

	debug ("Now running in RAM - U-Boot at: %08lx\n", dest_addr);

	gd->reloc_off = dest_addr - CFG_MONITOR_BASE;

	monitor_flash_len = (ulong)&uboot_end_data - dest_addr;

	/*
	 * We have to relocate the command table manually
	 */
 	for (cmdtp = &__u_boot_cmd_start; cmdtp !=  &__u_boot_cmd_end; cmdtp++) {
		ulong addr;

		addr = (ulong) (cmdtp->cmd) + gd->reloc_off;

		debug ("Command \"%s\": 0x%08lx => 0x%08lx\n", cmdtp->name, (ulong) (cmdtp->cmd), addr);

		cmdtp->cmd = (int (*)(struct cmd_tbl_s *, int, int, char *[]))addr;

		addr = (ulong)(cmdtp->name) + gd->reloc_off;
		cmdtp->name = (char *)addr;

		if (cmdtp->usage) {
			addr = (ulong)(cmdtp->usage) + gd->reloc_off;
			cmdtp->usage = (char *)addr;
		}
#ifdef	CFG_LONGHELP
		if (cmdtp->help) {
			addr = (ulong)(cmdtp->help) + gd->reloc_off;
			cmdtp->help = (char *)addr;
		}
#endif
	}
	/* there are some other pointer constants we must deal with */
#ifndef CFG_ENV_IS_NOWHERE
	env_name_spec += gd->reloc_off;
#endif

	/* configure available FLASH banks */
#ifndef CONFIG_SPI
	size = flash_init();
	display_flash_config (size);
#endif

	bd = gd->bd;
	bd->bi_flashstart = CFG_FLASH_BASE;
	bd->bi_flashsize = size;
#if CFG_MONITOR_BASE == CFG_FLASH_BASE
	bd->bi_flashoffset = monitor_flash_len;	/* reserved area for U-Boot */
#else
	bd->bi_flashoffset = 0;
#endif

	/* initialize malloc() area */
	mem_malloc_init();
	malloc_bin_reloc();

#ifdef CONFIG_SPI
#ifndef BOOTROM_IMP
	amazon_se_spi_init(); 
#else
	_sflash_init ();
#endif
	printf("\namazon_se_spi_init success\n");
	env_init();
#endif

	/* relocate environment function pointers etc. */
	env_relocate();

	/* board MAC address */
	s = getenv ("ethaddr");
	for (i = 0; i < 6; ++i) {
		bd->bi_enetaddr[i] = s ? simple_strtoul (s, &e, 16) : 0;
		if (s)
			s = (*e) ? e + 1 : e;
	}

	/* IP Address */
	bd->bi_ip_addr = getenv_IPaddr("ipaddr");

#if defined(CONFIG_PCI)
	/*
	 * Do pci configuration
	 */
	pci_init();
#endif

/** leave this here (after malloc(), environment and PCI are working) **/
	/* Initialize devices */
	devices_init ();

	jumptable_init ();

	/* Initialize the console (after the relocation and devices init) */
	console_init_r ();
	bg_init();
/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/

	/* Initialize from environment */
	if ((s = getenv ("loadaddr")) != NULL) {
		load_addr = simple_strtoul (s, NULL, 16);
	}
#if (CONFIG_COMMANDS & CFG_CMD_NET)
	if ((s = getenv ("bootfile")) != NULL) {
		copy_filename (BootFile, s, sizeof (BootFile));
	}
#endif	/* CFG_CMD_NET */

#if defined(CONFIG_MISC_INIT_R)
	/* miscellaneous platform dependent initialisations */
	misc_init_r ();
#endif

#if (CONFIG_COMMANDS & CFG_CMD_NET)
#if defined(CONFIG_NET_MULTI)
	puts ("Net:   ");
#endif
	eth_initialize(gd->bd);
#endif
	aconfigInit();

	/* main_loop() can return to retry autoboot, if so just run it again. */
	for (;;) {
		main_loop ();
	}

	/* NOTREACHED - no way out of command loop except booting */
}
#endif //CFG_BOOTSTRAP_CODE

void hang (void)
{
#ifdef DEBUG_ENABLE_BOOTSTRAP_PRINTF
	puts ("### ERROR ### Please RESET the board ###\n");
#endif //DEBUG_ENABLE_BOOTSTRAP_PRINTF
	for (;;);
}
