#include <common.h>
#define CFG_ACONFIG
#ifdef CFG_ACONFIG
#include <asm/addrspace.h>
#include <malloc.h>
#include <command.h>
#include <net.h>

#include "aconfig.h"

#define ACONFIG_INTERVAL                        1               /* packet send interval in second */
#define ACONFIG_INTERVAL_LED                    (CFG_HZ/5)      /* power led blink interval */
#define ACONFIG_COUNT_BROADCAST                 3600            /* times to broadcast */
#define ACONFIG_COUNT_RESEND                    120             /* times to resend */
#define ACONFIG_COUNT_FLASHING                  240             /* flashing timeout */
#define ACONFIG_COUNT_FAILED                    5               /* times to send failed packet to inform the windows utility */
#define ACONFIG_COUNT_COMPLETED                 5               /* times to send completed packet to the windows utility */

#define UINT32_LEN                              4               /* sizeof(uint_32) */

#define aconfigTimerExpired(x) ((x) && (aconfig_ticks() > (x)))
#define aconfigTimerRunning(x) ((x) != 0)

int    gStat = STATUS_VOID;    /* status we are in */
aconfigTimer    gTimer;                 /* timer for packet resending */
aconfigInfo     gInfo;                  /* aconfig information */
aconfigHeader   gHdr;                   /* aconfig header for packet resending */
aconfigData     gData;                  /* aconfig data for packet resending */

volatile short  gId;                    /* packet ID for packet checking */
volatile short  gSeq;                   /* packet sequence for packet checking */
uint32_t        gFilesize = 0;          /* file size for file checking */

extern uchar	NetBcastAddr[ENET_ADDR_LEN];

ulong gFlashsize;

//#define CFG_KERN_ADDR IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR
#define IFX_CFG_FLASH_FIRMWARE_IMAGE_START_ADDR         0xB0018000

#define CFG_FIRMWARE_SIZE IFX_CFG_FLASH_ROOTFS_IMAGE_START_ADDR - IFX_CFG_FLASH_FIRMWARE_IMAGE_START_ADDR
#define CFG_ROOTFS_SIZE IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR - IFX_CFG_FLASH_ROOTFS_IMAGE_START_ADDR
#define CFG_KERNEL_SIZE IFX_CFG_FLASH_SYSTEM_CFG_START_ADDR - IFX_CFG_FLASH_KERNEL_IMAGE_START_ADDR


//#define CFG_AZTECH_FS_SIZE		0xB0000
#define ACONFIG_IMAGE_DOWNLOAD_ADDRESS	load_addr
#define ACONFIG_IMAGE_DOWNLOAD_SIZE	(CFG_FIRMWARE_SIZE + CFG_ROOTFS_SIZE + CFG_KERNEL_SIZE)

#define KERNADDR			KSEG1ADDR
#define KMALLOC(size, align)		malloc((size))
#define KFREE(ptr)			{ free((ptr)); (ptr)=NULL; }
#define eth_broadcast			NetBcastAddr

/* where file download to */
uint8_t         *gPtr = NULL;           /* start address we download image to */
volatile uint8_t *gOffset = NULL;       /* offset */
ulong gLedTimer;			/* timer for led control */
ulong gFlashingLedTimer;			/* Flashing timer for led control */

unsigned short gBusyLedGpio = 12;	// HW550=>Sync_LED 	Cradle=LAN_LED(RJ45)
unsigned short gErrorLedGpio = 8;	// HW550=>USB_LED	Cradle=LAN/WAN_LED(RJ45)
unsigned short gProgressLedGpio = 11;	// HW550=>3G_LED	Cradle=3G_LED

static int aconfigSend(aconfigInfo *info);

/* To solve the limitation of 32bit only tick resolution */
static uint32_t last_tick_upper = 0;
static uint32_t last_tick_lower = 0;
static uint64_t aconfig_ticks(void)
{
	uint32_t now = get_ticks();

	if (now < last_tick_lower)
		last_tick_upper++;

	last_tick_lower = now;

	return (((uint64_t)last_tick_upper << (8*sizeof(uint32_t))) | last_tick_lower); 
}

void setLedOn(unsigned short led)
{
	//RT2882_REG(RT2880_REG_PIODATA) &= ~(1 << led);
}

void setLedOff(unsigned short led)
{
	//RT2882_REG(RT2880_REG_PIODATA) |= (1 << led);
}
void ledInit(void)
{
	//RT2882_REG(RT2880_REG_PIODIR) |= ((1 << gBusyLedGpio) | (1 << gErrorLedGpio) | (1 << gProgressLedGpio)) ;
}

void setAllLedsOff(void)
{
	//setLedOff(gBusyLedGpio);
	//setLedOff(gErrorLedGpio);
	//setLedOff(gProgressLedGpio);
}

/*
 * bootloader mode led control timer
 */
static void aconfigLedTimer(void * arg)
{
	static unsigned char on = 0;
	if((get_timer(0) - gLedTimer) > ACONFIG_INTERVAL_LED)
	{
		on ? setLedOn(gBusyLedGpio) : setLedOff(gBusyLedGpio);
		gLedTimer = get_timer(0);
		on = on ? 0 : 1;
	}
}

/* led blink control when flashing the image */
static void aconfigLedBlinkWhenFlashing(void * arg)
{
	static unsigned char on = 0;
	unsigned short gpio = *(unsigned short *)arg;
	if ((get_timer(0) - gFlashingLedTimer) > ACONFIG_INTERVAL_LED)
	{
		on ? setLedOn(gpio) : setLedOff(gpio);
		gFlashingLedTimer = get_timer(0);
		on = on ? 0 : 1;
	}
}

// Fill pdata with data
// pdata must malloced and length must be 4
static int aconfigFillDataLong(uint8_t * pdata, uint32_t data) {
	if(pdata == NULL)
		return data;
	pdata[0] = (data & 0xFF000000) >> 24;
	pdata[1] = (data & 0xFF0000) >> 16;
	pdata[2] = (data & 0xFF00) >> 8;
	pdata[3] = (data & 0xFF);
	return *(int*)pdata;
}

/* timer routines */
static void aconfigTimerSet(uint64_t interval, int count)
{
        /* to send enough packets, we need to add 1 to count */
        gTimer.timer = aconfig_ticks() + interval * (uint64_t)(count+1);
        gTimer.interval = interval;
        gTimer.count = count;
}

static int aconfigTimerIntervalReached(void)
{
        if( aconfigTimerExpired(gTimer.timer - (uint64_t)(gTimer.interval) * gTimer.count) )
        {
                gTimer.count--;
                return 1;
        }
        return 0;
}

static void aconfigTimerClear(void)
{
        gTimer.count = gTimer.interval = gTimer.timer = 0;
}

static void aconfigTimerOut(void)
{
        if(gData.pdata)
                KFREE(gData.pdata);

        switch(gStat)
        {
                case STATUS_COMPLETED:
                        setLedOff(gErrorLedGpio);
                        break;
                case STATUS_CONNECTING:
                        break;
                case STATUS_FAILED:
                case STATUS_DATAING:
                case STATUS_READY:
                        printf("\nFailed!\n");
                        gStat = STATUS_TIMEOUT;
                        setLedOn(gErrorLedGpio);

                        /* FIXME: do we need to reset the devoce if failed? */
                        //aconfigUninitialization();
                        //softReset();
                        break;
                case STATUS_FLASHING:
                        /* something error when flashing? */
                        break;
                case STATUS_TIMEOUT:
                case STATUS_VOID:
                default:
                        break;
        }
}

/*
  *  interval reached, resend last packet
  */
static void aconfigIntervalOut(void)
{
        switch(gStat)
        {
                case STATUS_COMPLETED:
                        //printf("\nInforming upgrade completed: %d\n", gTimer.count);
                        break;
                case STATUS_FAILED:
                        //printf("\nInforming upgrade failed: %d\n", gTimer.count);
                        break;
                case STATUS_CONNECTING:
                case STATUS_DATAING:
                case STATUS_READY:
                        //printf("\ntime out!: %d retrying ............\n", gTimer.count);
                        break;
                case STATUS_FLASHING:
                //      printf("\nInforming upgrade flashing: %d\n", gTimer.count);
                        break;
                case STATUS_TIMEOUT:
                case STATUS_VOID:
                default:
                        break;
        }

        aconfigSend(&gInfo);
}

/*
* monitor and manage the timer
*/

void aconfigTimerMonitor(void * arg)
{
        if(aconfigTimerExpired(gTimer.timer))
        {
                aconfigTimerClear();
                aconfigTimerOut();
        }
        else if(aconfigTimerIntervalReached())
        {
                aconfigIntervalOut();
        }
}

void aconfigInitialization(void)
{
	DECLARE_GLOBAL_DATA_PTR;
	bd_t *bd = gd->bd;

	gFlashsize = bd->bi_flashsize;

	gStat = STATUS_CONNECTING;

	void ledInit();
	setAllLedsOff();

	/* Add power led blink background task */
	bg_add(aconfigLedTimer, NULL);
	bg_add(aconfigTimerMonitor, NULL);
	gLedTimer = get_timer(0);
}

void aconfigUninitialization()
{
	gStat = STATUS_VOID;
	gBusyLedGpio = gErrorLedGpio = gProgressLedGpio = 0;
       gLedTimer = 0;
	bg_remove(aconfigLedTimer);
	bg_remove(aconfigTimerMonitor);
}

static inline void CopyLongToNet(ulong *to, ulong *from)
{
	ulong l;
	l = htonl(*from);
	memcpy((void *)to, (void *)&l, sizeof(l));
}

static inline void CopyLongFromNet(ulong *to, ulong *from)
{
	ulong l;
	memcpy((void *)&l, (void *)from, sizeof(l));
	*to = ntohl(l);
}

static int aconfigSend(aconfigInfo *info)
{
	volatile uchar *pkt;
	aconfigHeader *acon_hdr;
	aconfigDataHeader *acon_data_hdr;
	uchar *acon_data;
	int data_len;
	
	pkt = NetTxPacket;
	pkt += NetSetEther(pkt, info->dest_hwaddr, PROT_ACONFIG);

	/* Aconfig header */
	acon_hdr = (aconfigHeader *)pkt;
	CopyLongToNet(&acon_hdr->magic,  &info->hdr->magic);
	acon_hdr->revision = info->hdr->revision;
	/* FIXME: handle for Little Endian - Do htonl/htons for 3 bytes */
	memcpy(acon_hdr->build,  info->hdr->build, sizeof(info->hdr->build));
	acon_hdr->encryption_mode = htons(info->hdr->encryption_mode);
	acon_hdr->id = htons(info->hdr->id);
	acon_hdr->cookie = htons(info->hdr->cookie);
	acon_hdr->operation = htons(info->hdr->operation);
	CopyLongToNet(&acon_hdr->src_ip, &info->hdr->src_ip);
	CopyLongToNet(&acon_hdr->dst_ip, &info->hdr->dst_ip);
	acon_hdr->pkt_seq = htons(info->hdr->pkt_seq);
	acon_hdr->last_pkt = htons(info->hdr->last_pkt);
	memcpy(acon_hdr->reserved, info->hdr->reserved, sizeof(info->hdr->reserved));

	/* Aconfig data header */
	acon_data_hdr = (aconfigDataHeader *)(pkt + ACONFIG_HDR_LEN);
	CopyLongToNet(&acon_data_hdr->type, &info->data->data_hdr.type);
	acon_data_hdr->coding = htons(info->data->data_hdr.coding);
	acon_data_hdr->length = htons(info->data->data_hdr.length);

	/* Aconfig Data */
	acon_data = (uchar *)(pkt + ACONFIG_HDR_LEN + ACONFIG_DHDR_LEN);
	if (info->data->data_hdr.length > 0) {
		data_len = info->data->data_hdr.length;
		memcpy(acon_data, info->data->pdata, data_len);
	} else {
		data_len = sizeof(uint32_t);
		memset(acon_data, 0, data_len);
	}
	// add the end '\0'
	data_len += sizeof(uint8_t);
	memset(acon_data, 0, sizeof(uint8_t));

	NetSendPacket(NetTxPacket, (pkt - NetTxPacket) + ACONFIG_HDR_LEN + ACONFIG_DHDR_LEN + data_len);

        return 0;
}

static int aconfigPickData(ebuf_t *buf, aconfigData *data)
{
	aconfigDataHeader *acon_data_hdr;
	uchar *acon_data;

	if(buf->eb_length < ACONFIG_DHDR_LEN)
		return -1;

	/* Aconfig data header */
	acon_data_hdr = (aconfigDataHeader *)buf->eb_data;
	CopyLongFromNet(&data->data_hdr.type, &acon_data_hdr->type);
	data->data_hdr.coding = ntohs(acon_data_hdr->coding);
	data->data_hdr.length = ntohs(acon_data_hdr->length);

	/* Data header consumed */
	buf->eb_data += ACONFIG_DHDR_LEN;
	buf->eb_length -= ACONFIG_DHDR_LEN;

	/* Data length is header matches length of received data */
	if (buf->eb_length < data->data_hdr.length)
		return -1;

	/* Aconfig Data */
	acon_data = (uchar *)buf->eb_data;
	if(data->data_hdr.length)
	{
		if(data->pdata)
			KFREE(data->pdata);
		if( (data->pdata = KMALLOC(data->data_hdr.length+1, 0)) == NULL)
			return -1;
		memcpy(data->pdata, acon_data, data->data_hdr.length);
		data->pdata[data->data_hdr.length] = '\0';
	}
	return 0;
}

static int aconfigSendWithTimeout(aconfigInfo *info, int interval, int count)
{
        /* send the packet first */
        aconfigSend(info);

        /* setup global environment for timer routines */
        if(interval!= 0 && count != 0)
        {
                memcpy(&gHdr, info->hdr, sizeof(aconfigHeader));
                memcpy(&gData.data_hdr, &info->data->data_hdr, sizeof(aconfigDataHeader));
                if(info->data->data_hdr.length)
                {
                        if(gData.pdata)
                                KFREE(gData.pdata);
                        gData.pdata = KMALLOC(info->data->data_hdr.length, 0);
                        if(gData.pdata == NULL)
                        {
                                printf("memory out!\n");
                                return -1;
                        }
                        memcpy(gData.pdata, info->data->pdata, info->data->data_hdr.length);
                }
                gInfo.hdr = &gHdr;
                gInfo.data = &gData;
                memcpy(gInfo.dest_hwaddr, info->dest_hwaddr, ENET_ADDR_LEN);

                aconfigTimerSet((uint64_t)(interval) * get_tbclk(), count);
        }
        else
        {
                /* clear existed timer */
                if(aconfigTimerRunning(gTimer.timer))
                        aconfigTimerClear();
        }

        return 0;
}

int aconfigBroadcastRequest(void)
{
        aconfigHeader hdr;
        aconfigData data;
        aconfigInfo info;

        gPtr = (uint8_t *) KERNADDR(ACONFIG_IMAGE_DOWNLOAD_ADDRESS);
        gOffset = gPtr;

        setAllLedsOff();

        /* first receives passive broadcast packet */
        if(gStat == STATUS_VOID)
                aconfigInitialization();

        if(gStat != STATUS_CONNECTING)
                gStat = STATUS_CONNECTING;

        memset(&hdr, 0, sizeof(aconfigHeader));
        hdr.magic = ACONFIG_MAGIC;
        hdr.revision = ACONFIG_REVISION;
        /* FIXME: what's the build number we should send? */
        hdr.build[2] = ACONFIG_BUILD;
        hdr.id = 1;
        hdr.pkt_seq = 1;
        hdr.last_pkt = 1;
        hdr.operation= OP_FWUP;

        memset(&data, 0, sizeof(aconfigData));
        data.data_hdr.type = TYPE_FWUPREQUEST;

        data.data_hdr.coding = CODING_HEX;
        data.data_hdr.length = UINT32_LEN;
        data.pdata = KMALLOC(data.data_hdr.length, 0);
        if(data.pdata == NULL) {
                printf("Memory error!\n");
                return -1;
        }
        memset(data.pdata, 0, UINT32_LEN);

        gId = hdr.id;
        gSeq = hdr.pkt_seq;

        info.hdr = &hdr;
        info.data = &data;
        memcpy(info.dest_hwaddr, eth_broadcast, ENET_ADDR_LEN);

        aconfigSendWithTimeout(&info, ACONFIG_INTERVAL, ACONFIG_COUNT_BROADCAST);

	return 0;
}

static int aconfigOnReceiveInform(aconfigInfo * info)
{
	aconfigData data;
	int error = 0;

	if(info->data->data_hdr.length != sizeof(uint32_t))
		error = ERROR_DATA_LENGTH;

	if(info->data->data_hdr.coding != CODING_HEX)
		error = ERROR_DATA_CODING;

	if(error == 0)
	{
		/* TODO: some more information, file size, build version ... ? */
		ACONFIG_PRINTF("received file size data length: %d\n", info->data->data_hdr.length);
		gFilesize = info->data->pdata[0]<<24 | info->data->pdata[1]<<16
				| info->data->pdata[2]<<8 | info->data->pdata[3];
		ACONFIG_PRINTF("data: %02X %02X %02X %02X\n", info->data->pdata[0], info->data->pdata[1],
		info->data->pdata[2], info->data->pdata[3]);
		ACONFIG_PRINTF("received file size: %d\n", gFilesize);

		if(gFilesize > ACONFIG_IMAGE_DOWNLOAD_SIZE)
		{
			/* file size out of the flash size */
			printf("Image file too large!\n");
			error = ERROR_IMAGE_SIZE;
		}
	}

	info->hdr->id++;

	info->hdr->last_pkt = 1;
	info->hdr->operation = OP_FWUP;
	info->hdr->pkt_seq = 1;
	memset(info->hdr->reserved, 0, sizeof(info->hdr->reserved));

	memset(&data, 0, sizeof(aconfigData));
	data.data_hdr.coding = CODING_ACK;

	data.data_hdr.length = 4;
	data.pdata = KMALLOC(data.data_hdr.length, 0);
	if(data.pdata == NULL) {
		printf("Memory error!\n");
		return -1;
	}
	aconfigFillDataLong(data.pdata, error);
	info->data = &data;

	if(error != 0) {
		gStat = STATUS_FAILED;
		data.data_hdr.type = TYPE_FWUPFAILED;
		aconfigSendWithTimeout(info, ACONFIG_INTERVAL, ACONFIG_COUNT_FAILED);
		return -1;
	}

	data.data_hdr.type = TYPE_FWUPREADY;

	gStat = STATUS_READY;
	gId = info->hdr->id;
	gSeq = info->hdr->pkt_seq;
	gPtr = (uint8_t *) KERNADDR(ACONFIG_IMAGE_DOWNLOAD_ADDRESS);
	gOffset = gPtr;

	ACONFIG_PRINTF("Sending FWUPREADY: %d to %02X:%02X:%02X:%02X:%02X:%02X\n",
	TYPE_FWUPREADY, info->dest_hwaddr[0], info->dest_hwaddr[1], info->dest_hwaddr[2], info->dest_hwaddr[3],
	info->dest_hwaddr[4], info->dest_hwaddr[5]);
	aconfigSendWithTimeout(info, ACONFIG_INTERVAL, ACONFIG_COUNT_RESEND);

	return 0;
}

static int aconfigOnReceiveData(aconfigInfo * info)
{
	aconfigData data;
	uint8_t * pdata = NULL;
	int error = 0;

	/* receive the first data */
	if(gStat == STATUS_READY)
	{
		/* check the packet ID */
		if(info->hdr->id != gId)
		{
			printf("data ID error! %d: received ID %d:\n", gId, info->hdr->id);
			error = ERROR_HDR_ID;
		}
		gStat = STATUS_DATAING;
	}
	else
	{
		/* check the packet sequence */
		if((info->hdr->pkt_seq -1) != gSeq)
		{
			printf("data SEQ error!\n");
			error = ERROR_HDR_SEQ;
		}
	}

	/* deal with the data */
	if(error == 0 && info->data->data_hdr.length)
	{
		/* write the data to specified buffer? */
		pdata = info->data->pdata;
		if(info->data->data_hdr.coding == CODING_STRING)
		{
			memcpy((uint8_t *)gOffset, pdata, info->data->data_hdr.length);
			gOffset += info->data->data_hdr.length;
		}
		else
		{
			error = ERROR_DATA_CODING;
		}
	}

	gSeq = info->hdr->pkt_seq;
	info->hdr->operation = OP_FWUP;
	memset(info->hdr->reserved, 0, sizeof(info->hdr->reserved));

	memset(&data, 0, sizeof(aconfigData));
	data.data_hdr.coding = CODING_ACK;

	data.data_hdr.length = UINT32_LEN;
	data.pdata = KMALLOC(data.data_hdr.length, 0);
	if(data.pdata == NULL) {
		printf("Memory error!\n");
		return -1;
	}
	aconfigFillDataLong(data.pdata, error);
	info->data = &data;

	if(error != 0) {
		gStat = STATUS_FAILED;
		data.data_hdr.type = TYPE_FWUPFAILED;
		aconfigSendWithTimeout(info, ACONFIG_INTERVAL, ACONFIG_COUNT_FAILED);
		return -1;
	}

	data.data_hdr.type = TYPE_FWUPDATAACK;

	/* Last data packet, download finished */
	if(info->hdr->last_pkt)
	{
		int withTag = 0;
		printf("\nlast packet receviced: %d, Checking the image................\n\n", info->hdr->last_pkt);

		printf("Flashing.....\n");
		gStat = STATUS_FLASHING;
		info->data->data_hdr.type = TYPE_FWUPFLASHING;
		aconfigSendWithTimeout(info, ACONFIG_INTERVAL, ACONFIG_COUNT_FLASHING);

		char cmd[100] = "";
		sprintf(cmd,"upgrade %x %x",  gPtr, gFilesize);
		printf("Command = [%s]\n", cmd);
		
		if (run_command (cmd, 0) < 0) {
			/* send firmware upgrade failed */
			printf("Image flash error! %d\n", error);
			//error = ERROR_IMAGE_FLASH; // changed to more detailed errors
			gStat = STATUS_FAILED;
			data.data_hdr.type = TYPE_FWUPFAILED;
			aconfigFillDataLong(data.pdata, -error);
			aconfigSendWithTimeout(info, ACONFIG_INTERVAL, ACONFIG_COUNT_FAILED);
			setLedOn(gErrorLedGpio);
			setLedOff(gProgressLedGpio);
			return -1;
		}

		printf("Flash image successfully, firmware upgrade complete!\n");
		ACONFIG_PRINTF("Sending ADTYPE_FW_UP_COMPLETED: %d\n", TYPE_FWUPCOMPLETED);

		gStat = STATUS_COMPLETED;
		data.data_hdr.type = TYPE_FWUPCOMPLETED;
		aconfigSendWithTimeout(info, ACONFIG_INTERVAL, ACONFIG_COUNT_COMPLETED);

		/* upgrade successfully */
		setAllLedsOff();
		setLedOn(gProgressLedGpio);

		return 0;
	}

	// Not last packet, send data ack
	ACONFIG_PRINTF("Sending ADTYPE_FW_UP_DATA_ACK: %d\n", TYPE_FWUPDATAACK);
	aconfigSendWithTimeout(info, ACONFIG_INTERVAL, ACONFIG_COUNT_RESEND);

        return 0;
}

// ===================================================
// Get Functions
static int aconfigGetMacAddr(aconfigInfo *info)
{
	uchar buffer[20], *tmp;
	char *end;
	uchar mac_address[ENET_ADDR_LEN];
	int i;
#if CFG_FACTORY_OVERRIDE_ETHADDR
	uchar *enet_MAC;
#endif

	if (getenv_r("ethaddr", buffer, sizeof(buffer)) < 0) {
		printf("Failed to get environment variable\n");
		return -1;
	}

	tmp = buffer;
	for (i=0; i<6; i++) {
		if (*tmp == '"')
			tmp++;
		mac_address[i] = tmp ? simple_strtoul(tmp, &end, 16) : 0;
		if (tmp)
			tmp = (*end) ? end+1 : end;
	}

        info->hdr->operation = OP_REPLY;
        info->data->data_hdr.length = ENET_ADDR_LEN;
        if(info->data->pdata)
                KFREE(info->data->pdata);
        info->data->pdata = KMALLOC(info->data->data_hdr.length, 0);
        if(info->data->pdata) {
                memcpy(info->data->pdata, mac_address, ENET_ADDR_LEN);
#if CFG_FACTORY_OVERRIDE_ETHADDR
		enet_MAC = get_factory_enet_MAC_addr();
		if (!*enet_MAC)		/* Is Unicast */
			memcpy(info->data->pdata, enet_MAC, ENET_ADDR_LEN);
		else
			printf("Invalid Ethernet MAC address in Factory partition\n");
#endif
                ACONFIG_PRINTF("Get Mac Addr: %02X:%02X:%02X:%02X:%02X:%02X\n",
                        info->data->pdata[0],
                        info->data->pdata[1],
                        info->data->pdata[2],
                        info->data->pdata[3],
                        info->data->pdata[4],
                        info->data->pdata[5]);
                aconfigSendWithTimeout(info, 0, 0);
        }
        return 0;
}

// ==============================================================
// Set Functions
static int aconfigSetMacAddr(aconfigInfo * info)
{
	uchar buffer[20], *tmp;
	char *end;
	uchar mac_address[ENET_ADDR_LEN];
	int i;
        int status = CMD_STATUS_SUCCESS;

        if(info->data->data_hdr.length != ENET_ADDR_LEN )
                status = CMD_STATUS_DATA_LENGTH_INVALID;

        if(info->data->pdata == NULL)
                status = CMD_STATUS_DATA_INVALID;

	if (info->data->pdata[0] == 0)		/* Deny multicast MAC */
                status = CMD_STATUS_DATA_INVALID;

#if CFG_FACTORY_OVERRIDE_ETHADDR
	printf("Saving in Factory partition - Not implemented.\nProceeding to save in Bootloader's environment...\n");
#endif
        if(status == CMD_STATUS_SUCCESS) {

                printf("Set Mac Addr: %02X:%02X:%02X:%02X:%02X:%02X\n",
                        info->data->pdata[0],
                        info->data->pdata[1],
                        info->data->pdata[2],
                        info->data->pdata[3],
                        info->data->pdata[4],
                        info->data->pdata[5]);

		if (getenv_r("ethaddr", buffer, sizeof(buffer)) >= 0) {
			tmp = buffer;
			for (i=0; i<6; i++) {
				if (*tmp == '"')
					tmp++;
				mac_address[i] = tmp ? simple_strtoul(tmp, &end, 16) : 0;
				if (tmp)
					tmp = (*end) ? end+1 : end;
			}

			if(!memcmp(info->data->pdata, mac_address, ENET_ADDR_LEN)) 
				status = CMD_STATUS_DATA_SAME;
		}

		if (status != CMD_STATUS_DATA_SAME) {
			sprintf(buffer, "\"%02X:%02X:%02X:%02X:%02X:%02X\"",
					info->data->pdata[0],
					info->data->pdata[1],
					info->data->pdata[2],
					info->data->pdata[3],
					info->data->pdata[4],
					info->data->pdata[5]);
			setenv("ethaddr", buffer);
			saveenv();
                }

        }
        if(info->data->pdata != NULL)
                KFREE(info->data->pdata);
        info->hdr->operation = OP_REPLY;
        info->data->data_hdr.coding = CODING_ACK;
        info->data->data_hdr.length = UINT32_LEN;
        info->data->pdata = KMALLOC(info->data->data_hdr.length, 0);
        if(info->data->pdata) {
                aconfigFillDataLong(info->data->pdata, status);
                aconfigSendWithTimeout(info, 0, 0);
        }

        return 0;
}

// GET handler
static int aconfigOnReceiveGet(ebuf_t * buf, aconfigInfo * info)
{
	aconfigData data;
	int res = 0;

	if(!buf) return -1;

	memset(&data, 0, sizeof(aconfigData));

	if(aconfigPickData(buf, &data) != 0)
	{
		printf("aconfig pick data error!\n");
		return -1;
	}

	info->data = &data;

	switch(data.data_hdr.type)
	{
	case TYPE_MACADDR:
		res = aconfigGetMacAddr(info);
		break;
	default:
		break;
	}

	/* FIXME: Dont forget to free memory allocated when pick_ac_data */
	if(data.pdata)
	{
		KFREE(data.pdata);
		data.pdata = NULL;
	}

	return res;
}

// SET handler
static int aconfigOnReceiveSet(ebuf_t * buf, aconfigInfo * info)
{
	aconfigData data;
	int res = 0;

	if(!buf) return -1;

	memset(&data, 0, sizeof(aconfigData));

	if(aconfigPickData(buf, &data) != 0)
	{
		printf("aconfig pick data error!\n");
		return -1;
	}

	info->data = &data;

	switch(data.data_hdr.type)
	{
	case TYPE_MACADDR:
		res = aconfigSetMacAddr(info);
		break;
	default:
		break;
	}

	/* FIXME: Dont forget to free memory allocated when pick_ac_data */
	if(data.pdata)
	{
		KFREE(data.pdata);
		data.pdata = NULL;
	}

	return res;
}

static int aconfigOnReceiveFwUp(ebuf_t *buf, aconfigInfo *info)
{
	aconfigData data;

	if(!buf) return -1;

	memset(&data, 0, sizeof(aconfigData));

	/* FIXME: We dont support multidata in one packet right now
	 *  So here we only pick one available data packet,
	 *  and drop the follwing
	 */
	/*
	while(aconfigPickData(buf, &data))
	{
	*/
		ACONFIG_PRINTF("aconfig pick data...\n");
		if(aconfigPickData(buf, &data) != 0)
		{
			printf("aconfig pick data error!\n");
			return -1;
		}

		info->data = &data;

		switch(data.data_hdr.type)
		{
		case TYPE_FWUPPASSIVE:
			if(gStat == STATUS_VOID || gStat == STATUS_CONNECTING)
			{
				printf("received firmware upgrade passive\n");
				aconfigBroadcastRequest();
			}
			break;

		/* remote utility OK, ignore follwing packets and send FwUpReady */
		case TYPE_FWUPREQACK:

			ACONFIG_PRINTF("received FWUPREQACK: %d\n", TYPE_FWUPREQACK);
			//AC_PRINTF("aconfig status: %d: %d\n", ac_stat, AC_STATUS_CONNECTING);
			if(gStat == STATUS_CONNECTING)
			{
				ACONFIG_PRINTF("aconfig on receive firmware upgrade inform/ack\n");
				aconfigOnReceiveInform(info);
				printf("downloading........\n");
			}
			/* Status changed, drop the following packets */
			break;

		/* inform device is ready for upgrading, sent from device, omit it */
		//case FWUPREADY:
		//      break;

		case TYPE_FWUPDATA:
			ACONFIG_PRINTF("received a FWUPDATA: %d\n", TYPE_FWUPDATA);
			//AC_PRINTF("aconfig status: %d: %d: %d\n", ac_stat, AC_STATUS_READY, AC_STATUS_DATAING);
			if(gStat == STATUS_READY || gStat == STATUS_DATAING)
			{
				ACONFIG_PRINTF("aconfig on receive firmware upgrade data\n");
				if((info->hdr->pkt_seq & 0x0F) == 0x08)
				{
					setLedOn(gProgressLedGpio);
					printf(".");
				}

				if((info->hdr->pkt_seq & 0x0F) == 0x00)
				{
					setLedOff(gProgressLedGpio);
				}

				aconfigOnReceiveData(info);
			}
			/* Status changed, drop the following packets */
			break;
		/* size for image checking */
		//case FWUPSIZE:
		//      break;

		/* unsupported type, just drop it */
		default:
			break;
		}

		/* FIXME: Dont forget to free memory allocated when pick_ac_data */
		if(data.pdata)
			KFREE(data.pdata);
	/*
	}
	*/
	return 0;
}

int aconfigOnReceive(ebuf_t *buf)
{
	aconfigHeader hdr;
	aconfigInfo info;
	aconfigHeader *acon_hdr;

	// drop all the packets when flashing
	if(gStat == STATUS_FLASHING)
		return ETH_DROP;

	memset(&info, 0, sizeof(aconfigInfo));
	/* get the sender MAC address as the dest MAC address to send */
	memcpy(info.dest_hwaddr, buf->src_addr, ENET_ADDR_LEN);

#ifdef ACONFIG_DEBUG
	ACONFIG_PRINTF("Aconfig: packet received from %02X:%02X:%02X:%02X:%02X:%02X\n",
			buf->src_addr[0], buf->src_addr[1], buf->src_addr[2], buf->src_addr[3], buf->src_addr[4], buf->src_addr[5]);
#endif

	/* Aconfig header */
	acon_hdr = (aconfigHeader *)buf->eb_data;
	CopyLongFromNet(&hdr.magic, &acon_hdr->magic);
	hdr.revision = acon_hdr->revision;
	/* FIXME: handle for Little Endian - Do htonl/htons for 3 bytes */
	memcpy(hdr.build, acon_hdr->build, sizeof(hdr.build));
	hdr.encryption_mode = ntohs(acon_hdr->encryption_mode);
	hdr.id = ntohs(acon_hdr->id);
	hdr.cookie = ntohs(acon_hdr->cookie);
	hdr.operation = ntohs(acon_hdr->operation);
	CopyLongFromNet(&hdr.src_ip, &acon_hdr->src_ip);
	CopyLongFromNet(&hdr.dst_ip, &acon_hdr->dst_ip);
	hdr.pkt_seq = ntohs(acon_hdr->pkt_seq);
	hdr.last_pkt = ntohs(acon_hdr->last_pkt);
	memcpy(hdr.reserved, acon_hdr->reserved, sizeof(hdr.reserved));

	/* Header consumed */
	buf->eb_data += ACONFIG_HDR_LEN;
	buf->eb_length -= ACONFIG_HDR_LEN;

	/* check the megic */
	if (hdr.magic != ACONFIG_MAGIC ) goto drop;	

	/* check the revision */
	if (hdr.revision != ACONFIG_REVISION) goto drop;
	/* FIXME: should we check build number? */

	ACONFIG_PRINTF("Operation: %d\n", hdr.operation);

	info.hdr = &hdr;

	switch(hdr.operation)
	{
	case OP_FWUP:
		aconfigOnReceiveFwUp(buf, &info);
		break;
#if 0
	case OP_CMD:
		aconfigOnReceiveCmd(buf, &info);
		break;
#endif
	case OP_GET:
		aconfigOnReceiveGet(buf, &info);
		break;
	case OP_SET:
		aconfigOnReceiveSet(buf, &info);
		break;
	default:
		break;
	}

	return ETH_DROP;

drop:
        printf("Invalid packet, droped!!!\n");
        return ETH_DROP;
}
#endif	/* CFG_ACONFIG */

