/* $Id: miniupnpd.c,v 1.77 2007/03/11 00:17:10 nanard Exp $ */
/* MiniUPnP project
 * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
 * (c) 2006 Thomas Bernard
 * This software is subject to the conditions detailed
 * in the LICENCE file provided within the distribution */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <sys/file.h>
#include <syslog.h>
#include <sys/time.h>
#include <time.h>
#include <signal.h>
/* for BSD's sysctl */
#include <sys/param.h>
#include <sys/sysctl.h>

/* unix sockets */
/*#define USE_MINIUPNPDCTL*/
#include "config.h"
#ifdef USE_MINIUPNPDCTL
#include <sys/un.h>
#endif

#include "upnpglobalvars.h"
#include "upnpfunc.h"
#include "upnphttp.h"
#include "upnpdescgen.h"
#include "miniupnpdpath.h"
#include "getifaddr.h"
#include "daemonize.h"
#include "upnpsoap.h"
#include "options.h"
#include "minissdp.h"

#ifdef USE_MINIUPNPDCTL
struct ctlelem {
    int socket;
    LIST_ENTRY(ctlelem) entries;
};
#endif

#define MAX_ADD_LISTEN_ADDR (4)

static volatile int quitting = 0;


/* === main === */
/* call procession of HTTP or SSDP requests */
int
main()
{

    // Initialize all the global variables
    IFX_TR64GlobalVars_Init();
    // Initialize all the sockets 
    IFX_TR64_Init();

    /* main loop */
    while(!quitting) {

	IFX_TR64_SendNotify();

       /* select open sockets (SSDP, HTTP listen, and all HTTP soap sockets) */
        FD_ZERO(&readset);
        FD_SET(sudp, &readset);
        FD_SET(shttpl, &readset);

	CountTR64HTTPConn();

        /* for debug */
        if(select(FD_SETSIZE, &readset, 0, 0, &timeout) < 0) {
            SYSLOG(LOG_ERR, "select(all): %m");
            SYSLOG(LOG_ERR, "Failed to select open sockets. EXITING");
            return 1;           /* very serious cause of error */
        }
        /* process SSDP packets */
        if(FD_ISSET(sudp, &readset)) {
            /* SYSLOG(LOG_INFO, "Received UDP Packet"); */
            ProcessSSDPRequest(sudp, listen_addr, (unsigned short)tr64_port);
        }
        /* process active HTTP connections */
        /* LIST_FOREACH is not available under linux */
	ProcessTR64HTTPActiveConn();

        /* process incoming HTTP connections */
        if(FD_ISSET(shttpl, &readset)) {
		ProcessTR64HTTPRequest(shttpl);
        }
        /* delete finished HTTP connections */
	DeleteTR64HTTPCloseConn();
    }

    return 0;
}
