#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <sys/file.h>
#include <syslog.h>
#include <sys/time.h>
#include <time.h>
#include <signal.h>
/* for BSD's sysctl */
#include <sys/param.h>
#include <sys/sysctl.h>

#include "config.h"

#include "upnpglobalvars.h"
#include "upnpfunc.h"
#include "upnphttp.h"
#include "upnpdescgen.h"
#include "miniupnpdpath.h"
#include "getifaddr.h"
#include "daemonize.h"
#include "upnpsoap.h"
#include "options.h"
#include "minissdp.h"

#define PORT (1900)
#define UPNP_MCAST_ADDR ("239.255.255.250")

#define MAX_ADD_LISTEN_ADDR (4)

static int
OpenAndConfHTTPSocket(const char *addr, unsigned short port)
{
    int s;
    int i = 1;
    struct sockaddr_in listenname;

    if((s = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
        SYSLOG(LOG_ERR, "socket(http): %m");
        return -1;
    }

    if(setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i)) < 0) {
        SYSLOG(LOG_WARNING, "setsockopt(http, SO_REUSEADDR): %m");
    }

    memset(&listenname, 0, sizeof(struct sockaddr_in));
    listenname.sin_family = AF_INET;
    listenname.sin_port = htons(port);
    listenname.sin_addr.s_addr = htonl(INADDR_ANY);

    if(bind(s, (struct sockaddr *)&listenname, sizeof(struct sockaddr_in)) < 0) {
        SYSLOG(LOG_ERR, "bind(http): %m");
        close(s);
        return -1;
    }

    if(listen(s, 6) < 0) {
        SYSLOG(LOG_ERR, "listen(http): %m");
        close(s);
        return -1;
    }

    return s;
}


/* record the startup time, for returning uptime */
static void
set_startup_time(int sysuptime)
{
    startup_time = time(NULL);
    if(sysuptime) {
        /* use system uptime instead of daemon uptime */
#if defined(__linux__)
        char buff[64];
        int uptime, fd;
        fd = open("/proc/uptime", O_RDONLY);
        if(fd < 0) {
            SYSLOG(LOG_ERR, "open(\"/proc/uptime\" : %m");
        }
        else {
            memset(buff, 0, sizeof(buff));
            read(fd, buff, sizeof(buff) - 1);
            uptime = atoi(buff);
            SYSLOG(LOG_INFO, "system uptime is %d seconds", uptime);
            close(fd);
            startup_time -= uptime;
        }
#else
        struct timeval boottime;
        size_t size = sizeof(boottime);
        int name[2] = { CTL_KERN, KERN_BOOTTIME };
        if(sysctl(name, 2, &boottime, &size, NULL, 0) < 0) {
            SYSLOG(LOG_ERR, "sysctl(\"kern.boottime\") failed");
        }
        else {
            startup_time = boottime.tv_sec;
        }
#endif
    }
}

void DeleteTR64HTTPCloseConn(){

        for(e = upnphttphead.lh_first; e != NULL;) {
            next = e->entries.le_next;

            if(e->state >= 100) {
                LIST_REMOVE(e, entries);
                Delete_upnphttp(e);
            }
            e = next;
        }
}

void ProcessTR64HTTPRequest(int shttpl){
	socklen_t clientnamelen;
	struct sockaddr_in clientname;
	clientnamelen = sizeof(struct sockaddr_in);
	shttpcl =
		accept(shttpl, (struct sockaddr *)&clientname, &clientnamelen);
	if(shttpcl < 0) {
		SYSLOG(LOG_ERR, "accept(http): %m");
	}
	else {
		struct upnphttp *tmp = 0;
		SYSLOG(LOG_INFO, "HTTP connection from %s:%d",
				inet_ntoa(clientname.sin_addr),
				ntohs(clientname.sin_port));
		/* if (fcntl(shttp, F_SETFL, O_NONBLOCK) < 0) { SYSLOG(LOG_ERR, 
		   "fcntl F_SETFL, O_NONBLOCK"); } */
		/* Create a new upnphttp object and add it to the active
		   upnphttp object list */
		tmp = New_upnphttp(shttpcl);
		if(tmp) {
			LIST_INSERT_HEAD(&upnphttphead, tmp, entries);
		}
		else {
			SYSLOG(LOG_ERR, "New_upnphttp() failed");
			close(shttpcl);
		}
	}
}


void
IFX_TR64_SendNotify(){
        /* Check if we need to send SSDP NOTIFY messages and do it if needed */
        if(gettimeofday(&timeofday, 0) < 0) {
            SYSLOG(LOG_ERR, "gettimeofday(): %m");
            timeout.tv_sec = notify_interval;
            timeout.tv_usec = 0;
        }
        else {
            /* the comparaison is not very precise but who cares ? */
            if(timeofday.tv_sec >= (lasttimeofday.tv_sec + notify_interval)) {
                SendSSDPNotifies(snotify, listen_addr, (unsigned short)tr64_port);
                memcpy(&lasttimeofday, &timeofday, sizeof(struct timeval));
                timeout.tv_sec = notify_interval;
                timeout.tv_usec = 0;
            }
            else {
                timeout.tv_sec = lasttimeofday.tv_sec + notify_interval
                    - timeofday.tv_sec;
                if(timeofday.tv_usec > lasttimeofday.tv_usec) {
                    timeout.tv_usec = 1000000 + lasttimeofday.tv_usec
                        - timeofday.tv_usec;
                    timeout.tv_sec--;
                }
                else {
                    timeout.tv_usec = lasttimeofday.tv_usec - timeofday.tv_usec;
                }
            }
        }
}

//GLOBAL VARIABLES
/*
int sudp = -1;
int shttpl = -1;
int snotify = -1;
LIST_HEAD(httplisthead, upnphttp) upnphttphead;
struct upnphttp *e = 0;
struct upnphttp *next;
struct timeval timeout, timeofday, lasttimeofday = { 0, 0 };
*/
int
IFX_TR64_Init(){
    const char *presurl = 0;
    int pid=-1;
    int openlog_option;
    int n_add_listen_addr = 0;
    const char *add_listen_addr[MAX_ADD_LISTEN_ADDR];

//VIVEK : initialize all the variables required

    if(debug_flag) {
        pid = getpid();
    }
    else {
        pid = daemonize();
    }

    openlog_option = LOG_PID | LOG_CONS;
    if(debug_flag) {
        openlog_option |= LOG_PERROR;   /* also log on stderr */
    }

    if(checkforrunning(pidfilename) < 0) {
        SYSLOG(LOG_ERR, "MiniUPnPd is already running. EXITING");
        return 1;
    }

    writepidfile(pidfilename, pid);

    set_startup_time(sysuptime);

    SYSLOG(LOG_NOTICE, "listening on %s:%d", listen_addr, tr64_port);

    /* presentation url */
    if(presurl) {
        strncpy(presentationurl, presurl, PRESENTATIONURL_MAX_LEN);
        presentationurl[PRESENTATIONURL_MAX_LEN - 1] = '\0';
    }
    else {
        snprintf(presentationurl, PRESENTATIONURL_MAX_LEN,
                 "http://%s:%d/", listen_addr, 80);
    }

    LIST_INIT(&upnphttphead);

    /* open socket for SSDP connections */
    sudp = OpenAndConfSSDPReceiveSocket(listen_addr, n_add_listen_addr,
                                        add_listen_addr);
    if(sudp < 0) {
        SYSLOG(LOG_ERR, "Failed to open socket for receiving SSDP. EXITING");
        return 1;
    }
    /* open socket for HTTP connections */
    shttpl = OpenAndConfHTTPSocket(listen_addr, tr64_port);
    if(shttpl < 0) {
        SYSLOG(LOG_ERR, "Failed to open socket for HTTP. EXITING");
        return 1;
    }
    /* open socket for sending notifications */
    snotify = OpenAndConfSSDPNotifySocket(listen_addr);
    if(snotify < 0) {
        SYSLOG(LOG_ERR, "Failed to open socket for sending SSDP notify "
               "messages. EXITING");
        return 1;
    }
    return 0;
}

int
IFX_TR64_ReInit(int iTR64Status){
    int n_add_listen_addr = 0;
    const char *add_listen_addr[MAX_ADD_LISTEN_ADDR]={0};

    if ((iTR64Status == 1) && (sudp == -1)) {
        /* open socket for SSDP connections */
        sudp = OpenAndConfSSDPReceiveSocket(listen_addr, n_add_listen_addr,
                                            add_listen_addr);
        if(sudp < 0) {
            SYSLOG(LOG_ERR, "Failed to open socket for receiving SSDP. EXITING");
            return 1;
        }
    }
    if ((iTR64Status == 1) && (shttpl == -1)) {
        /* open socket for HTTP connections */
        shttpl = OpenAndConfHTTPSocket(listen_addr, tr64_port);
        if(shttpl < 0) {
            SYSLOG(LOG_ERR, "Failed to open socket for HTTP. EXITING");
            return 1;
        }
    }

    if ((iTR64Status == 0) && (shttpl != -1)) {
        shutdown(shttpl, SHUT_RDWR);
        close(shttpl);
        shttpl = -1;
    }
    if ((iTR64Status == 0) && (sudp != -1)) {
        shutdown(sudp, SHUT_RDWR);
        close(sudp);
        sudp = -1;
    }

    return 0;
}
