/* $Id: upnpglobalvars.c,v 1.8 2007/02/26 15:45:37 nanard Exp $ */
/* MiniUPnP project
 * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
 * (c) 2006 Thomas Bernard 
 * This software is subject to the conditions detailed
 * in the LICENCE file provided within the distribution */

#include <sys/types.h>
#include <netinet/in.h>
#include <string.h>
#include <syslog.h>
#include <stdlib.h>

#include "upnpglobalvars.h"
#include "upnphttp.h"
#include "getifaddr.h"
/* network interface for internet */
const char *ext_if_name = "br0";

/* forced ip address to use for this interface
 * when NULL, getifaddr() is used */
const char *use_ext_ip_addr = NULL;

/* LAN address */
char *listen_addr = NULL;

unsigned long downstream_bitrate = 1000000;
unsigned long upstream_bitrate = 1000000;

/* startup time */
time_t startup_time = 0;

/* use system uptime */
int sysuptime = 0;

/* log packets flag */
int logpackets = 0;

const char *pidfilename = "/var/run/miniupnpd.pid";

char uuidvalue[] = "uuid:aaa00001-bfde-11d3-832c-0008a184b9ad";
char serialnumber[SERIALNUMBER_MAX_LEN] = "00000000";

char modelnumber[MODELNUMBER_MAX_LEN] = "1.0";

char manufacturer[MANUFACTURER_MAX_LEN] = "Infineon";
char description[DESCRIPTION_MAX_LEN] = "Router";
char modelname[MODELNAME_MAX_LEN] = "Amaxon_SE";
char genericurl[GENERICURL_MAX_LEN] = "http://www.infineon.com";
/* presentation url :
 * http://nnn.nnn.nnn.nnn:ppppp/  => max 30 bytes including terminating 0 */
char presentationurl[PRESENTATIONURL_MAX_LEN];

/* UPnP permission rules : */
struct upnpperm *upnppermlist = 0;
unsigned int num_upnpperm = 0;

struct httplisthead upnphttphead;

int iTR64_HttpAuthEnable = 0;
int sudp = -1;
int shttpl = -1;  //socket for listening
int shttpcl = -1; //current working socket
int snotify = -1;
//LIST_HEAD(httplisthead, upnphttp) upnphttphead;
struct upnphttp *e = 0;
struct upnphttp *next;
struct timeval timeout, timeofday, lasttimeofday = { 0, 0 };
int tr64_port=5555;
int notify_interval = 180;   /* seconds between SSDP announces */
int debug_flag = 1;
extern int ifx_mod_get_lan_mac(char *psMac);
extern int IFX_TR64_DeviceInfoGet(char *sManu, char *sDesc, char *sModNum, char *sModName, char *sSerNum, char *sGenURL);

void IFX_TR64GlobalVars_Init()
{
	char sMac[32] = {0};
	char *pUUID = NULL;
	char *pMAC = NULL;
	int i=0;

        IFX_TR64_DeviceInfoGet(manufacturer, description, modelnumber, modelname, serialnumber, genericurl);

	listen_addr = (char *)malloc(INET_ADDRSTRLEN);

        if(use_ext_ip_addr)
        {
                strncpy(listen_addr, use_ext_ip_addr, INET_ADDRSTRLEN);
        }
	else if(getifaddr(ext_if_name, listen_addr, INET_ADDRSTRLEN) < 0)
	{
		
		SYSLOG(LOG_ERR, "Failed to get ip address for interface %s",ext_if_name);
		strncpy(listen_addr, "0.0.0.0", INET_ADDRSTRLEN);
	}

#ifdef MIPSTARGET
	ifx_mod_get_lan_mac(sMac);	// what to chck :( there is no return value for this func
#else
        strcpy(sMac,"AABBCCDDEEFF");
#endif

	pUUID = uuidvalue;
	pUUID += 29;

	pMAC = sMac;

	i = 0; 
	while(i <= 11){		// there are 12 bytes in MAC address
		if(*pMAC == ':'){
			pMAC = pMAC+1;
			continue;
		}

		//since its a hex value, due the validation check
		if(((*pMAC >= '0') && (*pMAC <= '9')) 
				|| ((*pMAC >= 'A') && (*pMAC <= 'F')) 
				|| ((*pMAC >= 'a') && (*pMAC <= 'f'))){

			pUUID[i] = *pMAC;
			i++;
			pMAC = pMAC+1;
		}else{
			return; //there is some corrupt data
		}
	}	
			
	
	return;	
}
