/* $Id: upnphttp.c,v 1.43 2007/01/11 15:55:33 nanard Exp $ */
/* Project :  miniupnp
 * Website :  http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
 * Author :   Thomas Bernard
 * Copyright (c) 2005 Thomas Bernard
 * This software is subject to the conditions detailed in the
 * LICENCE file included in this distribution.
 * */
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <syslog.h>
#include "upnphttp.h"
#include "upnpdescgen.h"
#include "miniupnpdpath.h"
#include "upnpsoap.h"
#include "global.h"
#include "config.h"
#include "httpdigcalc.h"
#include "upnpglobalvars.h"
#include "IFX_TR64.h"

extern int32 IFX_Config_GetLanConfPassword(char8 *, char8 *);

struct upnphttp * 
New_upnphttp(int s)
{
	struct upnphttp * ret;
	if(s<0)
		return NULL;
	ret = (struct upnphttp *)malloc(sizeof(struct upnphttp));
	memset(ret, 0, sizeof(struct upnphttp));
	ret->socket = s;
	return ret;
}

void
CloseSocket_upnphttp(struct upnphttp * h)
{
	close(h->socket);
	h->socket = -1;
	h->state = 100;
}

void
Delete_upnphttp(struct upnphttp * h)
{
	if(h)
	{
		if(h->socket >= 0)
			close(h->socket);
		if(h->req_buf)
			free(h->req_buf);
		if(h->res_buf)
			free(h->res_buf);
		free(h);
	}
}

/* parse HttpHeaders of the REQUEST */
static void
ParseHttpHeaders(struct upnphttp * h)
{
	char * line;
	char * colon;
	char * p;
	int n;
	line = h->req_buf;
	/* TODO : check if req_buf, contentoff are ok */
	while(line < (h->req_buf + h->req_contentoff))
	{
		colon = strchr(line, ':');
		if(colon)
		{
			if(strncasecmp(line, "Content-Length", 14)==0)
			{
				p = colon;
				while(*p < '0' || *p > '9')
					p++;
				h->req_contentlen = atoi(p);
				/*printf("*** Content-Lenght = %d ***\n", h->req_contentlen);
				printf("    readbufflen=%d contentoff = %d\n",
					h->req_buflen, h->req_contentoff);*/
			}
			else if(strncasecmp(line, "SOAPAction", 10)==0)
			{
				p = colon;
				n = 0;
				while(*p == ':' || *p == ' ' || *p == '\t')
					p++;
				while(p[n]>=' ')
				{
					n++;
				}
				if((p[0] == '"' && p[n-1] == '"')
				  || (p[0] == '\'' && p[n-1] == '\''))
				{
					p++; n -= 2;
				}
				h->req_soapAction = p;
				h->req_soapActionLen = n;
			}
			else if(strncasecmp(line, "Authorization",13)==0)
			{
				p = colon;
				n = 0;
				while(*p == ':' || *p == ' ' || *p == '\t')
                                        p++;
				while(p[n]>=' ')
                                {
                                        n++;
                                }
				h->auth_buf = p;
				h->auth_buflen = n;
			}
		}
		while(!(line[0] == '\r' && line[1] == '\n'))
			line++;
		line += 2;
	}
}

/* very minimalistic 404 error message */
static void
Send404(struct upnphttp * h)
{
	static const char error404[] = "HTTP/1.1 404 Not found\r\n"
		"Connection: close\r\n"
		"Content-type: text/html\r\n"
		"\r\n"
		"<HTML><HEAD><TITLE>404 Not Found</TITLE></HEAD>"
		"<BODY><H1>Not Found</H1>The requested URL was not found"
		" on this server.</BODY></HTML>\r\n";
	int n;
	n = send(h->socket, error404, sizeof(error404) - 1, 0);
	if(n < 0)
	{
		SYSLOG(LOG_ERR, "Send404: send(http): %m");
	}
	CloseSocket_upnphttp(h);
}

/* very minimalistic 501 error message */
static void
Send501(struct upnphttp * h)
{
	static const char error501[] = "HTTP/1.1 501 Not Implemented\r\n"
		"Connection: close\r\n"
		"Content-type: text/html\r\n"
		"\r\n"
		"<HTML><HEAD><TITLE>501 Not Implemented</TITLE></HEAD>"
		"<BODY><H1>Not Implemented</H1>The HTTP Method "
		"is not implemented by this server.</BODY></HTML>\r\n";
	int n;
	n = send(h->socket, error501, sizeof(error501) - 1, 0);
	if(n < 0)
	{
		SYSLOG(LOG_ERR, "Send501: send(http): %m");
	}
	CloseSocket_upnphttp(h);
}

static void
Send401(struct upnphttp *h)
{
        static const char error401[] = "HTTP/1.1 401 Unauthorized\r\n"
                "Content-Length: 0\r\n"
		"Connection: close\r\n"
                "Content-type: text/html\r\n"
		"WWW-Authenticate: Digest realm=" TR64_REALM ", domain=\"/\", qop=\"auth\", algorithm=MD5, nonce=\"" TR64_NONCE "\"\r\n"
                "\r\n"
                "<HTML><HEAD><TITLE>401 Unauthorized</TITLE></HEAD>"
                "<BODY><H1>Unauthorized</H1>You need to authorize.</BODY></HTML>\r\n";
        int n;
        n = send(h->socket, error401, sizeof(error401) - 1, 0);
        if(n < 0)
        {
                SYSLOG(LOG_ERR, "Send401: send(http): %m");
        }
        CloseSocket_upnphttp(h);
}


static const char *
findendheaders(const char * s, int len)
{
	while(len-->0)
	{
		if(s[0]=='\r' && s[1]=='\n' && s[2]=='\r' && s[3]=='\n')
			return s;
		s++;
	}
	return NULL;
}

/********************
static void
sendDummyDesc(struct upnphttp * h)
{
	static const char xml_desc[] = "<?xml version=\"1.0\"?>\n"
		"<scpd xmlns=\"" URN_TAG ":service-1-0\">"
		" <specVersion>"
		"    <major>1</major>"
		"    <minor>0</minor>"
		"  </specVersion>"
		"  <actionList />"
		"  <serviceStateTable />"
		"</scpd>";
	BuildResp_upnphttp(h, xml_desc, sizeof(xml_desc)-1);
	SendResp_upnphttp(h);
	CloseSocket_upnphttp(h);
}***********/

/* Sends the description generated by the parameter */
/*********
static void
sendXMLdesc(struct upnphttp * h, char * (f)(int *))
{
	char * desc;
	int len;

	desc = f(&len);
	if(!desc)
	{
		SYSLOG(LOG_ERR, "Failed to generate XML description");
		return;
	}
	BuildResp_upnphttp(h, desc, len);
	SendResp_upnphttp(h);
	CloseSocket_upnphttp(h);
	free(desc);
}*********/

/* Sends the description generated by the parameter */
static void
sendTR64XMLdesc(struct upnphttp * h, char *HttpUrl)
{
    char * desc;
    int len;
    int32 iRet = IFX_CWMP_SUCCESS;

#ifdef IFX_DEFINE_TR64
    if(strstr(HttpUrl, ROOTDESC_PATH))
#else
    if(strcmp(ROOTDESC_PATH, HttpUrl) == 0)
#endif
    {
        iRet = IFX_TR64_DDDGenerate(&desc, (unsigned int *)&len);
        if (iRet != IFX_CWMP_SUCCESS)
        {
            SYSLOG(LOG_ERR, "Failed to generate XML description");
            SYSLOG(LOG_NOTICE, "%s not found, responding ERROR 404", HttpUrl);
            Send404(h);
            return;
        }
    }
    else
    {
        iRet = IFX_TR64_SDDGenerate(HttpUrl, &desc, (unsigned int *)&len);
        if (iRet != IFX_CWMP_SUCCESS)
        {
            SYSLOG(LOG_ERR, "Failed to generate XML description");
            SYSLOG(LOG_NOTICE, "%s not found, responding ERROR 404", HttpUrl);
            Send404(h);
            return;
        }
    }

    BuildResp_upnphttp(h, desc, len);
    SendResp_upnphttp(h);
    CloseSocket_upnphttp(h);
    free(desc);
}


/* ProcessHTTPPOST_upnphttp()
 * executes the SOAP query if it is possible */
static void
ProcessHTTPPOST_upnphttp(struct upnphttp * h)
{
	if((h->req_buflen - h->req_contentoff) >= h->req_contentlen)
	{
		if(h->req_soapAction)
		{
			/* we can process the request */
			SYSLOG(LOG_INFO, "SOAPAction: %.*s",
		    	   h->req_soapActionLen, h->req_soapAction);
			ExecuteSoapAction(h); 
		}
		else
		{
			static const char err400str[] =
				"<html><body>Bad request</body></html>";
			SYSLOG(LOG_INFO, "No SOAPAction in HTTP headers");
			BuildResp2_upnphttp(h, 400, "Bad Request",
			                    err400str, sizeof(err400str) - 1);
			SendResp_upnphttp(h);
			CloseSocket_upnphttp(h);
		}
	}
	else
	{
		/* waiting for remaining data */
		h->state = 1;
	}
}

/*
This function parses the upnphttp buffer SOAPAction field to check the type

RETURN TYPE
SOAPACTION_NULL
SOAPACTION_UNRESTRICTED
SOAPACTION_RESTRICTED
SOAPACTION_SETCONFIGPASSWD
*/

static int
HttpSOAPActionType(struct upnphttp *h)
{
	char *pHeader;	
	char *pAction;
	char * Action;
	int index=0;
	int iRet=0;
	int type;
        char8  sServiceId[IFX_TR64_MAX_SERVICEID_LEN];


	Action = h->Action;

	if(h->req_soapAction == NULL)
		return SOAPACTION_NULL;

	pHeader = strstr(h->req_soapAction,URN_TAG);	
	if(pHeader == NULL){
		SYSLOG(LOG_ERR, "TR64 : SOAPACTION field is blank!!");	
		return SOAPACTION_NULL;
	}

	pAction = strchr(pHeader,'#');
	pAction++;
	while(*pAction != '"'){
		Action[index] = *pAction;
		index++; pAction++;
	}
	Action[index] = '\0';
	
	if(!strncmp(Action,"SetConfigPassword",17))
		return SOAPACTION_SETCONFIGPASSWD;

        /* Get ServiceId from URL */
        iRet = IFX_TR64_SOAP_URLToServiceId(h->HttpUrl, sServiceId);
        /* Check for Error */
        if (iRet != IFX_CWMP_SUCCESS)
        {
            SYSLOG(LOG_ERR, "URL does not have ServiceId !!");
	    return SOAPACTION_NULL;
        }

	//VIVEK : this should call a function which is an interface 
	//to TR64 DB file that tells if it is restricted or what -- func(Action)
        
	iRet = IFX_CheckIfPasswordProtectAction(sServiceId, h->Action, &type);
	if(iRet != IFIN_CWMP_SUCCESS){
		SYSLOG(LOG_ERR, "TR64 : IFX_CheckIfPasswordProtectAction failed !!");
		return SOAPACTION_NULL;	// default error return
	}

	if(type == 1)
#ifdef IFX_DEFINE_TR64
		return SOAPACTION_RESTRICTED;
#else
		return SOAPACTION_UNRESTRICTED;
#endif
	else if(type == 0)
		return SOAPACTION_UNRESTRICTED;
	else
		return SOAPACTION_NULL;	// default error return

}

/*
Given a pointer to a field, it returns the value
eg. 
source ----> username="000319-DMA66"
response --> 000319-DMA66

source ----> nc=00000003
response --> 00000003
*/
static void
ExtractValue(char *source, char *response)
{
	char *p;

	p = strchr(source,'=');
	while((*p != ',') && (*p != ' ') && (*p != '\0')){
		if((*p == '"') || (*p == '=') || (*p == '\\')){
			p++;
			continue;
		}

		*response = *p;
		p++; response++;
	}	

	*response = '\0';
}



/*
This function does the parsing of the auth_buff in the upnphttp struct and 
then does the authentication based on password passed

RETURN VAL :
	IFX_CWMP_SUCCESS
	IFX_CWMP_FAILURE

*/
	static int
AuthVerify(struct upnphttp *h, char * passwd)
{
#ifndef IFX_DEFINE_TR64
	return IFX_CWMP_SUCCESS;
#endif

	HASHHEX HA1;
	HASHHEX HA2 = "";
	HASHHEX INResponse;
	HASHHEX OUTResponse;
	char algorithm[100]={0};
	char username[100]={0};
	char realm[100]={0};
	char nonce[100]={0};
	char cnonce[100]={0};
	char nc[100]={0};
	char qop[100]={0};
	char uri[100]={0};
	char method[100]={0};
	char *source;
        char * p;
        int i;


        p = h->req_buf;

        if(!p)
                return IFX_CWMP_FAILURE;

        for(i = 0; i<15 && *p != ' ' && *p != '\r'; i++)
                method[i] = *(p++);
        method[i] = '\0';

	source = strstr(h->auth_buf, "username=");
	ExtractValue(source,username);
		
	source = strstr(h->auth_buf, "realm=");
	ExtractValue(source,realm);
	
	source = strstr(h->auth_buf, "nonce=");
	ExtractValue(source,nonce);
	
	source = strstr(h->auth_buf, "uri=");
	ExtractValue(source,uri);
	
	source = strstr(h->auth_buf, "response=");
	ExtractValue(source,INResponse);
	
	source = strstr(h->auth_buf, "algorithm=");
	ExtractValue(source,algorithm);

	source = strstr(h->auth_buf, "cnonce=");
	ExtractValue(source,cnonce);

	source = strstr(h->auth_buf, "qop=");
	ExtractValue(source,qop);
	
	source = strstr(h->auth_buf, "nc=");
	ExtractValue(source,nc);

	printf("%s %d %s %s %s\n",__FUNCTION__,__LINE__,username,realm,passwd);
	DigestCalcHA1(algorithm, username, realm, passwd, nonce, cnonce, HA1);

	DigestCalcResponse(HA1, nonce, nc, cnonce, qop, method, uri,	HA2, OUTResponse);

	if(strncmp(INResponse,OUTResponse,32))
		return IFX_CWMP_SUCCESS;
	else
		return IFX_CWMP_FAILURE;

}



/*
This function does authorization if required or rejects 

Return Val
	IFX_CWMP_FAILURE
	IFX_CWMP_SUCCESS

*/
static int
HttpAuthorize(struct upnphttp *h)
{
	int SOAPActionType=0;
	int iRet=0;
	int iTR64State = STATE_FACTORY;
	char config_passwd[100]={0};
	char reset_passwd[100]={0};


        // check for SOAPAction, if unrestricted, then return success
        SOAPActionType = HttpSOAPActionType(h);

        if(iTR64_HttpAuthEnable != 1)
                return IFX_CWMP_SUCCESS;


	//VIVEK : Get the two passwords from somewhere	
	iRet = IFX_Config_GetLanConfPassword(config_passwd, reset_passwd);
	if(iRet != IFX_CWMP_SUCCESS){
		SYSLOG(LOG_ERR, "[%s:%d] IFX_Config_GetLanConfPassword failed !! ",__func__,__LINE__);	
		return IFX_CWMP_FAILURE;
	}
//	strcpy(config_passwd,"configadmin"); //VIVEK 
//	strcpy(reset_passwd,"resetadmin");

	// set TR64 state
	if(!strcmp(config_passwd,""))
		iTR64State = STATE_FACTORY;
	else
		iTR64State = STATE_NORMAL;	


	switch(SOAPActionType)
	{
		case SOAPACTION_NULL		:
							iRet = IFX_CWMP_FAILURE;
							break;

		case SOAPACTION_UNRESTRICTED	:
							iRet = IFX_CWMP_SUCCESS;
							break;
		
		case SOAPACTION_SETCONFIGPASSWD	:
							if(iTR64State == STATE_FACTORY){
								iRet = AuthVerify(h,reset_passwd);
								if(iRet == IFX_CWMP_SUCCESS)
									iTR64State = STATE_NORMAL;
							}else if(iTR64State == STATE_NORMAL){	
								iRet = AuthVerify(h,config_passwd);
								if(iRet == IFX_CWMP_FAILURE)
									iRet = AuthVerify(h,reset_passwd);
							}
							break;

		case SOAPACTION_RESTRICTED	:
							if(iTR64State == STATE_FACTORY)
								iRet = IFX_CWMP_FAILURE;
							else
							//	iRet = AuthVerify(h, config_passwd);
							break;

		default :
							iRet = IFX_CWMP_FAILURE;
	}

	return iRet;

}




/* Parse and process Http Query 
 * called once all the HTTP headers have been received. */
static void
ProcessHttpQuery_upnphttp(struct upnphttp * h)
{
	char * HttpCommand;
	char * HttpUrl;
	char * HttpVer;
	char * p;
	int i;

	p = h->req_buf;
	if(!p)
		return;

	HttpCommand = h->HttpCommand;
	HttpUrl = h->HttpUrl;
	HttpVer = h->HttpVer;

	for(i = 0; i<15 && *p != ' ' && *p != '\r'; i++)
		HttpCommand[i] = *(p++);
	HttpCommand[i] = '\0';
	while(*p==' ')
		p++;
	for(i = 0; i<127 && *p != ' ' && *p != '\r'; i++)
		HttpUrl[i] = *(p++);
	HttpUrl[i] = '\0';
	while(*p==' ')
		p++;
	for(i = 0; i<15 && *p != '\r'; i++)
		HttpVer[i] = *(p++);
	HttpVer[i] = '\0';
	SYSLOG(LOG_INFO, "HTTP REQUEST : %s %s (%s)",
	       HttpCommand, HttpUrl, HttpVer);

	ParseHttpHeaders(h);


	if(strcmp("POST", HttpCommand) == 0)
	{
		if(HttpAuthorize(h) == IFX_CWMP_FAILURE){
			Send401(h);
			return;
		}

		h->req_command = EPost;
		ProcessHTTPPOST_upnphttp(h);
		CloseSocket_upnphttp(h);
	}
	else if(strcmp("GET", HttpCommand) == 0)
	{
		h->req_command = EGet;
		sendTR64XMLdesc(h, HttpUrl);
	}
	else if(strcmp("SUBSCRIBE", HttpCommand) == 0)
	{
		SYSLOG(LOG_NOTICE, "SUBSCRIBE not implemented yet");
		Send501(h);
	}
	else
	{
		SYSLOG(LOG_NOTICE, "Unsupported HTTP Command %s", HttpCommand);
		Send501(h);
	}
}


void
Process_upnphttp(struct upnphttp * h)
{
	char buf[2048];
	int n;
	if(!h)
		return;
	switch(h->state)
	{
	case 0:
		n = recv(h->socket, buf, 2048, 0);
		if(n<0)
		{
			SYSLOG(LOG_ERR, "recv (state0): %m");
			h->state = 100;
		}
		else if(n==0)
		{
			SYSLOG(LOG_WARNING, "HTTP Connection closed unexpectedly");
			h->state = 100;
		}
		else
		{
			const char * endheaders;
			/* if 1st arg of realloc() is null,
			 * realloc behaves the same as malloc() */
			h->req_buf = (char *)realloc(h->req_buf, n + h->req_buflen + 1);
			memcpy(h->req_buf + h->req_buflen, buf, n);
			h->req_buflen += n;
			h->req_buf[h->req_buflen] = '\0';
			/* search for the string "\r\n\r\n" */
			endheaders = findendheaders(h->req_buf, h->req_buflen);
			if(endheaders)
			{
				h->req_contentoff = endheaders - h->req_buf + 4;
				ProcessHttpQuery_upnphttp(h);
			}
		}
		break;
	case 1:
		n = recv(h->socket, buf, 2048, 0);
		if(n<0)
		{
			SYSLOG(LOG_ERR, "recv (state1): %m");
			h->state = 100;
		}
		else if(n==0)
		{
			SYSLOG(LOG_WARNING, "HTTP Connection closed inexpectedly");
			h->state = 100;
		}
		else
		{
			/*fwrite(buf, 1, n, stdout);*/	/* debug */
			h->req_buf = (char *)realloc(h->req_buf, n + h->req_buflen);
			memcpy(h->req_buf + h->req_buflen, buf, n);
			h->req_buflen += n;
			if((h->req_buflen - h->req_contentoff) >= h->req_contentlen)
			{
				ProcessHTTPPOST_upnphttp(h);
			}
		}
		break;
	default:
		SYSLOG(LOG_WARNING, "Unexpected state: %d", h->state);
	}
}

static const char httpresphead[] =
	"%s %d %s\r\n"
	"Content-Type: text/xml; charset=\"utf-8\"\r\n"
	"Connection: close\r\n"
	"Content-Length: %d\r\n"
	/*"Server: miniupnpd/1.0 UPnP/1.0\r\n"*/
	"Server: " MINIUPNPD_SERVER_STRING "\r\n"
	"Ext:\r\n"
	"\r\n";
/*
		"<?xml version=\"1.0\"?>\n"
		"<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" "
		"s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
		"<s:Body>"

		"</s:Body>"
		"</s:Envelope>";
*/
/* with response code and response message
 * also allocate enough memory */

void
BuildHeader_upnphttp(struct upnphttp * h, int respcode,
                     const char * respmsg,
                     int bodylen)
{
	int templen;
	if(!h->res_buf)
	{
		templen = sizeof(httpresphead) + 64 + bodylen;
		h->res_buf = (char *)malloc(templen);
		h->res_buf_alloclen = templen;
	}
	h->res_buflen = snprintf(h->res_buf, h->res_buf_alloclen,
	                         httpresphead, h->HttpVer,
	                         respcode, respmsg, bodylen);
	if(h->res_buf_alloclen < (h->res_buflen + bodylen))
	{
		h->res_buf = (char *)realloc(h->res_buf, (h->res_buflen + bodylen));
		h->res_buf_alloclen = h->res_buflen + bodylen;
	}
}

void
BuildResp2_upnphttp(struct upnphttp * h, int respcode,
                    const char * respmsg,
                    const char * body, int bodylen)
{
	BuildHeader_upnphttp(h, respcode, respmsg, bodylen);
	memcpy(h->res_buf + h->res_buflen, body, bodylen);
	h->res_buflen += bodylen;
}

/* responding 200 OK ! */
void
BuildResp_upnphttp(struct upnphttp * h,
                        const char * body, int bodylen)
{
	BuildResp2_upnphttp(h, 200, "OK", body, bodylen);
}

void
SendResp_upnphttp(struct upnphttp * h)
{
	int n;
	n = send(h->socket, h->res_buf, h->res_buflen, 0);
	if(n<0)
	{
		SYSLOG(LOG_ERR, "send(res_buf): %m");
	}
	else if(n < h->res_buflen)
	{
		SYSLOG(LOG_ERR, "send(res_buf): %d bytes sent (out of %d)",
						n, h->res_buflen);
	}
}

