/* $Id: upnphttp.h,v 1.14 2007/02/07 22:28:07 nanard Exp $ */ 
/* MiniUPnP project
 * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
 * (c) 2006 Thomas Bernard 
 * This software is subject to the conditions detailed
 * in the LICENCE file provided within the distribution */

#ifndef __UPNPHTTP_H__
#define __UPNPHTTP_H__

#include <sys/queue.h>

#include "config.h"

/***define IFX_UPNP(default) or IFX_TR64*********/
//#define IFX_DEFINE_UPNP

/* server: HTTP header returned in all HTTP responses : */
#define MINIUPNPD_SERVER_STRING	OS_VERSION " UPnP/1.0 miniupnpd/1.0"
#define TR64_REALM  "CpeAuthentication"
#define TR64_NONCE "Viveknonce"
// CPE STATES 
#define STATE_FACTORY 0
#define STATE_NORMAL  1

//SOAPActionTypes
#define SOAPACTION_NULL 		0 
#define SOAPACTION_UNRESTRICTED		1
#define SOAPACTION_RESTRICTED		2
#define SOAPACTION_SETCONFIGPASSWD	3



#define UPNP_URN_TAG	"urn:schemas-upnp-org"
#define TR64_URN_TAG	"urn:dslforum-org"

#ifdef IFX_DEFINE_TR64
#define URN_TAG  TR64_URN_TAG
#else
#define URN_TAG  UPNP_URN_TAG
#endif


/*
 states :
  0 - waiting for data to read
  1 - waiting for HTTP Post Content.
  ...
  >= 100 - to be deleted
*/
enum httpCommands {
	EUnknown = 0,
	EGet,
	EPost
};

struct upnphttp {
	int socket;
	int state;
	char HttpVer[16];	
	char Action[64];	// Service action for TR64
        char HttpUrl[128];	// contains the control URL for TR^4
	char HttpCommand[16];   // store GET , POST etc ..
/* request */
	char * req_buf;
	int req_buflen;
	int req_contentlen;
	int req_contentoff;     /* header length */
	enum httpCommands req_command;
	char * req_soapAction;
	int req_soapActionLen;
	/* response */
	char * res_buf;
	int res_buflen;
	int res_buf_alloclen;
	char * auth_buf;
	int auth_buflen;
	/*int res_contentlen;*/
	/*int res_contentoff;*/		/* header length */
	LIST_ENTRY(upnphttp) entries;
};

/* New_upnphttp() */
struct upnphttp *
New_upnphttp(int);

/* CloseSocket_upnphttp() */
void
CloseSocket_upnphttp(struct upnphttp *);

/* Delete_upnphttp() */
void
Delete_upnphttp(struct upnphttp *);

/* Process_upnphttp() */
void
Process_upnphttp(struct upnphttp *);

/* BuildHeader_upnphttp()
 * build the header for the HTTP Response
 * also allocate the buffer for body data */
void
BuildHeader_upnphttp(struct upnphttp * h, int respcode,
                     const char * respmsg,
                     int bodylen);

/* BuildResp_upnphttp() 
 * fill the res_buf buffer with the complete
 * HTTP 200 OK response from the body passed as argument */
void
BuildResp_upnphttp(struct upnphttp *, const char *, int);

/* BuildResp2_upnphttp()
 * same but with given response code/message */
void
BuildResp2_upnphttp(struct upnphttp * h, int respcode,
                    const char * respmsg,
                    const char * body, int bodylen);

/* SendResp_upnphttp() */
void
SendResp_upnphttp(struct upnphttp *);

#endif

