/* $Id: upnpsoap.c,v 1.53 2007/02/07 22:16:19 nanard Exp $ */
/* MiniUPnP project
 * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
 * (c) 2006 Thomas Bernard 
 * This software is subject to the conditions detailed
 * in the LICENCE file provided within the distribution */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>

#include "upnpglobalvars.h"
#include "upnphttp.h"
#include "upnpsoap.h"
#include "upnpreplyparse.h"
#include "IFX_TR64.h" 
//#include "upnpredirect.h"
//#include "getifaddr.h"
//#include "getifstats.h"

static void
BuildSendAndCloseSoapResp(struct upnphttp * h,
                          const char * body, int bodylen)
{
	static const char beforebody[] =
		"<?xml version=\"1.0\"?>\n"
		"<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" "
		"s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
		"<s:Body>";

	static const char afterbody[] =
		"</s:Body>"
		"</s:Envelope>";

	BuildHeader_upnphttp(h, 200, "OK",  sizeof(beforebody) - 1
		+ sizeof(afterbody) - 1 + bodylen );

	memcpy(h->res_buf + h->res_buflen, beforebody, sizeof(beforebody) - 1);
	h->res_buflen += sizeof(beforebody) - 1;

	memcpy(h->res_buf + h->res_buflen, body, bodylen);
	h->res_buflen += bodylen;

	memcpy(h->res_buf + h->res_buflen, afterbody, sizeof(afterbody) - 1);
	h->res_buflen += sizeof(afterbody) - 1;

	SendResp_upnphttp(h);
	CloseSocket_upnphttp(h);
}


static void
GetConnectionTypeInfo(struct upnphttp * h)
{
	static const char resp[] =
		"<u:GetConnectionTypeInfoResponse "
		"xmlns:u=\"" URN_TAG ":service:WANIPConnection:1\">"
		"<NewConnectionType>IP_Routed</NewConnectionType>"
		"<NewPossibleConnectionTypes>IP_Routed</NewPossibleConnectionTypes>"
		"</u:GetConnectionTypeInfoResponse>";
	BuildSendAndCloseSoapResp(h, resp, sizeof(resp)-1);
}


/* Windows XP as client send the following requests :
 * GetConnectionTypeInfo
 * GetNATRSIPStatus
 * ? GetTotalBytesSent - WANCommonInterfaceConfig
 * ? GetTotalBytesReceived - idem
 * ? GetTotalPacketsSent - idem
 * ? GetTotalPacketsReceived - idem
 * GetCommonLinkProperties - idem
 * GetStatusInfo - WANIPConnection
 * GetExternalIPAddress
 * QueryStateVariable / ConnectionStatus!
 */
static const struct 
{
	const char * methodName; 
	void (*methodImpl)(struct upnphttp *);
}
soapMethods[] =
{
	{ "GetConnectionTypeInfo", GetConnectionTypeInfo },
	{ 0, 0 }
};

void
ExecuteSoapAction(struct upnphttp * h)
{
	char *pSOAPBuf = NULL;
        char8  sServiceId[IFX_TR64_MAX_SERVICEID_LEN];
        int32 iRet = IFX_CWMP_SUCCESS;

	//search for soap envelop in the buffer and call SOAP
	pSOAPBuf = strstr(h->req_buf, "<s:Envelope");
        if(pSOAPBuf == NULL){
                pSOAPBuf = strstr(h->req_buf, "<SOAP-ENV:Envelope");
        }

        /* Get ServiceId from URL */
        iRet = IFX_TR64_SOAP_URLToServiceId(h->HttpUrl, sServiceId);
        /* Check for Error */
        if (iRet != IFX_CWMP_SUCCESS)
        {
            SYSLOG(LOG_ERR, "URL does not have ServiceId !!");
	    goto cleanup;
        }


	//call SOAP with soap body

	iRet = IFX_TR64_SOAP_Parse(pSOAPBuf,sServiceId, h->Action); 


cleanup :
	if(iRet != IFX_CWMP_SUCCESS)
	SoapError(h, 401, "Invalid Action");
}

/* Standard Errors:
 *
 * errorCode  	errorDescription  	Description
 * --------		----------------	-----------
 * 401 			Invalid Action 		No action by that name at this service.
 * 402 			Invalid Args 		Could be any of the following: not enough in args,
 * 									too many in args, no in arg by that name, 
 * 									one or more in args are of the wrong data type.
 * 403 			Out of Sync 		Out of synchronization.
 * 501 			Action Failed 		May be returned in current state of service prevents 
 * 									invoking that action.
 * 600-699 		TBD 				Common action errors. Defined by UPnP Forum Technical 
 * 									Committee.
 * 700-799 		TBD 				Action-specific errors for standard actions. Defined 
 * 									by UPnP Forum working committee.
 * 800-899 		TBD 				Action-specific errors for non-standard actions. 
 * 									Defined by UPnP vendor.
*/
void
SoapError(struct upnphttp * h, int errCode, const char * errDesc)
{
	static const char resp[] = 
		"<s:Envelope "
		"xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" "
		"s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
		"<s:Body>"
		"<s:Fault>"
		"<faultcode>s:Client</faultcode>"
		"<faultstring>UPnPError</faultstring>"
		"<detail>"
		"<UPnPError xmlns=\"" URN_TAG ":control-1-0\">"
		"<errorCode>%d</errorCode>"
		"<errorDescription>%s</errorDescription>"
		"</UPnPError>"
		"</detail>"
		"</s:Fault>"
		"</s:Body>"
		"</s:Envelope>";

	char body[2048];
	int bodylen;

	bodylen = snprintf(body, sizeof(body), resp, errCode, errDesc);
	BuildResp2_upnphttp(h, 500, "Internal Server Error", body, bodylen);
	SendResp_upnphttp(h);
	CloseSocket_upnphttp(h);
}

