/* Support for cookies.
*/

#include <config.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <time.h>
#include "httpreadwrite.h"
#include "httpcookie.h"

int
cookie_match_tail (const char *string, const char *tail, int fold_case_p)
{
  int i, j;

  if (!fold_case_p)
    {
      for (i = strlen (string), j = strlen (tail); i >= 0 && j >= 0; i--, j--)
        if (string[i] != tail[j])
          break;
    }
  else
    {
      for (i = strlen (string), j = strlen (tail); i >= 0 && j >= 0; i--, j--)
        if (TOLOWER (string[i]) != TOLOWER (tail[j]))
          break;
    }

 if (j == -1)
    return 1;
  else
    return 0;
}

#define countof(array) (sizeof (array) / sizeof ((array)[0]))

static int
check_end (const char *p)
{
  if (!p)
    return 0;
  while (ISSPACE (*p))
    ++p;
  if (!*p
      || (p[0] == 'G' && p[1] == 'M' && p[2] == 'T')
      || ((p[0] == '+' || p[0] == '-') && ISDIGIT (p[1])))
    return 1;
  else
    return 0;
}


time_t
cookie_atotm (const char *time_string)
{
  struct tm t;

  memset (&t, '\0', sizeof (t));
//  strptime (time_string, "%a, %d-%h-%y %H:%M:%S", &t);
char cook_time[200]; strftime(cook_time,sizeof(cook_time),"%a, %d-%h-%Y %H:%M:%S",&t);

  if (check_end (( char * )strptime (time_string, "%a, %d-%h-%y %H:%M:%S", &t)))
	return mktime(&t);
    /* All formats have failed.  */
  return -1;
}
 
time_t cookies_now;

static int
 httpcookie_compare(void *param1,  void *param2)
{
    assert( param1 != NULL );
    assert( param2 != NULL );

    return ( ( struct cookie * ) param1  == ( struct cookie * ) param2 ? 1 : 0 );

}

void httpcookie_free(void *ptr)
{
  struct cookie *cookie = (struct cookie *)ptr;
  http_delete_cookie (cookie);
  
}

struct cookie_jar *
http_cookie_jar_new (void)
{
  struct cookie_jar *jar =( struct cookie_jar *) malloc (sizeof(struct cookie_jar) * 1);
  memset(jar,0,sizeof( struct cookie_jar));
//  jar->chains = make_nocase_string_hash_table (0);
  ListInit( &jar->chains, httpcookie_compare, httpcookie_free );
  jar->cookie_count = 0;
  return jar;
}


#define PORT_ANY (-1)

/* Allocate and return a new, empty cookie structure. */

static struct cookie *
http_cookie_new (void)
{
  struct cookie *cookie = malloc (sizeof(struct cookie) * 1);
  memset(cookie,0,sizeof(struct cookie));
  membuffer_init(&cookie->domain);
  membuffer_init(&cookie->path);
  membuffer_init(&cookie->attr);
  membuffer_init(&cookie->value);
  membuffer_init(&cookie->host);

  cookie->port = PORT_ANY;
  return cookie;
}

/* Non-zero if the cookie has expired.  Assumes cookies_now has been
   set by one of the entry point functions.  */

static int
cookie_expired_p (const struct cookie *c)
{
  return c->expiry_time != 0 && c->expiry_time < cookies_now;
}

/* Deallocate COOKIE and its components. */

 void
http_delete_cookie (struct cookie *cookie)
{
  membuffer_destroy (&cookie->domain);
  membuffer_destroy(&cookie->path);
  membuffer_destroy(&cookie->attr);
  membuffer_destroy(&cookie->value);
  membuffer_destroy(&cookie->host);

  free (cookie);
}

static struct cookie *
http_find_matching_cookie (struct cookie_jar *jar, struct cookie *cookie,
		      struct cookie **prevptr)
{
  struct cookie *chain;
  ListNode *node, *prev;

  prev = NULL;

  node = ListHead( &jar->chains );

  while(node != NULL) 
  {  
    chain = ( struct cookie * ) node->item;

    if (0 == strcmp (cookie->path.buf, chain->path.buf)
	&& 0 == strcmp (cookie->attr.buf, chain->attr.buf)
	&& cookie->port == chain->port)
    {
	if( prev != NULL)
	       *prevptr = prev->item;
	else 
		*prevptr = NULL;

	return chain;
    }
    prev = node ;  
    node = ListNext( &jar->chains, node );

  }//while
  return NULL;
 
}

/* Store COOKIE to the jar.

   This is done by placing COOKIE at the head of its chain.  However,
   if COOKIE matches a cookie already in memory, as determined by
   http_find_matching_cookie, the old cookie is unlinked and destroyed.

*/
static void
http_store_cookie (struct cookie_jar *jar, struct cookie *cookie)
{
  
  
  struct cookie *prev;
  struct cookie *victim;
  ListNode *node;

   
     victim = http_find_matching_cookie (jar, cookie, &prev);
     //head = ListHead( &jar->chains );
     if(victim )
     {
	     node = ListFind( &jar->chains, NULL, victim );
	     ListDelNode(&jar->chains,node,1);
	     --jar->cookie_count;
     }
     ListAddHead( &jar->chains, cookie );
     ++jar->cookie_count;

}

/* Discard a cookie matching COOKIE's domain, port, path, and
   attribute name.  This gets called when we encounter a cookie whose
   expiry date is in the past, or whose max-age is set to 0.  The
   former corresponds to netscape cookie spec, while the latter is
   specified by rfc2109.  */

static void
discard_matching_cookie (struct cookie_jar *jar, struct cookie *cookie)
{
  struct cookie *prev, *victim;
  ListNode * head,*node;

  head= ListHead(&jar->chains);
  if (head== NULL)
//    No elements == nothing to discard. 
    return;

   victim = http_find_matching_cookie (jar, cookie, &prev);
     //head = ListHead( &jar->chains );
     if(victim )
     {
             node = ListFind( &jar->chains, NULL, victim );
             ListDelNode(&jar->chains,node,1);
             --jar->cookie_count;
     }

}

/* Functions for parsing the `Set-Cookie' header, and creating new
   cookies from the wire.  */

#define NAME_IS(string_literal)					\
  BOUNDED_EQUAL_NO_CASE (name_b, name_e, string_literal)

#define VALUE_EXISTS (value_b && value_e)

#define VALUE_NON_EMPTY (VALUE_EXISTS && (value_b != value_e))


static int
http_set_cookie_field (struct cookie *cookie,
		     const char *name_b, const char *name_e,
		     const char *value_b, const char *value_e)
{

  char *value_copy;
  time_t expires;
  double maxage = -1;

  assert (name_b != NULL && name_e != NULL);

  if (cookie->attr.length == 0)
    {
      if (!VALUE_EXISTS)
	return 0;
	membuffer_assign(&cookie->attr,name_b,name_e-name_b);
        membuffer_assign(&cookie->value,value_b,value_e-value_b);
        return 1;
    }

  if (NAME_IS ("domain"))
    {
      if (!VALUE_NON_EMPTY)
	return 0;
//      free (cookie->domain);
      /* Strictly speaking, we should set cookie->domain_exact if the
	 domain doesn't begin with a dot.  But many sites set the
	 domain to "foo.com" and expect "subhost.foo.com" to get the
	 cookie, and it apparently works.  */
      if (*value_b == '.')
	++value_b;
      membuffer_assign(&cookie->domain,value_b,value_e-value_b);
//      cookie->domain = strdupdelim (value_b, value_e);
      return 1;
    }
  else if (NAME_IS ("path"))
    {
      if (!VALUE_NON_EMPTY)
	return 0;
      //free (cookie->path);
      membuffer_assign(&cookie->path,value_b,value_e-value_b);
//      cookie->path = strdupdelim (value_b, value_e);
      return 1;
    }
  else if (NAME_IS ("expires"))
    {

      if (!VALUE_NON_EMPTY)
	return 0;
      BOUNDED_TO_ALLOCA (value_b, value_e, value_copy);

      expires = cookie_atotm (value_copy);
      if (expires != (time_t) -1)
	{
	  cookie->permanent = 1;
	  cookie->expiry_time = expires;
	}
      else
	/* Error in expiration spec.  Assume default (cookie doesn't
	   expire, but valid only for this session.)  */
	;

      /* According to netscape's specification, expiry time in the
	 past means that discarding of a matching cookie is
	 requested.  */
      if (cookie->expiry_time < cookies_now)
	cookie->discard_requested = 1;

      return 1;
    }
  else if (NAME_IS ("max-age"))
    {
      if (!VALUE_NON_EMPTY)
	return 0;
      BOUNDED_TO_ALLOCA (value_b, value_e, value_copy);

      sscanf (value_copy, "%lf", &maxage);
      if (maxage == -1)
	/* something went wrong. */
	return 0;
      cookie->permanent = 1;
      cookie->expiry_time = cookies_now + maxage;

      /* According to rfc2109, a cookie with max-age of 0 means that
	 discarding of a matching cookie is requested.  */
      if (maxage == 0)
	cookie->discard_requested = 1;

      return 1;
    }
  else if (NAME_IS ("secure"))
    {
      /* ignore value completely */
      cookie->secure = 1;
      return 1;
    }
  else
    /* Unrecognized attribute; ignore it. */
    return 1;
}

#undef NAME_IS


#define ATTR_NAME_CHAR(c) ((c) > 32 && (c) < 127	\
			   && (c) != '"' && (c) != '='	\
			   && (c) != ';' && (c) != ',')

/* Parse the contents of the `Set-Cookie' header.  The header looks
   like this:

   name1=value1; name2=value2; ...

   Trailing semicolon is optional; spaces are allowed between all
   tokens.  Additionally, values may be quoted.

   A new cookie is returned upon success, NULL otherwise.  The
   specified CALLBACK function (normally `http_set_cookie_field' is used
   to update the fields of the newly created cookie structure.  */

static struct cookie *
http_parse_set_cookies (const char *sc,
		   int (*callback) (struct cookie *,
				    const char *, const char *,
				    const char *, const char *),
		   int silent)
{
  int legal;
  struct cookie *cookie ;
  const char *p = sc;
  char c;
  const char *name_b  = NULL, *name_e  = NULL;
  const char *value_b = NULL, *value_e = NULL;
  char *name;
  enum { S_START, S_NAME, S_NAME_POST,
         S_VALUE_PRE, S_VALUE, S_QUOTED_VALUE, S_VALUE_TRAILSPACE,
         S_ATTR_ACTION, S_DONE, S_ERROR
	}state = S_START;
  c = *p;

  cookie = http_cookie_new ();


  while (state != S_DONE && state != S_ERROR)
    {
      switch (state)
	{
	case S_START:
	  if (!c)
	    state = S_DONE;
	  else if (ISSPACE (c))
	    /* Strip all whitespace preceding the name. */
	    c = *++p;
	  else if (ATTR_NAME_CHAR (c))
	    {
	      name_b = p;
	      state = S_NAME;
	    }
	  else
	    /* empty attr name not allowed */
	    state = S_ERROR;
	  break;
	case S_NAME:
	  if (!c || c == ';' || c == '=' || ISSPACE (c))
	    {
	      name_e = p;
	      state = S_NAME_POST;
	    }
	  else if (ATTR_NAME_CHAR (c))
	    c = *++p;
	  else
	    state = S_ERROR;
	  break;
	case S_NAME_POST:
	  if (!c || c == ';')
	    {
	      value_b = value_e = NULL;
	      if (c == ';')
		c = *++p;
	      state = S_ATTR_ACTION;
	    }
	  else if (c == '=')
	    {
	      c = *++p;
	      state = S_VALUE_PRE;
	    }
	  else if (ISSPACE (c))
	    /* Ignore space and keep the state. */
	    c = *++p;
	  else
	    state = S_ERROR;
	  break;
	case S_VALUE_PRE:
	  if (!c || c == ';')
	    {
	      value_b = value_e = p;
	      if (c == ';')
		c = *++p;
	      state = S_ATTR_ACTION;
	    }
	  else if (c == '"')
	    {
	      c = *++p;
	      value_b = p;
	      state = S_QUOTED_VALUE;
	    }
	  else if (ISSPACE (c))
	    c = *++p;
	  else
	    {
	      value_b = p;
	      value_e = NULL;
	      state = S_VALUE;
	    }
	  break;
	case S_VALUE:
	  if (!c || c == ';' || ISSPACE (c))
	    {
	      value_e = p;
	      state = S_VALUE_TRAILSPACE;
	    }
	  else
	    {
	      value_e = NULL;	/* no trailing space */
	      c = *++p;
	    }
	  break;
	case S_QUOTED_VALUE:
	  if (c == '"')
	    {
	      value_e = p;
	      c = *++p;
	      state = S_VALUE_TRAILSPACE;
	    }
	  else if (!c)
	    state = S_ERROR;
	  else
	    c = *++p;
	  break;
	case S_VALUE_TRAILSPACE:
	  if (c == ';')
	    {
	      c = *++p;
	      state = S_ATTR_ACTION;
	    }
	  else if (!c)
	    state = S_ATTR_ACTION;
	  else if (ISSPACE (c))
	    c = *++p;
	  else
	    state = S_VALUE;
	  break;
	case S_ATTR_ACTION:
	  {
	    legal = callback (cookie, name_b, name_e, value_b, value_e);
	    if (!legal)
	      {
		if (!silent)
		  {
		    BOUNDED_TO_ALLOCA (name_b, name_e, name);
		    
		  }
		state = S_ERROR;
		break;
	      }
	    state = S_START;
//state = S_DONE;
	  }
	  break;
	case S_DONE:
	case S_ERROR:
	  /* handled by loop condition */
	  break;
	}
    }
  if (state == S_DONE)
    return cookie;

  http_delete_cookie (cookie);
  if (state != S_ERROR)
    abort ();

  if (!silent)
    printf       ("Syntax error in Set-Cookie: %s at position %d.\n",
	       sc, p - sc);
  return NULL;
}

/* Sanity checks.  These are important, otherwise it is possible for
   mailcious attackers to destroy important cookie information and/or
   violate your privacy.  */


#define REQUIRE_DIGITS(p) do {			\
  if (!ISDIGIT (*p))				\
    return 0;					\
  for (++p; ISDIGIT (*p); p++)			\
    ;						\
} while (0)

#define REQUIRE_DOT(p) do {			\
  if (*p++ != '.')				\
    return 0;					\
} while (0)

/* Check whether ADDR matches <digits>.<digits>.<digits>.<digits>.

   We don't want to call network functions like inet_addr() because
   all we need is a check, preferrably one that is small, fast, and
   well-defined.  */

static int
numeric_address_p (const char *addr)
{
  const char *p = addr;

  REQUIRE_DIGITS (p);		/* A */
  REQUIRE_DOT (p);		/* . */
  REQUIRE_DIGITS (p);		/* B */
  REQUIRE_DOT (p);		/* . */
  REQUIRE_DIGITS (p);		/* C */
  REQUIRE_DOT (p);		/* . */
  REQUIRE_DIGITS (p);		/* D */

  if (*p != '\0')
    return 0;
  return 1;
}

/* Check whether COOKIE_DOMAIN is an appropriate domain for HOST.
   Originally I tried to make the check compliant with rfc2109, but
   the sites deviated too often, so I had to fall back to "tail
   matching", as defined by the original Netscape's cookie spec.  */

static int
check_domain_match (const char *cookie_domain, const char *host)
{
   const char *p = cookie_domain;
    int dccount = 1;            /* number of domain components */
    int ldcl  = 0;              /* last domain component length */
    int nldcl = 0;              /* next to last domain component length */
    int out;
    int i;
    int known_toplevel = 0;
    static const char *known_toplevel_domains[] = {
   ".com", ".edu", ".net", ".org", ".gov", ".mil", ".int"
   };
   int dlen,hlen;
   dlen = strlen (cookie_domain);
   hlen = strlen (host);

  /* Numeric address requires exact match.  It also requires HOST to
     be an IP address.  */
  if (numeric_address_p (cookie_domain))
    return 0 == strcmp (cookie_domain, host);


  /* For the sake of efficiency, check for exact match first. */
  if (0 == strcasecmp (cookie_domain, host))
    return 1;


  /* HOST must match the tail of cookie_domain. */
  if (!cookie_match_tail (host, cookie_domain, 1))
    return 0;

  /* We know that COOKIE_DOMAIN is a subset of HOST; however, we must
     make sure that somebody is not trying to set the cookie for a
     subdomain shared by many entities.  For example, "company.co.uk"
     must not be allowed to set a cookie for ".co.uk".  On the other
     hand, "sso.redhat.de" should be able to set a cookie for
     ".redhat.de".

     The only marginally sane way to handle this I can think of is to
     reject on the basis of the length of the second-level domain name
     (but when the top-level domain is unknown), with the assumption
     that those of three or less characters could be reserved.  For
     example:

          .co.org -> works because the TLD is known
           .co.uk -> doesn't work because "co" is only two chars long
          .com.au -> doesn't work because "com" is only 3 chars long
          .cnn.uk -> doesn't work because "cnn" is also only 3 chars long (ugh)
          .cnn.de -> doesn't work for the same reason (ugh!!)
         .abcd.de -> works because "abcd" is 4 chars long
      .img.cnn.de -> works because it's not trying to set the 2nd level domain
       .cnn.co.uk -> works for the same reason

    That should prevent misuse, while allowing reasonable usage.  If
    someone knows of a better way to handle this, please let me
    know.  */
  {
    if (*p == '.')
      /* Ignore leading period in this calculation. */
      ++p;
    
    for (out = 0; !out; p++)
      switch (*p)
	{
	case '\0':
	  out = 1;
	  break;
	case '.':
	  if (ldcl == 0)
	    /* Empty domain component found -- the domain is invalid. */
	    return 0;
	  if (*(p + 1) == '\0')
	    {
	      /* Tolerate trailing '.' by not treating the domain as
		 one ending with an empty domain component.  */
	      out = 1;
	      break;
	    }
	  nldcl = ldcl;
	  ldcl  = 0;
	  ++dccount;
	  break;
	default:
	  ++ldcl;
	}

    

    if (dccount < 2)
      return 0;

    

    if (dccount == 2)
      {
	for (i = 0; i < countof (known_toplevel_domains); i++)
	  if (cookie_match_tail (cookie_domain, known_toplevel_domains[i], 1))
	    {
	      known_toplevel = 1;
	      break;
	    }
	if (!known_toplevel && nldcl <= 3)
	  return 0;
      }
  }


  /* Don't allow the host "foobar.com" to set a cookie for domain
     "bar.com".  */
  if (*cookie_domain != '.')
    {
      dlen = strlen (cookie_domain);
      hlen = strlen (host);
      /* cookie host:    hostname.foobar.com */
      /* desired domain:             bar.com */
      /* '.' must be here in host-> ^        */
      if (hlen > dlen && host[hlen - dlen - 1] != '.')
	return 0;
    }


  return 1;
}

static int cookie_path_matches (const char *, const char *);

/* Check whether PATH begins with COOKIE_PATH. */

static int
cookie_check_path_match (const char *cookie_path, const char *path)
{
  return cookie_path_matches (path, cookie_path);
}

/* Process the HTTP `Set-Cookie' header.  This results in storing the
   cookie or discarding a matching one, or ignoring it completely, all
   depending on the contents.  */

void
http_cookie_handle_set_cookie (struct cookie_jar *jar,
			  const char *host, int port,
			  const char *path, const char *set_cookie)
{
  struct cookie *cookie;
  char *trailing_slash=NULL;

  cookies_now = time (NULL);

  cookie = http_parse_set_cookies (set_cookie, http_set_cookie_field, 0);
  membuffer_assign(&cookie->host,host,strlen(host));
  if (!cookie)
    goto out;

  /* Sanitize parts of cookie. */

  if (cookie->domain.length ==0)
    {
    copy_domain:
      /* If the domain was not provided, we use the one we're talking
	 to, and set exact match.  */
      membuffer_assign(&cookie->domain ,"",1);//xstrdup (host);
    cookie->domain_exact = 0;  //cookie->domain_exact = 1;
      /* Set the port, but only if it's non-default. */
      if (port != 80 && port != 443)
	cookie->port = port;
    }
  else
    {
      if (!check_domain_match (cookie->domain.buf, host))
	{
	  printf (
		     "Cookie coming from %s attempted to set domain to %s\n",
		      (host),  (cookie->domain.buf));
//	  membuffer_destroly (&cookie->domain);
	  goto copy_domain;
	}
    }

  if (cookie->path.length ==0)
    {
      /* The cookie doesn't set path: set it to the URL path, sans the
	 file part ("/dir/file" truncated to "/dir/").  */
      trailing_slash = strrchr (path, '/');
      if (trailing_slash)
	membuffer_assign(&cookie->path,path,trailing_slash -path+1);
      else
	/* no slash in the string -- can this even happen? */
	membuffer_assign(&cookie->path, path,strlen(path));//xstrdup (path);
    }
  else
    {
      /* The cookie sets its own path; verify that it is legal. */
      if (!cookie_check_path_match (cookie->path.buf, path))
	{
	  printf ("Attempt to fake the path: %s, %s\n",
		   cookie->path.buf, path);
	  goto out;
	}
    }

  /* Now store the cookie, or discard an existing cookie, if
     discarding was requested.  */

  if (cookie->discard_requested)
    {
      discard_matching_cookie (jar, cookie);
      goto out;
    }

  http_store_cookie (jar, cookie);
  return;

 out:
  if (cookie)
    http_delete_cookie (cookie);
}

/* Support for sending out cookies in HTTP requests, based on
   previously stored cookies.  Entry point is
   `build_cookies_request'.  */
   
/* Return a count of how many times CHR occurs in STRING. */

static int
count_char (const char *string, char chr)
{
  const char *p;
  int count = 0;
  for (p = string; *p; p++)
    if (*p == chr)
      ++count;
  return count;
}

/* Find the cookie chains whose domains match HOST and store them to
   DEST.
*/
static int
cookie_find_chains_of_host (struct cookie_jar *jar, const char *host,
		     struct cookie *dest[])
{
  int dest_count = 0;
  int passes, passcnt;
  struct cookie *chain;
  ListNode *node = ListHead(&jar->chains);
  /* Bail out quickly if there are no cookies in the jar.  */
  if (node == NULL)
    return 0;

  if (numeric_address_p (host))
    /* If host is an IP address, only check for the exact match. */
    passes = 1;
  else
    /* Otherwise, check all the subdomains except the top-level (last)
       one.  As a domain with N components has N-1 dots, the number of
       passes equals the number of dots.  */
    passes = count_char (host, '.');

  passcnt = 0;

  /* Find chains that match HOST, starting with exact match and
     progressing to less specific domains.  For instance, given HOST
     fly.srk.fer.hr, first look for fly.srk.fer.hr's chain, then
     srk.fer.hr's, then fer.hr's.  */
//      struct cookie *chain = hash_table_get (jar->chains, host);
      node = ListHead (&jar->chains);    
      while(node != NULL)
      {
		chain = node->item;
		if(chain){
			if(!strcmp(chain->host.buf,host) ){
				dest[dest_count++] = chain;
			}
/*			if (++passcnt >= passes)
				break;
			 host = strchr (host, '.') + 1;*/
		}
		node = ListNext(&jar->chains,node);
      }

  return dest_count;
}

/* If FULL_PATH begins with PREFIX, return the length of PREFIX, zero
   otherwise.  */

static int
cookie_path_matches (const char *full_path, const char *prefix)
{
  int len;

  if (*prefix != '/')
    /* Wget's HTTP paths do not begin with '/' (the URL code treats it
       as a mere separator, inspired by rfc1808), but the '/' is
       assumed when matching against the cookie stuff.  */
    return 0;

//  ++prefix;
  len = strlen (prefix);

  if (0 != strncmp (full_path, prefix, len))
    /* FULL_PATH doesn't begin with PREFIX. */
    return 0;

  /* Length of PREFIX determines the quality of the match. */
  return len + 1;
}

/* Return non-zero iff COOKIE matches the provided parameters of the
   URL being downloaded: HOST, PORT, PATH, and SECFLAG.

   If PATH_GOODNESS is non-NULL, store the "path goodness" value
   there.  That value is a measure of how closely COOKIE matches PATH,
   used for ordering cookies.  */

static int
cookie_matches_url (const struct cookie *cookie,
		    const char *host, int port, const char *path,
		    int secflag, int *path_goodness)
{
  int pg;

  if (cookie_expired_p (cookie))
    /* Ignore stale cookies.  Don't bother unchaining the cookie at
       this point -- Wget is a relatively short-lived application, and
       stale cookies will not be saved by `save_cookies'.  On the
       other hand, this function should be as efficient as
       possible.  */
    return 0;

  if (cookie->secure && !secflag)
    /* Don't transmit secure cookies over insecure connections.  */
    return 0;
  if (cookie->port != PORT_ANY && cookie->port != port)
    return 0;

  /* If exact domain match is required, verify that cookie's domain is
     equal to HOST.  If not, assume success on the grounds of the
     cookie's chain having been found by cookie_find_chains_of_host.  */
  if (cookie->domain_exact
      && 0 != strcasecmp (host, cookie->domain.buf))
    return 0;

  pg = cookie_path_matches (path, cookie->path.buf);
  if (!pg)
    return 0;

  if (path_goodness)
    /* If the caller requested path_goodness, we return it.  This is
       an optimization, so that the caller doesn't need to call
       cookie_path_matches() again.  */
    *path_goodness = pg;
  return 1;
}

/* A structure that points to a cookie, along with the additional
   information about the cookie's "goodness".  This allows us to sort
   the cookies when returning them to the server, as required by the
   spec.  */

struct weighed_cookie {
  struct cookie *cookie;
  int domain_goodness;
  int path_goodness;
};

/* Comparator used for uniquifying the list. */

static int
http_compare_equal (const void *p1, const void *p2)
{
  struct weighed_cookie *wc1 = (struct weighed_cookie *)p1;
  struct weighed_cookie *wc2 = (struct weighed_cookie *)p2;

  int namecmp  = strcmp (wc1->cookie->attr.buf, wc2->cookie->attr.buf);
  int valuecmp = strcmp (wc1->cookie->value.buf, wc2->cookie->value.buf);

  /* We only really care whether both name and value are equal.  We
     return them in this order only for consistency...  */
  return namecmp ? namecmp : valuecmp;
}

/* Eliminate duplicate cookies.  "Duplicate cookies" are any two
   cookies with the same attr name and value.  Whenever a duplicate
   pair is found, one of the cookies is removed.  */

static int
eliminate_dups (struct weighed_cookie *outgoing, int count)
{
  struct weighed_cookie *h = NULL;	/* hare */
  struct weighed_cookie *t;	/* tortoise */
  struct weighed_cookie *end = outgoing + count;
  struct cookie *c0 = h[0].cookie;
  struct cookie *c1 = h[1].cookie;

  /* We deploy a simple uniquify algorithm: first sort the array
     according to our sort criteria, then copy it to itself, comparing
     each cookie to its neighbor and ignoring the duplicates.  */

  qsort (outgoing, count, sizeof (struct weighed_cookie), http_compare_equal);

  /* "Hare" runs through all the entries in the array, followed by
     "tortoise".  If a duplicate is found, the hare skips it.
     Non-duplicate entries are copied to the tortoise ptr.  */

  for (h = t = outgoing; h < end; h++)
    {
      if (h != end - 1)
	{
	  c0 = h[0].cookie;
	  c1 = h[1].cookie;
	  if (!strcmp (c0->attr.buf, c1->attr.buf) && !strcmp (c0->value.buf, c1->value.buf))
	    continue;		/* ignore the duplicate */
	}

      /* If the hare has advanced past the tortoise (because of
	 previous dups), make sure the values get copied.  Otherwise,
	 no copying is necessary.  */
      if (h != t)
	*t++ = *h;
      else
	t++;
    }
  return t - outgoing;
}

/* Comparator used for sorting by quality. */

static int
goodness_comparator (const void *p1, const void *p2)
{
 int dgdiff;
  int pgdiff;
  struct weighed_cookie *wc1 = (struct weighed_cookie *)p1;
  struct weighed_cookie *wc2 = (struct weighed_cookie *)p2;

  /* Subtractions take `wc2' as the first argument becauase we want a
     sort in *decreasing* order of goodness.  */
   dgdiff = wc2->domain_goodness - wc1->domain_goodness;
   pgdiff = wc2->path_goodness - wc1->path_goodness;

  /* Sort by domain goodness; if these are the same, sort by path
     goodness.  (The sorting order isn't really specified; maybe it
     should be the other way around.)  */
  return dgdiff ? dgdiff : pgdiff;
}

/* Generate a `Cookie' header for a request that goes to HOST:PORT and
   requests PATH from the server.  The resulting string is allocated
   with `malloc', and the caller is responsible for freeing it.  If no
   cookies pertain to this request, i.e. no cookie header should be
   generated, NULL is returned.  */

char *
http_cookie_header (struct cookie_jar *jar, const char *host,
	       int port, const char *path, int secflag)
{
  struct cookie **chains;
  int chain_count;

  struct cookie *cookie;
  struct weighed_cookie *outgoing;
  int count, i, ocnt;
  char *result;
  int result_size, pos;
  ListNode *node=NULL;
  int namlen ;
  int vallen ;
  struct cookie *c;
  /* First, find the cookie chains whose domains match HOST. */

  /* Allocate room for cookie_find_chains_of_host to write to.  The number of
     chains can at most equal the number of subdomains, hence
     1+<number of dots>.  */
  chains = alloca_array (struct cookie *, 10 + count_char (host, '.'));
  chain_count = cookie_find_chains_of_host (jar, host, chains);

  /* No cookies for this host. */
  if (!chain_count){
    free(chains);
    return NULL;
  }

  cookies_now = time (NULL);

  /* Now extract from the chains those cookies that match our host
     (for domain_exact cookies), port (for cookies with port other
     than PORT_ANY), etc.  See matching_cookie for details.  */

  /* Count the number of matching cookies. */
  count = 0;

   node = ListHead(&jar->chains);
  for (i = 0; i < chain_count; i++)
  {
      cookie = chains[i];
      if (cookie_matches_url (cookie, host, port, path, secflag, NULL))
	++count;
  }
  if (!count){
   free(chains);
    return NULL;		/* no cookies matched */
  }

  /* Allocate the array. */
  outgoing = alloca_array (struct weighed_cookie, count);

  ocnt = 0;

   node = ListHead(&jar->chains);
for (i = 0; i < chain_count; i++)
{
      cookie = node->item;
      node = ListNext(&jar->chains,node);
  int pg;
        if (!cookie_matches_url (cookie, host, port, path, secflag, &pg))
          continue;
        outgoing[ocnt].cookie = cookie;
        outgoing[ocnt].domain_goodness = cookie->domain.length;
        outgoing[ocnt].path_goodness   = pg;
        ++ocnt;
      }
  assert (ocnt == count);

  /* Eliminate duplicate cookies; that is, those whose name and value
     are the same.  */
  count = eliminate_dups (outgoing, count);

  /* Sort the array so that best-matching domains come first, and
     that, within one domain, best-matching paths come first. */
  qsort (outgoing, count, sizeof (struct weighed_cookie), goodness_comparator);

  /* Count the space the name=value pairs will take. */
  result_size = 0;
  for (i = 0; i < count; i++)
    {
      c = outgoing[i].cookie;
      /* name=value */
      result_size += c->attr.length + 1 + c->value.length;
    }

  /* Allocate output buffer:
     name=value pairs -- result_size
     "; " separators  -- (count - 1) * 2
     \0 terminator    -- 1  
    2 quotes for each cookie -- (count *2)*/
  //result_size = result_size + (count - 1) * 2 + 1 + (count*2);
  result_size = result_size + (count - 1) * 2 + 1;

  result = malloc (result_size);
  pos = 0;
  for (i = 0; i < count; i++)
    {
      c = outgoing[i].cookie;
      namlen = c->attr.length;
      vallen = c->value.length;

      memcpy (result + pos, c->attr.buf, namlen);
      pos += namlen;
      result[pos++] = '=';
      //result[pos++] = '"';
      memcpy (result + pos, c->value.buf, vallen);
      pos += vallen;
      //result[pos++] = '"';
      if (i < count - 1)
	{
	  result[pos++] = ';';
	  result[pos++] = ' ';
	}
    }
//memcpy (result + pos, c->version.buf, vallen);

  result[pos++] = '\0';
  assert (pos == result_size);

  free(chains);
  free(outgoing);

  return result;
}


/* If the region [B, E) ends with :<digits>, parse the number, return
   it, and store new boundary (location of the `:') to DOMAIN_E_PTR.
   If port is not specified, return 0.  */


#define GET_WORD(p, b, e) do {			\
  b = p;					\
  while (*p && *p != '\t')			\
    ++p;					\
  e = p;					\
  if (b == e || !*p)				\
    goto next;					\
  ++p;						\
} while (0)

/* Load cookies from FILE.  */

/* Mapper for save_cookies callable by hash_table_map.  VALUE points
   to the head in a chain of cookies.  The function prints the entire
   chain.  */



void
http_cookie_jar_delete (struct cookie_jar *jar)
{
  ListDestroy (&jar->chains, 1);
//  hash_table_destroy (jar->chains);
  free (jar);
}

char * http_get_cookie_header (struct cookie_jar *jar, const char *urlstr)
{
	uri_type url;
	char host[256];
	int port;

        http_FixStrUrl(( char * ) urlstr,strlen(urlstr),&url );
        strcpy(host,inet_ntoa(url.hostport.IPv4address.sin_addr));
        port =ntohs(url.hostport.IPv4address.sin_port);
//vivek        url.pathquery.buff[url.pathquery.size] ='\0';
	return (http_cookie_header (jar,host,port, url.pathquery.buff, 2));

}


void http_set_cookie_header (char *hdr_val, struct cookie_jar *jar, const char *urlstr)
{
    uri_type url;
    char host[256];
    int port;
    char *set_cookie = hdr_val;

    assert (jar != NULL);
    http_FixStrUrl(( char * ) urlstr,strlen(urlstr),&url );
    strcpy(host,inet_ntoa(url.hostport.IPv4address.sin_addr));
    port =ntohs(url.hostport.IPv4address.sin_port);
//vivek    url.pathquery.buff[url.pathquery.size] ='\0';
    http_cookie_handle_set_cookie (jar, host,port, url.pathquery.buff,  set_cookie);

 }


