#!/bin/sh
APPS_NAME="busybox"
if [ -e sysconfig.sh ]; then
	. sysconfig.sh
	. config.sh
	. model_config.sh
else
        echo "Application "$APPS_NAME" not configured"
        exit 0
fi                                                                                                                                       
busybox_always_on_config()
{
        echo "USING_CROSS_COMPILER=y" >> .config
        echo "CROSS_COMPILER_PREFIX=\"mips-linux-\"" >> .config
        if [ "$IFX_CONFIG_INETD" = "1" ]; then
                echo "CONFIG_INETD=y" >> .config
                echo "# CONFIG_FEATURE_INETD_SUPPORT_BILTIN_ECHO is not set" >> .config
                echo "# CONFIG_FEATURE_INETD_SUPPORT_BILTIN_DISCARD is not set" >> .config
                echo "# CONFIG_FEATURE_INETD_SUPPORT_BILTIN_TIME is not set" >> .config
                echo "# CONFIG_FEATURE_INETD_SUPPORT_BILTIN_DAYTIME is not set" >> .config
                echo "# CONFIG_FEATURE_INETD_SUPPORT_BILTIN_CHARGEN is not set" >> .config
        else
                echo "# CONFIG_INETD is not set" >> .config
        fi
        if [ "$IFX_CONFIG_DHCP_SERVER" = "1" ]; then
                echo "CONFIG_UDHCPD=y" >> .config
                echo "CONFIG_DUMPLEASES=y" >> .config
        else
                echo "# CONFIG_UDHCPD is not set" >> .config
        fi

        if [ "$IFX_CONFIG_DHCP_CLIENT" = "1" ]; then
                echo "CONFIG_UDHCPC=y" >> .config
        else
                echo "# CONFIG_UDHCPC is not set" >> .config
        fi

        if [ "$IFX_CONFIG_DHCP_SERVER" = "1"  -o "$IFX_CONFIG_DHCP_CLIENT" = "1" ]; then
                echo "CONFIG_FEATURE_UDHCP_SYSLOG=y" >> .config
                echo "# CONFIG_FEATURE_UDHCP_DEBUG is not set" >> .config
	fi

        if [ "$IFX_CONFIG_DHCP_RELAY" = "1" ]; then
                echo "CONFIG_UDHCPR=y" >> .config
        else
                echo "# CONFIG_UDHCPR is not set" >> .config
        fi

        if [ "$IFX_CONFIG_SYSTEM_LOG" = "1" ]; then
                echo "CONFIG_SYSLOGD=y" >> .config
                echo "CONFIG_FEATURE_ROTATE_LOGFILE=y" >> .config
                echo "CONFIG_FEATURE_REMOTE_LOG=y" >> .config
                echo "# CONFIG_FEATURE_IPC_SYSLOG is not set" >> .config
                echo "# CONFIG_KLOGD is not set" >> .config
                echo "# CONFIG_LOGGER is not set" >> .config
                echo "CONFIG_FEATURE_UDHCP_SYSLOG=y" >> .config

        else
                echo "# CONFIG_SYSLOGD is not set" >> .config
                echo "# CONFIG_LOGGER is not set" >> .config
                echo "# CONFIG_FEATURE_UDHCP_SYSLOG is not set" >> .config
        fi

        if [ "$IFX_CONFIG_TR69_DEVICE" = "1" ]; then
                echo "CONFIG_TR69_DEVICE=y" >> .config
        else
                echo "# CONFIG_TR69_DEVICE is not set" >> .config
        fi

        if [ "$IFX_CONFIG_TR111_DEVICE_ASSOCIATION" = "1" ]; then
                echo "CONFIG_TR111_DEVICE_ASSOCIATION=y" >> .config
        else
                echo "# CONFIG_TR111_DEVICE_ASSOCIATION is not set" >> .config
        fi
}

install_busybox_new()
{
        prefix=$1
        if [ "$prefix" = "" ]; then
                echo "No installation directory, aborting."
                exit 1;
        fi
        linkopts="-fs"
        h=`sort busybox.links | uniq`
        rm -f $prefix/bin/busybox2 || exit 1
        mkdir -p $prefix/bin || exit 1
        install -m 755 busybox $prefix/bin/busybox2 || exit 1

        for i in $h ; do
                appdir=`dirname $i`
                mkdir -p $prefix/$appdir || exit 1
                if [ "$2" = "--hardlinks" ]; then
                    bb_path="$prefix/bin/busybox2"
                else
                    case "$appdir" in
                        /)
                            bb_path="bin/busybox2"
                        ;;
                        /bin)
                            bb_path="busybox2"
                        ;;
                        /sbin)
                            bb_path="../bin/busybox2"
                        ;;
                        /usr/bin|/usr/sbin)
                            bb_path="../../bin/busybox2"
                        ;;
                        *)
                        echo "Unknown installation directory: $appdir"
                        exit 1
                        ;;
                    esac
                fi
                echo "  $prefix$i -> $bb_path"
                ln $linkopts $bb_path $prefix$i || exit 1
        done
}

create_busybox_config()
{
	#cp -f ifx_busybox_config .config
#608241:hsur add openswan code 	
	#if [ "${IFX_CONFIG_OPENSWAN}" == 1 ]; then    
	    #cp -f ifx_busybox_config_ipsec .config
    	    #chmod u+w .config	    
	#fi
	cp -f ifx_config .config
	chmod u+w .config

	# append options to config file
	if [ "$IFX_CONFIG_WIRELESS_ATH" == "1" ]; then
		echo "CONFIG_IWCONFIG=y" >> .config
	else
		echo "# CONFIG_IWCONFIG is not set" >> .config
	fi
	if [ "$IFX_CONFIG_TELNET_SERVER" = "1" ]; then
		echo "CONFIG_TELNETD=y" >> .config
		echo "CONFIG_FEATURE_TELNETD_INACTIVE_TIMEOUT=y" >> .config
		if [ "$IFX_CONFIG_INETD" = "1" ]; then
			echo "CONFIG_FEATURE_TELNETD_INETD=y" >> .config
		else
			echo "# CONFIG_FEATURE_TELNETD_INETD is not set" >> .config
		fi
	else
		echo "# CONFIG_TELNETD is not set" >> .config
	fi

	#if [ "$IFX_CONFIG_DNSRELAY" = "1" ];then
		#echo "CONFIG_DNSD=y" >> .config
	#else
		#echo "# CONFIG_DNSD is not set" >> .config
	#fi
	if [ "$IFX_CONFIG_TFTP_SERVER" = "1" ];then
		echo "CONFIG_TFTPD=y" >> .config
	else
		echo "# CONFIG_TFTPD is not set" >> .config
	fi

	if [ "$IFX_CONFIG_HTTPD_SSL_SUPPORT" = "1" ];then
        sed -i -e "s/.*CONFIG_HTTPD_SSL.*/CONFIG_HTTPD_SSL=y/" .config
    else
        sed -i -e "s/.*CONFIG_HTTPD_SSL.*/# CONFIG_SELINUX is not set/" .config
    fi
}

#--------------- main  function ----------------------------

display_info "----------------------------------------------------------------------"
display_info "-----------------------      build busybox    ------------------------"
display_info "----------------------------------------------------------------------"

parse_args $@

if [ $BUILD_CLEAN -eq 1 ]; then
	rm -f libiw.a
	make distclean
	rm -f .config
	rm -f .config_ok 
	[ ! $BUILD_CONFIGURE -eq 1 ] && exit 0
fi

if [ "$1" = "config_only" -a ! -f .config_ok -o $BUILD_CONFIGURE -eq 1 ]; then
	ln -sf ../wireless_tools.27/libiw.a .
	create_busybox_config
	make oldconfig 
	ifx_error_check $?
	echo -n > .config_ok
fi

if [ "$1" = "config_only" ] ; then
	exit 0
fi

make AR=${IFX_AR} AS=${IFX_AS} LD=${IFX_LD} NM=${IFX_NM} CC=${IFX_CC} BUILDCC=${IFX_HOSTCC} GCC=${IFX_CC} CXX=${IFX_CXX} CPP=${IFX_CPP} RANLIB=${IFX_RANLIB} IFX_CFLAGS="${IFX_CFLAGS}" IFX_LDFLAGS="${IFX_LDFLAGS}" all
ifx_error_check $? 

make PREFIX=${BUILD_ROOTFS_DIR} install 
ifx_error_check $? 
mv .config .config_busybox

make distclean
make allnoconfig
busybox_always_on_config
make oldconfig
ifx_error_check $?
cp .config .config_busybox2
make AR=${IFX_AR} AS=${IFX_AS} LD=${IFX_LD} NM=${IFX_NM} CC=${IFX_CC} BUILDCC=${IFX_HOSTCC} GCC=${IFX_CC} CXX=${IFX_CXX} CPP=${IFX_CPP} RANLIB=${IFX_RANLIB} IFX_CFLAGS="${IFX_CFLAGS}" IFX_LDFLAGS="${IFX_LDFLAGS}" all
ifx_error_check $?
install_busybox_new ${BUILD_ROOTFS_DIR}
make distclean
mv .config_busybox .config
make oldconfig

