/* options.h */
#ifndef _OPTIONS_H
#define _OPTIONS_H

#include "packet.h"
#include "ifx_amazon_cfg.h"
#define _IFX_API_INCLUDE_H
#include "ifx_common.h"

#define TYPE_MASK	0x0F

enum {
	OPTION_IP=1,
	OPTION_IP_PAIR,
	OPTION_STRING,
	OPTION_BOOLEAN,
	OPTION_U8,
	OPTION_U16,
	OPTION_S16,
	OPTION_U32,
	OPTION_S32
};

#define OPTION_REQ	0x10 /* have the client request this option */
#define OPTION_LIST	0x20 /* There can be a list of 1 or more of these */

struct dhcp_option {
	char name[10];
	char flags;
	uint8_t code;
};


#ifdef IFX_CONFIG_TR111_DEVICE_ASSOCIATION 
/* added for TR-111 */
#define VSI_ENTERPRISE_NO_DSL	3561
#define IFX_IPC_APP_ID_DHCP		211

#define OPT_VSI_ENTPNO_LEN 4
#define SUBOPT_CODE		0
#define SUBOPT_LEN  	1
#define SUBOPT_DATA 	2

typedef struct
{
   /* ID of the module sending the message */
   unsigned char ucFrom;
   /* ID of the module to which the message is addressed */
   unsigned char ucTo;
   /* ucTo may be just used for strict validation, since
      the message destined for a module will be in the
      module's FIFO anyway
    */
   /* Size of the Message */
   uint16_t unMsgSize;
   /* Reserved for future use */
   uint32_t uiReserved;

} x_IFX_IPC_MsgHdr;

#define IFX_IPC_HDR_SIZE sizeof(x_IFX_IPC_MsgHdr)
#define IFX_IPC_MAX_MSG_SIZE (4192 - IFX_IPC_HDR_SIZE)

typedef struct
{
   x_IFX_IPC_MsgHdr xHdr;

   /* Buffer to contain the message */
   char acMsg[IFX_IPC_MAX_MSG_SIZE];
   /* Each module is expected to write a union in the acMsg part */

} x_IFX_IPC_Msg;

#define IFX_DEVM_DHCP_PARAM_SIZE 65
extern char subopt1[IFX_DEVM_DHCP_PARAM_SIZE];
extern char subopt2[IFX_DEVM_DHCP_PARAM_SIZE];
extern char subopt3[IFX_DEVM_DHCP_PARAM_SIZE];
extern char subopt4[IFX_DEVM_DHCP_PARAM_SIZE];
extern char subopt5[IFX_DEVM_DHCP_PARAM_SIZE];
extern char subopt6[IFX_DEVM_DHCP_PARAM_SIZE];

#endif

extern const struct dhcp_option dhcp_options[];
extern const int option_lengths[];

uint8_t *get_option(struct dhcpMessage *packet, int code);
int end_option(uint8_t *optionptr);
int add_option_string(uint8_t *optionptr, uint8_t *string);
int add_simple_option(uint8_t *optionptr, uint8_t code, uint32_t data);
struct option_set *find_option(struct option_set *opt_list, char code);
void attach_option(struct option_set **opt_list, struct dhcp_option *option, char *buffer, int length);

/* added for TR-111 */
#ifdef IFX_CONFIG_TR111_DEVICE_ASSOCIATION 
uint8_t *get_vsi_option(uint8_t *pVSO, uint32_t iEnterpriseNo, uint8_t option_len);
int writenullmsg(uint32_t ipaddr);
int form_vsi_string(uint8_t *pVSD,char *string);
int writemsg(char *file, uint8_t *msg, uint8_t operation, uint32_t ipaddr);
void ifx_init_tr111_vars(void);
#endif
#endif
