/* Shared library add-on to iptables to add PTOS target support. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>

#include <iptables.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_TOS.h>

struct tosinfo {
	struct ipt_entry_target t;
	struct ipt_tos_target_info tos;
};

/* Function which prints out usage message. */
static void
help(void)
{
	unsigned int i;

	printf(
"PTOS target v%s options:\n"
"  --set-ptos value                   Set Type of Service field to one of the\n"
"                                following numeric.\n",
IPTABLES_VERSION);

}

static struct option opts[] = {
	{ "set-ptos", 1, 0, '1' },
	{ 0 }
};

/* Initialize the target. */
static void
init(struct ipt_entry_target *t, unsigned int *nfcache)
{
}

static void
parse_tos(const unsigned char *s, struct ipt_tos_target_info *info)
{
	unsigned int i, tos;

	if (string_to_number(s, 0, 255, &tos) != -1) {
		if ( (tos & 0xe0)!=0 )
		{
		    	info->tos = (u_int8_t )tos;
		    	return;
		}
	}
	exit_error(PARAMETER_PROBLEM, "Bad predence value `%s' for TOS ", s);
}

/* Function which parses command options; returns true if it
   ate an option */
static int
parse(int c, char **argv, int invert, unsigned int *flags,
      const struct ipt_entry *entry,
      struct ipt_entry_target **target)
{
	struct ipt_tos_target_info *tosinfo
		= (struct ipt_tos_target_info *)(*target)->data;

	switch (c) {
	case '1':
		if (*flags)
			exit_error(PARAMETER_PROBLEM,
			           "PTOS target: Cant specify --set-ptos twice");
		parse_tos(optarg, tosinfo);
		*flags = 1;
		break;

	default:
		return 0;
	}

	return 1;
}

static void
final_check(unsigned int flags)
{
	if (!flags)
		exit_error(PARAMETER_PROBLEM,
		           "PTOS target: Parameter --set-ptos is required");
}

static void
print_tos(u_int8_t tos, int numeric)
{
	unsigned int i;

	if (numeric) {
		printf("0x%02x ", tos);
	}
}

/* Prints out the targinfo. */
static void
print(const struct ipt_ip *ip,
      const struct ipt_entry_target *target,
      int numeric)
{
	const struct ipt_tos_target_info *tosinfo =
		(const struct ipt_tos_target_info *)target->data;
	printf("PTOS set ");
	print_tos(tosinfo->tos, numeric);
}

/* Saves the union ipt_targinfo in parsable form to stdout. */
static void
save(const struct ipt_ip *ip, const struct ipt_entry_target *target)
{
	const struct ipt_tos_target_info *tosinfo =
		(const struct ipt_tos_target_info *)target->data;

	printf("--set-ptos 0x%02x ", tosinfo->tos);
}

static
struct iptables_target ptos
= { NULL,
    "PTOS",
    IPTABLES_VERSION,
    IPT_ALIGN(sizeof(struct ipt_tos_target_info)),
    IPT_ALIGN(sizeof(struct ipt_tos_target_info)),
    &help,
    &init,
    &parse,
    &final_check,
    &print,
    &save,
    opts
};

void _init(void)
{
	register_target(&ptos);
}
