/* Shared library add-on to iptables to add tcp Dos Attacks matching support. */
#include <stdio.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>

#include <iptables.h>
#include <linux/netfilter_ipv4/ipt_ifxattack.h>

#define STR_CODE_REDI		"code_redI"
#define STR_CODE_REDII		"code_redII"
#define STR_LAND_ATTACK		"land_attack"
#define STR_ICQ_DOS			"icq_dos"
#define STR_MALFORMED_MIME	"malformed_mime"
#define STR_IIS_FRONTPAGE	"iis_frontpage"
#define STR_UDP_BOMB		"udp_bomb"
#define STR_FTP_PORT_REST		"ftp_port"

/* Function which prints out usage message. */
static void
help(void)
{
	printf(
"ifxattack match v%s options:\n"
"--name		attack	Match attack name.\n",
IPTABLES_VERSION);
}

static struct option opts[] = {
	{ "name", 1, 0, '1' },
	{0}
};

/* Initialize the match. */
static void
init(struct ipt_entry_match *m, unsigned int *nfcache)
{
	*nfcache |= NFC_IP_PROTO_UNKNOWN;
}

/* Function which parses command options; returns true if it
   ate an option */
static int
parse(int c, char **argv, int invert, unsigned int *flags,
      const struct ipt_entry *entry,
      unsigned int *nfcache,
      struct ipt_entry_match **match)
{
	struct ipt_ifxattack_match_info *attackinfo =
		(struct ipt_ifxattack_match_info *)(*match)->data;
	int iFound = 0;

	switch (c) {
	case '1':
		if (*flags)
			exit_error(PARAMETER_PROBLEM,
				   "Only one `--name' allowed");
		if (invert)
			exit_error(PARAMETER_PROBLEM,
				   "No '!' allowed");
#ifdef CONFIG_CODE_REDI
		if (strcmp(argv[optind - 1],STR_CODE_REDI) == 0) {
			attackinfo->attack_name = CODE_REDI;
			iFound = 1;
		} 
#endif
#ifdef CONFIG_CODE_REDII
		if (strcmp(argv[optind - 1],STR_CODE_REDII) == 0) {
			attackinfo->attack_name = CODE_REDII;
			iFound = 1;
		} 
#endif
#ifdef CONFIG_LAND_ATTACK
		if (strcmp(argv[optind - 1],STR_LAND_ATTACK) == 0) {
			attackinfo->attack_name = LAND_ATTACK;
			iFound = 1;
		} 
#endif
#ifdef CONFIG_ICQ_DOS
		if (strcmp(argv[optind - 1],STR_ICQ_DOS) == 0) {
			attackinfo->attack_name = ICQ_DOS;
			iFound = 1;
		} 
#endif
#ifdef CONFIG_MALFORMED_MIME
		if (strcmp(argv[optind - 1],STR_MALFORMED_MIME) == 0) {
			attackinfo->attack_name = MALFORMED_MIME;
			iFound = 1;
		} 
#endif

#ifdef CONFIG_IIS_FRONTPAGE
		if (strcmp(argv[optind - 1], STR_IIS_FRONTPAGE) == 0) {
			attackinfo->attack_name = IIS_FRONTPAGE;
			iFound = 1;
		}
#endif

#ifdef CONFIG_UDP_BOMB
		if (strcmp(argv[optind - 1], STR_UDP_BOMB) == 0) {
			attackinfo->attack_name = UDP_BOMB;
			iFound = 1;
		}
#endif
#ifdef CONFIG_FTP_PORT_REST
		if (strcmp(argv[optind - 1], STR_FTP_PORT_REST) == 0) {
			attackinfo->attack_name = FTP_PORT_REST;
			iFound = 1;
		}
#endif
		if (!iFound) {
			exit_error(PARAMETER_PROBLEM,
				   "Only one of attacks "
#ifdef CONFIG_CODE_REDI
				    "["STR_CODE_REDI"] " 
#endif
#ifdef CONFIG_CODE_REDI
				    "["STR_CODE_REDII"] " 
#endif
#ifdef CONFIG_LAND_ATTACK
				    "["STR_LAND_ATTACK"] " 
#endif
#ifdef CONFIG_ICQ_DOS
				    "["STR_ICQ_DOS"] "
#endif
#ifdef CONFIG_MALFORMED_MIME
					"["STR_MALFORMED_MIME"] "
#endif
#ifdef CONFIG_IIS_FRONTPAGE
					"["STR_IIS_FRONTPAGE"] "
#endif
#ifdef CONFIG_UDP_BOMB
					"["STR_UDP_BOMB"] "
#endif
#ifdef CONFIG_FTP_PORT_REST
					"["STR_FTP_PORT_REST"] "
#endif
				   	"allowed");
		}
		*flags = 1;
		break;
	default:
		return 0;
	}
	return 1;
}

/* Final check; must have specified --mss. */
static void
final_check(unsigned int flags)
{
	if (!flags)
		exit_error(PARAMETER_PROBLEM,
			   "ifxattack match: You must specify `--name name'");
}

/* Prints out the matchinfo. */
static void
print(const struct ipt_ip *ip,
      const struct ipt_entry_match *match,
      int numeric)
{
	struct ipt_ifxattack_match_info *attackinfo =
		(struct ipt_ifxattack_match_info *)match->data;

	printf("ifxattack match ");
#ifdef CONFIG_CODE_REDI
	if (attackinfo->attack_name == CODE_REDI)
		printf(STR_CODE_REDI);
#endif
#ifdef CONFIG_CODE_REDII
	if (attackinfo->attack_name == CODE_REDII)
		printf(STR_CODE_REDII);
#endif
#ifdef CONFIG_LAND_ATTACK
	if (attackinfo->attack_name == LAND_ATTACK)
		printf(STR_LAND_ATTACK);
#endif
#ifdef CONFIG_ICQ_DOS
	if (attackinfo->attack_name == ICQ_DOS)
		printf(STR_ICQ_DOS);
#endif
#ifdef CONFIG_MALFORMED_MIME
	if (attackinfo->attack_name == MALFORMED_MIME)
		printf(STR_MALFORMED_MIME);
#endif
#ifdef CONFIG_IIS_FRONTPAGE
	if (attackinfo->attack_name == IIS_FRONTPAGE)
		printf(STR_IIS_FRONTPAGE);
#endif
#ifdef CONFIG_UDP_BOMB
	if (attackinfo->attack_name == UDP_BOMB)
		printf(STR_UDP_BOMB);
#endif
#ifdef CONFIG_FTP_PORT_REST
	if (attackinfo->attack_name == FTP_PORT_REST)
		printf(STR_FTP_PORT_REST);
#endif
}

/* Saves the union ipt_matchinfo in parsable form to stdout. */
static void
save(const struct ipt_ip *ip, const struct ipt_entry_match *match)
{
	struct ipt_ifxattack_match_info *attackinfo =
		(struct ipt_ifxattack_match_info *)match->data;

	printf("--name ");
#ifdef CONFIG_CODE_REDI
	if (attackinfo->attack_name == CODE_REDI)
		printf(STR_CODE_REDI);
#endif
#ifdef CONFIG_CODE_REDII
	if (attackinfo->attack_name == CODE_REDII)
		printf(STR_CODE_REDII);
#endif
#ifdef CONFIG_LAND_ATTACK
	if (attackinfo->attack_name == LAND_ATTACK)
		printf(STR_LAND_ATTACK);
#endif
#ifdef CONFIG_ICQ_DOS
	if (attackinfo->attack_name == ICQ_DOS)
		printf(STR_ICQ_DOS);
#endif
#ifdef CONFIG_MALFORMED_MIME
	if (attackinfo->attack_name == MALFORMED_MIME)
		printf(STR_MALFORMED_MIME);
#endif
#ifdef CONFIG_IIS_FRONTPAGE
	if (attackinfo->attack_name == IIS_FRONTPAGE)
		printf(STR_IIS_FRONTPAGE);
#endif
#ifdef CONFIG_UDP_BOMB
	if (attackinfo->attack_name == UDP_BOMB)
		printf(STR_UDP_BOMB);
#endif
#ifdef CONFIG_FTP_PORT_REST
	if (attackinfo->attack_name == FTP_PORT_REST)
		printf(STR_FTP_PORT_REST);
#endif
}

static
struct iptables_match ifxattack
= { NULL,
    "ifxattack",
    IPTABLES_VERSION,
    IPT_ALIGN(sizeof(struct ipt_ifxattack_match_info)),
    IPT_ALIGN(sizeof(struct ipt_ifxattack_match_info)),
    &help,
    &init,
    &parse,
    &final_check,
    &print,
    &save,
    opts
};

void _init(void)
{
	register_match(&ifxattack);
}
