/* General includes */
#include <stdio.h>
#include <sys/param.h>
#include <string.h>
#include <sys/time.h>
#include <sys/utsname.h>
#include <unistd.h>

#if 1
#include <stdlib.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
//#include <linux/if.h>
#include <net/if.h>
#include <net/route.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <sys/queue.h>
#include <netinet/if_ether.h>
#include <sys/sysctl.h>
#endif //0

/* SNMP includes */
#include "asn1.h"
#include "snmp.h"
#include "agt_mib.h"
#include "agt_engine.h"
#include "if-mib.h"

#include <snmp_config.h>


#include "ifx_common.h"
#include "ifx_config.h"
#include "ifx_amazon_cfg.h"
#include "ifx_api_structs.h"
#include <interfaces.h>

#if 0
#define ifx_snmp_dbg(format, args...) printf(format, ##args)
#else
#define ifx_snmp_dbg(format, args...)
#endif

#define ip_addr_maskcmp(addr1, addr2, mask) ((addr1 & mask) == \
                                             (addr2 & mask))
#define LOG_ERR         3
#define MINLOADFREQ 0                     /* min reload frequency in seconds */

#define DEBUGMSGTL(x)

extern int read_write_flag;

unsigned long if_number = 0;
unsigned long if_number_count = 0;

unsigned long long_ret;
unsigned long long long_long_ret;
unsigned char char_ptr_ret[ 20 ];


typedef struct {
	unsigned long if_stack_status;
} IfStackEntry;


typedef struct {
	unsigned long if_test_id;
	unsigned long if_test_status;
	oid if_test_type;//ObjId
	unsigned long if_test_result;
	oid if_test_code;//ObjId
	unsigned char *if_test_owner;
} IfTestEntry;


typedef struct {
	unsigned long if_rcv_address_status;
	unsigned long if_rcv_address_type;
} IfRcvAddressEntry;

unsigned long if_table_last_change = 0;
unsigned long if_stack_last_change = 0;
// code copied from net-snmp
#define LOG_ERR         3
#define MINLOADFREQ 0                     /* min reload frequency in seconds */
#define IFNET_SYMBOL "ifnet"
static u_char  *if_list = 0;
static const u_char *if_list_end;
static size_t   if_list_size = 0;
static struct ifnet *ifnetaddr_list;
static struct ifnet *ifnetaddr, *saveifnet, *saveifnetaddr;
static char     saveName[16];
static int      saveIndex = 0;

static struct ifnet *ifnet_list_name = NULL;


/*
 * if_type_from_name
 * Return interface type using the interface name as a clue.
 * Returns 1 to imply "other" type if name not recognized.
 */
static int
if_type_from_name(const char *pcch)
{
    typedef struct _match_if {
        int             mi_type;
        const char     *mi_name;
    }              *pmatch_if, match_if;

    static match_if lmatch_if[] = {
        {24, "lo"},
        {6, "eth"},
        {9, "tr"},
        {23, "ppp"},
        {28, "sl"},
        {94, "MEI_PHY"},
        {124, "MEI_INTL"},
        {125, "MEI_FAST"},
        {0, 0}                  /* end of list */
    };
    int             ii, len;
    register pmatch_if pm;

    for (ii = 0, pm = lmatch_if; pm->mi_name; pm++) {
        len = strlen(pm->mi_name);
        if (0 == strncmp(pcch, pm->mi_name, len)) {
            return (pm->mi_type);
        }
    }
    return (1);                 /* in case search fails */
}

/**
* Determines network interface speed. It is system specific. Only linux
* realization is made.
*/
unsigned int getIfSpeed(int fd, struct ifreq ifr){
    unsigned int retspeed = 10000000;
/* the code is based on mii-diag utility by Donald Becker
* see ftp://ftp.scyld.com/pub/diag/mii-diag.c
*/
    ushort *data = (ushort *)(&ifr.ifr_data);
    unsigned phy_id;
    unsigned char new_ioctl_nums = 0;
    int mii_reg, i;
    ushort mii_val[32];
    ushort bmcr, bmsr, nway_advert, lkpar;
    const unsigned int media_speeds[] = {10000000, 10000000, 100000000, 100000000, 10000000, 0};
/* It corresponds to "10baseT", "10baseT-FD", "100baseTx", "100baseTx-FD", "100baseT4", "Flow-control", 0, */

    data[0] = 0;

    if (ioctl(fd, 0x8947, &ifr) >= 0) {
        new_ioctl_nums = 1;
    } else if (ioctl(fd, SIOCDEVPRIVATE, &ifr) >= 0) {
        new_ioctl_nums = 0;
    } else {
        DEBUGMSGTL(("mibII/interfaces", "SIOCGMIIPHY on %s failed\n", ifr.ifr_name));
        return retspeed;
    }
/* Begin getting mii register values */
    phy_id = data[0];
    for (mii_reg = 0; mii_reg < 8; mii_reg++){
        data[0] = phy_id;
        data[1] = mii_reg;
        if(ioctl(fd, new_ioctl_nums ? 0x8948 : SIOCDEVPRIVATE+1, &ifr) <0){
            DEBUGMSGTL(("mibII/interfaces", "SIOCGMIIREG on %s failed\n", ifr.ifr_name));
        }
        mii_val[mii_reg] = data[3];
    }
/*Parsing of mii values*/
/*Invalid basic mode control register*/
    if (mii_val[0] == 0xffff  ||  mii_val[1] == 0x0000) {
        DEBUGMSGTL(("mibII/interfaces", "No MII transceiver present!.\n"));
        return retspeed;
    }
    /* Descriptive rename. */
    bmcr = mii_val[0];    /*basic mode control register*/
    bmsr = mii_val[1];    /* basic mode status register*/
    nway_advert = mii_val[4]; /* autonegotiation advertisement*/
    lkpar = mii_val[5];       /*link partner ability*/

/*Check for link existence, returns 0 if link is absent*/
    if ((bmsr & 0x0016) != 0x0004){
        DEBUGMSGTL(("mibII/interfaces", "No link...\n"));
        retspeed = 0;
        return retspeed;
    }

    if(!(bmcr & 0x1000) ){
        DEBUGMSGTL(("mibII/interfaces", "Auto-negotiation disabled.\n"));
        retspeed = bmcr & 0x2000 ? 100000000 : 10000000;
        return retspeed;
    }
/* Link partner got our advertised abilities */
    if (lkpar & 0x4000) {
        int negotiated = nway_advert & lkpar & 0x3e0;
        int max_capability = 0;
        /* Scan for the highest negotiated capability, highest priority
           (100baseTx-FDX) to lowest (10baseT-HDX). */
        int media_priority[] = {8, 9, 7, 6, 5};     /* media_names[i-5] */
        for (i = 0; media_priority[i]; i++){
            if (negotiated & (1 << media_priority[i])) {
                max_capability = media_priority[i];
                break;
            }
        }
        if (max_capability)
            retspeed = media_speeds[max_capability - 5];
        else
            DEBUGMSGTL(("mibII/interfaces", "No common media type was autonegotiated!\n"));
    }else if(lkpar & 0x00A0){
        retspeed = (lkpar & 0x0080) ? 100000000 : 10000000;
    }
    return retspeed;
}

int GetIfNameFromIndex (short Index, char *Name)
{
    int skfd =0;
    struct ifreq ifr;
   /* Create a socket to the INET6 kernel. */
    if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        perror("socket");
        return;
    }

 memset(&ifr, 0, sizeof(ifr));
 ifr.ifr_ifindex = Index;

 if (ioctl(skfd, SIOCGIFNAME, &ifr) < 0) {

            return;

         } else {

             strcpy (Name,ifr.ifr_name);
             ifx_snmp_dbg("\nName of interface  =%s\n",Name);
         }

    close(skfd);

}

void GetIfIndex(char *savename,short *Index)
{

    int skfd =0;
    struct ifreq ifr;
  /* Create a socket to the INET6 kernel. */
 if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
 perror("socket");
           *Index = 0;
            return;
 }

 memset(&ifr, 0, sizeof(ifr));
 strcpy(ifr.ifr_name, savename);

 if (ioctl(skfd, SIOGIFINDEX, &ifr) < 0) {

           *Index = 0;
            return;

         } else {

             *Index = ifr.ifr_ifindex;
             ifx_snmp_dbg("\nName of interface  =%s\n",savename);
             ifx_snmp_dbg("\nIndex for %s interface  =%d\n",savename,*Index);
         }

    close(skfd);
}

void
Interface_Scan_Init(void)
{
    char            line[256], ifname_buf[64], *ifname, *ptr;
    struct ifreq    ifrq;
    struct ifnet  **ifnetaddr_ptr;
    FILE           *devin;
    unsigned long   rec_pkt, rec_oct, rec_err, rec_drop;
    unsigned long   snd_pkt, snd_oct, snd_err, snd_drop, coll;
    int             i, fd;
    conf_if_list   *if_ptr;
    const char     *scan_line_2_2 =
        "%lu %lu %lu %lu %*lu %*lu %*lu %*lu %lu %lu %lu %lu %*lu %lu";
    const char     *scan_line_2_0 =
        "%lu %lu %*lu %*lu %*lu %lu %lu %*lu %*lu %lu";
    const char     *scan_line_to_use;
    struct timeval et;                              /* elapsed time */


    saveIndex = 0;


    /*  disallow reloading of structures too often */
    gettimeofday ( &et, ( struct timezone * ) 0 );  /*  get time-of-day */
    if ( et.tv_sec < LastLoad + MINLOADFREQ ) {     /*  only reload so often */
      ifnetaddr = ifnetaddr_list;                   /*  initialize pointer */
      return;
    }
    LastLoad = et.tv_sec;

    /*
     * free old list:
     */
    while (ifnetaddr_list) {
        struct ifnet   *old = ifnetaddr_list;
        ifnetaddr_list = ifnetaddr_list->if_next;
        free(old->if_name);
        free(old->if_unit);
        free(old);
    }

    ifnetaddr = 0;
    ifnetaddr_ptr = &ifnetaddr_list;

    if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        DEBUGMSGTL(("snmpd",
                    "socket open failure in Interface_Scan_Init\n"));
        return; /** exit (1); **/
    }

    /*
     * build up ifnetaddr list by hand:
     */

    /*
     * at least linux v1.3.53 says EMFILE without reason...
     */
    if (!(devin = fopen("/proc/net/dev", "r"))) {
        close(fd);
        /* snmp_log(LOG_ERR, "cannot open /proc/net/dev - continuing...\n"); */
        return; /** exit (1); **/
    }

    i = 0;

    /*
     * read the second line (a header) and determine the fields we
     * should read from.  This should be done in a better way by
     * actually looking for the field names we want.  But thats too
     * much work for today.  -- Wes
     */
    fgets(line, sizeof(line), devin);
    fgets(line, sizeof(line), devin);
    if (strstr(line, "compressed")) {
        scan_line_to_use = scan_line_2_2;
        DEBUGMSGTL(("mibII/interfaces",
                    "using linux 2.2 kernel /proc/net/dev\n"));
    } else {
        scan_line_to_use = scan_line_2_0;
        DEBUGMSGTL(("mibII/interfaces",
                    "using linux 2.0 kernel /proc/net/dev\n"));
    }


    while (fgets(line, sizeof(line), devin)) {
        struct ifnet   *nnew;
        char           *stats, *ifstart = line;

        if (line[strlen(line) - 1] == '\n')
            line[strlen(line) - 1] = '\0';

        while (*ifstart && *ifstart == ' ')
            ifstart++;

        if (!*ifstart || ((stats = strrchr(ifstart, ':')) == NULL)) {
            /*snmp_log(LOG_ERR,
                     "/proc/net/dev data format error, line ==|%s|", line); */
            continue;
        }
        if ((scan_line_to_use == scan_line_2_2) && ((stats - line) < 6)) {
            /* snmp_log(LOG_ERR,
                     "/proc/net/dev data format error, line ==|%s|", line); */
        }

        *stats   = 0;
        strncpy(ifname_buf, ifstart, sizeof(ifname_buf));
        ifname_buf[ sizeof(ifname_buf)-1 ] = 0;
        *stats++ = ':';
        while (*stats == ' ')
            stats++;

        if ((scan_line_to_use == scan_line_2_2 &&
             sscanf(stats, scan_line_to_use, &rec_oct, &rec_pkt, &rec_err,
                    &rec_drop, &snd_oct, &snd_pkt, &snd_err, &snd_drop,
                    &coll) != 9) || (scan_line_to_use == scan_line_2_0
                                     && sscanf(stats, scan_line_to_use,
                                               &rec_pkt, &rec_err,
                                               &snd_pkt, &snd_err,
                                               &coll) != 5)) {
            if ((scan_line_to_use == scan_line_2_2)
                && !strstr(line, "No statistics available"))
                /* snmp_log(LOG_ERR,
                         "/proc/net/dev data format error, line ==|%s|",
                         line); */
            continue;
        }

        nnew = (struct ifnet *) calloc(1, sizeof(struct ifnet));
        if (nnew == NULL)
            break;              /* alloc error */

        /*
         * chain in:
         */
        *ifnetaddr_ptr = nnew;
        ifnetaddr_ptr = &nnew->if_next;
        i++;

        /*
         * linux previous to 1.3.~13 may miss transmitted loopback pkts:
         */
        if (!strcmp(ifname_buf, "lo") && rec_pkt > 0 && !snd_pkt)
            snd_pkt = rec_pkt;

        nnew->if_ipackets = rec_pkt;
        nnew->if_ierrors = rec_err;
        nnew->if_opackets = snd_pkt;
        nnew->if_oerrors = snd_err;
        nnew->if_collisions = coll;
        if (scan_line_to_use == scan_line_2_2) {
            nnew->if_ibytes = rec_oct;
            nnew->if_obytes = snd_oct;
            nnew->if_iqdrops = rec_drop;
            nnew->if_snd.ifq_drops = snd_drop;
        } else {
            nnew->if_ibytes = rec_pkt * 308;
            nnew->if_obytes = snd_pkt * 308;
        }

        /*
         * ifnames are given as ``   eth0'': split in ``eth'' and ``0'':
         */
        for (ifname = ifname_buf; *ifname && *ifname == ' '; ifname++);

        /*
         * set name and interface# :
         */
        nnew->if_name = (char *) strdup(ifname);
        for (ptr = nnew->if_name; *ptr && (*ptr < '0' || *ptr > '9');
             ptr++);
        nnew->if_unit = strdup(*ptr ? ptr : "");
        *ptr = 0;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        if (ioctl(fd, SIOCGIFADDR, &ifrq) < 0)
            memset((char *) &nnew->if_addr, 0, sizeof(nnew->if_addr));
        else
            nnew->if_addr = ifrq.ifr_addr;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        if (ioctl(fd, SIOCGIFBRDADDR, &ifrq) < 0)
            memset((char *) &nnew->ifu_broadaddr, 0,
                   sizeof(nnew->ifu_broadaddr));
        else
            nnew->ifu_broadaddr = ifrq.ifr_broadaddr;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        if (ioctl(fd, SIOCGIFNETMASK, &ifrq) < 0)
            memset((char *) &nnew->ia_subnetmask, 0,
                   sizeof(nnew->ia_subnetmask));
        else
            nnew->ia_subnetmask = ifrq.ifr_netmask;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        nnew->if_flags = ioctl(fd, SIOCGIFFLAGS, &ifrq) < 0
            ? 0 : ifrq.ifr_flags;

        nnew->if_type = 0;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        if (ioctl(fd, SIOCGIFHWADDR, &ifrq) < 0)
            memset(nnew->if_hwaddr, (0), 6);
        else {
            memcpy(nnew->if_hwaddr, ifrq.ifr_hwaddr.sa_data, 6);

#ifdef ARPHRD_LOOPBACK
            switch (ifrq.ifr_hwaddr.sa_family) {
            case ARPHRD_ETHER:
                nnew->if_type = 6;
                break;
            case ARPHRD_TUNNEL:
            case ARPHRD_TUNNEL6:
#ifdef ARPHRD_IPGRE
            case ARPHRD_IPGRE:
#endif
            case ARPHRD_SIT:
                nnew->if_type = 131;
                break;          /* tunnel */
            case ARPHRD_SLIP:
            case ARPHRD_CSLIP:
            case ARPHRD_SLIP6:
            case ARPHRD_CSLIP6:
                nnew->if_type = 28;
                break;          /* slip */
            case ARPHRD_PPP:
                nnew->if_type = 23;
                break;          /* ppp */
            case ARPHRD_LOOPBACK:
                nnew->if_type = 24;
                break;          /* softwareLoopback */
            case ARPHRD_FDDI:
                nnew->if_type = 15;
                break;
            case ARPHRD_ARCNET:
                nnew->if_type = 35;
                break;
            case ARPHRD_LOCALTLK:
                nnew->if_type = 42;
                break;
#ifdef ARPHRD_HIPPI
            case ARPHRD_HIPPI:
                nnew->if_type = 47;
                break;
#endif
#ifdef ARPHRD_ATM
            case ARPHRD_ATM:
                nnew->if_type = 37;
                break;
#endif
                /*
                 * XXX: more if_arp.h:ARPHDR_xxx to IANAifType mappings...
                 */
            }
#endif
        }

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        nnew->if_metric = ioctl(fd, SIOCGIFMETRIC, &ifrq) < 0
            ? 0 : ifrq.ifr_metric;

#ifdef SIOCGIFMTU
        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        nnew->if_mtu = (ioctl(fd, SIOCGIFMTU, &ifrq) < 0)
            ? 0 : ifrq.ifr_mtu;
#else
        nnew->if_mtu = 0;
#endif

        for (if_ptr = conf_list; if_ptr; if_ptr = if_ptr->next)
            if (!strcmp(if_ptr->name, ifname))
                break;

        if (if_ptr) {
            nnew->if_type = if_ptr->type;
            nnew->if_speed = if_ptr->speed;
        } else {
            /*
             * do only guess if_type from name, if we could not read
             * * it before from SIOCGIFHWADDR
             */
            if (!nnew->if_type)
                nnew->if_type = if_type_from_name(nnew->if_name);
            nnew->if_speed = nnew->if_type == 6 ? getIfSpeed(fd, ifrq) :
                nnew->if_type == 24 ? 10000000 :
                nnew->if_type == 9 ? 4000000 : 0;
            /*Zero speed means link problem*/
            if(nnew->if_speed == 0 && nnew->if_flags & IFF_UP){
                nnew->if_flags &= ~IFF_RUNNING;
            }
        }

    }                           /* while (fgets ... */

    ifnetaddr = ifnetaddr_list;
#if 0
    if (snmp_get_do_debugging()) {
        {
            struct ifnet   *x = ifnetaddr;
            DEBUGMSGTL(("mibII/interfaces", "* see: known interfaces:"));
            while (x) {
                DEBUGMSG(("mibII/interfaces", " %s", x->if_name));
                x = x->if_next;
            }
            DEBUGMSG(("mibII/interfaces", "\n"));
        }                       /* XXX */
    }
#endif //0

    fclose(devin);
    close(fd);
}
#if 1
/*
 * **  4.2 BSD doesn't have ifaddr
 * **
 */
int
Interface_Scan_Next(short *Index,
                    char *Name,
                    struct ifnet *Retifnet, struct in_ifaddr *dummy)
{
    struct ifnet    ifnet;
    register char  *cp;

    while (ifnetaddr) {
        /*
         *      Get the "ifnet" structure and extract the device name
         */
        ifnet = *ifnetaddr;
		ifx_snmp_dbg ("[%s]:[%d] ifnetaddr:[0x%8x]\n", __FUNCTION__, __LINE__, ifnetaddr);
        strncpy(saveName, ifnet.if_name, sizeof(saveName));
        if (strcmp(saveName, "ip") == 0) {
            ifnetaddr = ifnet.if_next;
            continue;
        }
        saveName[sizeof(saveName) - 1] = '\0';
        cp = (char *) strchr(saveName, '\0');
        strncat(cp, ifnet.if_unit, sizeof(saveName)-strlen(saveName)-1);
        saveName[sizeof(saveName) - 1] = '\0';
        if (1 || strcmp(saveName, "lo0") != 0) {        /* XXX */
#if 0
#if 1
// Modification made for Amazon
           if (Index)
              GetIfIndex(saveName,Index);
#else
            if (Index)
                *Index = ++saveIndex;
#endif // 0
#endif // 0
            if (Retifnet)
                *Retifnet = ifnet;
            if (Name)
                strcpy(Name, saveName);
            saveifnet = &ifnet;
            saveifnetaddr = ifnetaddr;
            ifnetaddr = ifnet.if_next;
            return (1);         /* DONE */
        }
        ifnetaddr = ifnet.if_next;
    }
    return (0);                 /* EOF */
}

int
Interface_Scan_By_Index(int Index,
                        char *Name,
                        struct ifnet *Retifnet,
                        struct in_ifaddr *Retin_ifaddr)
{
    short           i;

    Interface_Scan_Init();
    while (Interface_Scan_Next(&i, Name, Retifnet, Retin_ifaddr)) {
        if (i == Index)
            break;
    }
    if (i != Index)
        return (-1);            /* Error, doesn't exist */
    return (0);                 /* DONE */
}
#endif // 0


#if 1
struct ifnet *Get_Ifnet_From_Addr (struct in_addr *dest)
{
    struct ifnet    *ifnet;
    unsigned int addr, mask;

    Interface_Scan_Init();

    while (ifnetaddr) {
        /*
         *      Get the "ifnet" structure and extract the device name
         */
        ifnet = ifnetaddr;
	addr = ifnet->if_addr.sa_data;
	mask = ifnet->ia_subnetmask.sa_data;
	if (!ip_addr_maskcmp (dest->s_addr, addr, mask)){
            ifnetaddr = ifnet->if_next;
            continue;
	}
    	ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
            return ifnet;         /* DONE */
    }
    ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
    return NULL;
	

}
#endif

#if 1
struct ifnet *Get_Ifnet (short Index, char *Name)
{
    struct ifnet    *ifnet;
	char   *cp;
	char   saveName[16];

    Interface_Scan_Init();

	ifnet = ifnetaddr;
    while (ifnet) {
        /*
         *      Get the "ifnet" structure and extract the device name
         */
		
        strncpy(saveName, ifnet->if_name, sizeof(saveName));
		strcat(saveName, ifnet->if_unit);
		ifx_snmp_dbg ("saveName %s ifnet->if_unit %s\n", saveName, ifnet->if_unit);
        if ((strcmp(saveName, Name)) != 0) {
            ifnet = ifnet->if_next;
            continue;
        }
    	    ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
            return ifnet;         /* DONE */
    }
    ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
    return NULL;
}
#else
struct ifnet *Get_Ifnet (short Index, char *Name)
{
	struct ifnet ifnet_entry;
	struct ifnet *ifnet;
	
	ifx_snmp_dbg ("in file %s, fxn %s line %d index %d name %s\n", __FILE__, __FUNCTION__, __LINE__, Index, Name);
	Interface_Scan_By_Index (&Index, Name, &ifnet_entry, NULL);
	ifnet = &ifnet_entry;
	ifx_snmp_dbg ("ifnet %x\n", ifnet);
	return ifnet;
    
}
#endif //0

static int      Interface_Count = 0;
static time_t   scan_time = 0;

int
Interface_Scan_Get_Count(void)
{
    time_t          time_now = time(NULL);
    if (!Interface_Count || (time_now > scan_time + 60)) {
        scan_time = time_now;
        Interface_Scan_Init();
        Interface_Count = 0;
        while (Interface_Scan_Next(NULL, NULL, NULL, NULL) != 0) {
            Interface_Count++;
        }
    }
    // This is mem fix
    while (ifnetaddr_list) {
        struct ifnet   *old = ifnetaddr_list;
        ifnetaddr_list = ifnetaddr_list->if_next;
        free(old->if_name);
        free(old->if_unit);
        free(old);
    }

    return (Interface_Count);
}


int Set_Interface_Flag (char *Name, unsigned long action)
{
	int skfd =0;
    struct ifreq ifr;

	memset(&ifr, 0x0, sizeof(ifr));

    if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        perror("socket");
        return -1;
    }
	strncpy (ifr.ifr_name, Name, IFNAMSIZ);
	ioctl(skfd, SIOCGIFFLAGS, &ifr);
	
	if (action == 1) {
		ifr.ifr_flags |= IFF_UP; 
		ioctl(skfd, SIOCSIFFLAGS, &ifr);
	}
	else if (action == 2) {
		ifr.ifr_flags &= ~IFF_UP; 
		ioctl(skfd, SIOCSIFFLAGS, &ifr);
	}
	else {
		close (skfd);
		return -1;
	}
	close (skfd);
	return 0;
}

/* IF_MIB initialisation (must also register the MIB module tree) */
void init_IF_MIB()
{

	register_subtrees_of_IF_MIB();

}



/* The number of network interfaces (regardless of their
 * current state) present on this system.
 */

unsigned char *
var_ifNumber(int *var_len, snmp_info_t *mesg,
        int (**write_method)())
{
	/* Add value computations */
//	if_number = cyg_snmp_num_interfaces();
	if_number = Interface_Scan_Get_Count ();

	/* Set size (in bytes) and return address of the variable */
	*var_len = sizeof(long);
	return (unsigned char *) (& if_number);
}

/* The value of sysUpTime at the time of the last creation or
 * deletion of an entry in the ifTable.  If the number of
 * entries has been unchanged since the last re-initialization
 * of the local network management subsystem, then this object
 * contains a zero value.
 */

unsigned char *
var_ifTableLastChange(int *var_len, snmp_info_t *mesg,
        int (**write_method)())
{
	/* Add value computations */

	/* Set size (in bytes) and return address of the variable */
	*var_len = sizeof(long);
	return (unsigned char *) (& if_table_last_change);
}

#if 1
//added by Anath
//int //int was replaced by unsigned char * by imra for testing
int write_ifAdminStatus(int action,
        unsigned char *var_val, unsigned char varval_type, int var_val_len,
        unsigned char *statP, Oid *reqoid)
{
	int i = 0, initFlag = 0;
	unsigned long long_int = 0;
	struct ifnet *ifnet;
	char Name[16];
	unsigned long temp_int = 0;
	register int interface;

	if_number_count = Interface_Scan_Get_Count ();
	interface = reqoid->name[reqoid->namelen - 1];
	ifx_snmp_dbg ("[%s]:[%d] ifnumber:[%d] interface:[%d]\n", __FUNCTION__, __LINE__, if_number_count, interface);	
	initFlag = 1;
	Interface_Scan_Init();
	if (ifnet_list_name == NULL) {
		initFlag = 0;
        ifnet_list_name = ifnetaddr;
		ifx_snmp_dbg ("in file %s, fxn %s line %d ifnetaddr:[0x%8x]\n",__FILE__,__FUNCTION__,__LINE__,ifnetaddr);
	}
	i = 1;
	if (initFlag) {
		for ( ifnet = ifnetaddr; ifnet; ifnet = ifnet->if_next) {
			if (i == interface) {
				ifnet_list_name = ifnet;
				break;
			}
			i++;
			continue;
		}
	}
    ifx_snmp_dbg("[%s]:[%d] ifnet_list_name =%s\n",__FUNCTION__, __LINE__, ifnet_list_name->if_name);
	strncpy(Name, ifnet_list_name->if_name, sizeof(Name));
	strcat(Name, ifnet_list_name->if_unit);
    ifx_snmp_dbg("[%s]:[%d] name =%s\n",__FUNCTION__, __LINE__,Name);

	for(i = 0; i < var_val_len; i++)
	{
		temp_int <<= sizeof(char) * 8;
		temp_int += var_val[i];
	}

	switch (action) {
	case RESERVE1:
		return 0;
	case RESERVE2:
		return 0;
	case COMMIT:
		long_int = ntohl(temp_int);
		if (long_int == 1) {
			Set_Interface_Flag (Name, long_int);
		} else if (long_int == 2) {
			Set_Interface_Flag (Name, long_int);
		}	
		return 0;
		
	case ACTION:
		return 0;
	case FREE:
		return 0;
	}
}
#endif

static struct ifnet *ifnet_list = NULL;

/* An entry containing management information applicable to a
 * particular interface.
 */

unsigned char *
var_ifEntry(int *var_len,
        Oid *newoid, Oid *reqoid, int searchType,
        snmp_info_t *mesg, int (**write_method)())
{
	/* Last subOID of COLUMNAR OID is column */
	int	column = newoid->name[(newoid->namelen - 1)];
	int	result =1, initFlag = 0, i =0;

#if 0
    while (0 /* Test whether the entry has been found */) {
        /* Add indexes for the entry to OID newname */
        /* Determine whether it is the requested OID  */
        result = compare(reqoid, newoid);
        if (((searchType == EXACT) && (result != 0)) ||
                ((searchType == NEXT) && (result >= 0))) {
            return NO_MIBINSTANCE;
        }
    }
    /* Place here the computation functionality */
#endif
#if 1
	register int interface;
//	IfEntry if_entry;
	//struct netif *netif_entry;
	//if_number = cyg_snmp_num_interfaces();
	if_number = Interface_Scan_Get_Count ();
	struct ifnet *ifnet_entry, *ifnet;
	char Name[16];

	ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
	
	interface = reqoid->name[(newoid->namelen)];
	ifx_snmp_dbg ("in file %s, fxn %s line %d interface:%d reqoid:%x newoid->namelen:%d\n", __FILE__, __FUNCTION__, __LINE__, interface, reqoid->name, newoid->namelen);
#if 0
	result = compare(reqoid, newoid); 
	ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);

	if (result <= 0) {
		ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
		return NO_MIBINSTANCE;
	}
#endif //0 commented by Santosh
	ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
	if (searchType == NEXT) {
		  
		initFlag = 1;
	    Interface_Scan_Init();
		if (ifnet_list == NULL) {
				initFlag = 0;
        	   ifnet_list = ifnetaddr;
			   ifx_snmp_dbg ("in file %s, fxn %s line %d ifnetaddr:[0x%8x]\n",__FILE__,__FUNCTION__,__LINE__,ifnetaddr);
		}
		i = 0;
		if (initFlag) {
			for ( ifnet = ifnetaddr; ifnet; ifnet = ifnet->if_next) {
				if (i == interface) {
					ifnet_list = ifnet;
					break;
				}
				i++;
				continue;
			}
		}
		for (interface += 1; interface <= if_number; interface++) {
#if 0
 			GetIfNameFromIndex (interface, Name);
			if ((ifnet_entry = Get_Ifnet(interface, Name)) != NULL) {
				break; 
			}
#endif
			ifx_snmp_dbg ("in file %s, fxn %s line %d ifnet:[%8x]\n", __FILE__, __FUNCTION__, __LINE__, ifnet_list);
			ifnet_entry = ifnet_list;
			ifx_snmp_dbg ("in file %s, fxn %s line %d ifnet:[%8x]\n", __FILE__, __FUNCTION__, __LINE__, ifnet_list);
			 break;
		}
	} else {
		// netif_entry = cyg_snmp_get_if(interface);
 		GetIfNameFromIndex (interface, Name);
		ifnet_entry = Get_Ifnet (interface, Name);
		ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
	}
	ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);

	if (interface > if_number) {
		ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
		return NO_MIBINSTANCE;
	}

	ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
	newoid->name[newoid->namelen] = interface;
	newoid->name[newoid->namelen + 1] = 0;
	newoid->namelen += 1;

#endif

	/* Set write-function */
	*write_method = 0;
	/* Set size (in bytes) and return address of the variable */
	*var_len = sizeof(long);
	switch (column) {
	case I_ifIndex:
		long_ret = interface;
		return (unsigned char *) (& long_ret);
	case I_ifDescr:
		// sprintf(char_ptr_ret, "%c%c%d", netif_entry->name[0], netif_entry->name[1], netif_entry->num);
		ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
		sprintf(char_ptr_ret, "%s%s", ifnet_entry->if_name, ifnet_entry->if_unit);
		*var_len = strlen(char_ptr_ret);
		return (unsigned char *) (char_ptr_ret);
	case I_ifType:
//		if_entry.if_type = netif_entry->link_type;
#if 0
		if (strcmp(netif_entry->name, "na") == 0) {
			long_ret = 37;
		} else if (strcmp(netif_entry->name, "et") == 0) {
			long_ret = 6;
		} else if (strcmp(netif_entry->name, "lo") == 0) {
			long_ret = 24;
		} else {
			long_ret = 1;
		}
		return (unsigned char *) (& long_ret);
#endif //0
		if (strcmp(ifnet_entry->if_name, "nas") == 0) {
			long_ret = 37;
		} else if (strcmp(ifnet_entry->if_name, "eth") == 0) {
			long_ret = 6;
		} else if (strcmp(ifnet_entry->if_name, "lo") == 0) {
			long_ret = 24;
		} else if (strcmp(ifnet_entry->if_name, "sit") == 0) {
			long_ret = 131;
		}else {
			long_ret = 1;
		}
		ifx_snmp_dbg ("in file %s, fxn %s line %d\n", __FILE__, __FUNCTION__, __LINE__);
		return (unsigned char *) (& long_ret);
	case I_ifMtu:
		long_ret = ifnet_entry->if_mtu;
		return (unsigned char *) (& long_ret);
	case I_ifSpeed:
		long_ret = 100000000;
		return (unsigned char *) (& long_ret);
	case I_ifPhysAddress:
		*var_len = sizeof(ifnet_entry->if_hwaddr);
//		if_entry.if_phys_address = netif_entry->hwaddr;
		return (unsigned char *) (ifnet_entry->if_hwaddr);
	case I_ifAdminStatus:
		if (read_write_flag == 3) {
			*write_method = write_ifAdminStatus;
		}
		long_ret = (ifnet_entry->if_flags & IFF_UP ? 1 : 2);
		return (unsigned char *) (& long_ret);
	case I_ifOperStatus:
		long_ret = (ifnet_entry->if_flags & IFF_UP ? 1 : 2);
		return (unsigned char *) (& long_ret);
	case I_ifLastChange:
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_entry.if_last_change;
		// return (unsigned char *) (& long_ret);
		long_ret = 0;
		return (unsigned char *) (& long_ret);
	case I_ifInOctets:
		long_ret = ifnet_entry->if_ibytes;
		return (unsigned char *) (& long_ret);
	case I_ifInUcastPkts:
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_entry.if_in_ucast_pkts;
		//return (unsigned char *) (& long_ret);
		long_ret = 0;
		return (unsigned char *) (& long_ret);
	case I_ifInNUcastPkts:
		long_ret = 0;
		return (unsigned char *) (& long_ret);
	case I_ifInDiscards:
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_entry.if_in_discards;
		//return (unsigned char *) (& long_ret);
		long_ret = 0;
		return (unsigned char *) (& long_ret);
	case I_ifInErrors:
		long_ret = ifnet_entry->if_ierrors;
		return (unsigned char *) (& long_ret);
	case I_ifInUnknownProtos:
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_entry.if_in_unknown_protos;
		//return (unsigned char *) (& long_ret);
		long_ret = 0;
		return (unsigned char *) (& long_ret);
	case I_ifOutOctets:
		long_ret = ifnet_entry->if_obytes;
		return (unsigned char *) (& long_ret);
	case I_ifOutUcastPkts:
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_entry.if_out_ucast_pkts;
		//return (unsigned char *) (& long_ret);
		long_ret = 0;
		return (unsigned char *) (& long_ret);
	case I_ifOutNUcastPkts:
		long_ret = 0;
		return (unsigned char *) (& long_ret);
	case I_ifOutDiscards:
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_entry.if_out_discards;
		//return (unsigned char *) (& long_ret);
		long_ret = 0;
		return (unsigned char *) (& long_ret);
	case I_ifOutErrors:
		long_ret = ifnet_entry->if_oerrors;
		return (unsigned char *) (& long_ret);
	case I_ifOutQLen:
		long_ret = 0;
		return (unsigned char *) (& long_ret);
	case I_ifSpecific:
		strncpy(char_ptr_ret, "00", 2);
		*var_len = 2;
		return (unsigned char *) (char_ptr_ret);
	default:
		return NO_MIBINSTANCE;
	}
}

#if 1
//added by Anath
//int //int was replaced by unsigned char * by imra for testing
int write_ifLinkUpDownTrapEnable(int action,
        unsigned char *var_val, unsigned char varval_type, int var_val_len,
        unsigned char *statP, Oid *reqoid)
{
	int i;
//	IfXEntry if_x_entry;
	// struct netif *netif_entry;
	struct ifnet *ifnet_entry;
	char Name[16];
	unsigned long temp_int = 0;
	int interface;

	interface = reqoid->name[reqoid->namelen - 1];
 	GetIfNameFromIndex (interface, Name);
	ifnet_entry = Get_Ifnet (interface, Name);
	// netif_entry = cyg_snmp_get_if(interface);

	for(i = 0; i < var_val_len; i++)
	{
		temp_int <<= sizeof(char) * 8;
		temp_int += var_val[i];
	}

	switch (action) {
	case RESERVE1:
		return 0;
	case RESERVE2:
		return 0;
	case COMMIT:
		
		// FIXME: Santosh, not supported in ifnet struct
		// netif_entry->if_xentry.if_link_up_down_trap_enable = ntohl(temp_int);
		return 0;
	case ACTION:
		return 0;
	case FREE:
		return 0;
	}
}

//added by Anath
//int //int was replaced by unsigned char * by imra for testing
int write_ifPromiscuousMode(int action,
        unsigned char *var_val, unsigned char varval_type, int var_val_len,
        unsigned char *statP, Oid *reqoid)
{
	int i;
//	IfXEntry if_x_entry;
	unsigned long long_int = 0;
	//struct netif *netif_entry;
	struct ifnet *ifnet_entry;
	char Name[16];
	unsigned long temp_int = 0;
	int interface;

	interface = reqoid->name[reqoid->namelen - 1];
 	GetIfNameFromIndex (interface, Name);
	ifnet_entry = Get_Ifnet (interface, Name);
	//netif_entry = cyg_snmp_get_if(interface);

	for(i = 0; i < var_val_len; i++)
	{
		temp_int <<= sizeof(char) * 8;
		temp_int += var_val[i];
	}

	switch (action) {
	case RESERVE1:
		return 0;
	case RESERVE2:
		return 0;
	case COMMIT:
		long_int = ntohl(temp_int);
		if (long_int == 1)
			ifnet_entry->if_flags |= IFF_PROMISC;
			// netif_entry->flags = ( netif_entry->flags | NETIF_FLAG_PROMISC);
		else if (long_int == 2)
			ifnet_entry->if_flags &= ~IFF_PROMISC;
			// netif_entry->flags = ( netif_entry->flags & (~NETIF_FLAG_PROMISC));
		return 0;
	case ACTION:
		return 0;
	case FREE:
		return 0;
	}
}

//added by Anath
//int //int was replaced by unsigned char * by imra for testing
int write_ifAlias(int action,
        unsigned char *var_val, unsigned char varval_type, int var_val_len,
        unsigned char *statP, Oid *reqoid)
{
	int interface;
//	IfXEntry if_x_entry;
	// struct netif *netif_entry;
	struct ifnet *ifnet_entry;
	char Name[16];

	interface = reqoid->name[reqoid->namelen - 1];
 	GetIfNameFromIndex (interface, Name);
	ifnet_entry = Get_Ifnet (interface, Name);
	//netif_entry = cyg_snmp_get_if(interface);

	switch (action) {
	case RESERVE1:
		return 0;
	case RESERVE2:
		return 0;
	case COMMIT:
		// FIXME: Santosh, not supported in ifnet struct
		//strncpy(netif_entry->if_xentry.if_alias, var_val, var_val_len);
		//netif_entry->if_xentry.if_alias[var_val_len] = 0;
		//return NO_MIBINSTANCE;
		return 0;
	case ACTION:
		return 0;
	case FREE:
		return 0;
	}
}

/* An entry containing additional management information
 * applicable to a particular interface.
 */

unsigned char *
var_ifXEntry(int *var_len,
        Oid *newoid, Oid *reqoid, int searchType,
        snmp_info_t *mesg, int (**write_method)())
{
	/* Last subOID of COLUMNAR OID is column */
	int	column = newoid->name[(newoid->namelen - 1)];
	int	result;

#if 0
    while (0 /* Test whether the entry has been found */) {
        /* Add indexes for the entry to OID newname */
        /* Determine whether it is the requested OID  */
        result = compare(reqoid, newoid);
        if (((searchType == EXACT) && (result != 0)) ||
                ((searchType == NEXT) && (result >= 0))) {
            return NO_MIBINSTANCE;
        }
    }
    /* Place here the computation functionality */
#endif

#if 1
	register int interface;
//	IfXEntry if_x_entry;
	//struct netif *netif_entry;
	struct ifnet *ifnet_entry;
	char Name[16];

	//if_number = cyg_snmp_num_interfaces();
	if_number = Interface_Scan_Get_Count ();

	interface = reqoid->name[(newoid->namelen)];
	result = compare(reqoid, newoid); 

	if (result <= 0) {
		return NO_MIBINSTANCE;
	}

	if (searchType == NEXT) {
		for (interface += 1; interface <= if_number; interface++) {
//		if (searchType == EXACT) {
//			interface--;
//			netif_entry = cyg_snmp_get_if(interface);
//			break; 
//		}
		//	if ((netif_entry = cyg_snmp_get_if(interface)) != NULL) {
 			GetIfNameFromIndex (interface, Name);
			if ((ifnet_entry = Get_Ifnet(interface, Name)) != NULL) {
				break; 
			}
		}
	} else {
 		GetIfNameFromIndex (interface, Name);
		ifnet_entry = Get_Ifnet(interface, Name);
		// netif_entry = cyg_snmp_get_if(interface);
	}

	if (interface > if_number) {
		return NO_MIBINSTANCE;
	}

	newoid->name[newoid->namelen] = interface;
	newoid->name[newoid->namelen + 1] = 0;
	newoid->namelen += 1;
#endif

	/* Set write-function */
	*write_method = 0;
	/* Set size (in bytes) and return address of the variable */
	*var_len = sizeof(long);
	switch (column) {
	case I_ifName:
		sprintf(char_ptr_ret, "%c%c%d", ifnet_entry->if_name[0], ifnet_entry->if_name[1]);
		*var_len = strlen(char_ptr_ret);
		return (unsigned char *) (char_ptr_ret);
	case I_ifInMulticastPkts:
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_xentry.if_in_multicast_pkts;
		//return (unsigned char *) (& long_ret);
		return NO_MIBINSTANCE;
	case I_ifInBroadcastPkts:
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_xentry.if_in_broadcast_pkts;
		//return (unsigned char *) (& long_ret);
		return NO_MIBINSTANCE;
	case I_ifOutMulticastPkts:
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_xentry.if_out_multicast_pkts;
		//return (unsigned char *) (& long_ret);
		return NO_MIBINSTANCE;
	case I_ifOutBroadcastPkts:
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_xentry.if_out_broadcast_pkts;
		//return (unsigned char *) (& long_ret);
		return NO_MIBINSTANCE;
	case I_ifHCInOctets:
		*var_len = sizeof(unsigned long long);
		long_long_ret = 0;
		//return (unsigned char *) (& long_long_ret);
		return NO_MIBINSTANCE;
	case I_ifHCInUcastPkts:
		*var_len = sizeof(unsigned long long);
		long_long_ret = 0;
		//return (unsigned char *) (& long_long_ret);
		return NO_MIBINSTANCE;
	case I_ifHCInMulticastPkts:
		*var_len = sizeof(unsigned long long);
		long_long_ret = 0;
		//return (unsigned char *) (& long_long_ret);
		return NO_MIBINSTANCE;
	case I_ifHCInBroadcastPkts:
		*var_len = sizeof(unsigned long long);
		long_long_ret = 0;
		//return (unsigned char *) (& long_long_ret);
		return NO_MIBINSTANCE;
	case I_ifHCOutOctets:
		*var_len = sizeof(unsigned long long);
		long_long_ret = 0;
		//return (unsigned char *) (& long_long_ret);
		return NO_MIBINSTANCE;
	case I_ifHCOutUcastPkts:
		*var_len = sizeof(unsigned long long);
		long_long_ret = 0;
		//return (unsigned char *) (& long_long_ret);
		return NO_MIBINSTANCE;
	case I_ifHCOutMulticastPkts:
		*var_len = sizeof(unsigned long long);
		long_long_ret = 0;
		//return (unsigned char *) (& long_long_ret);
		return NO_MIBINSTANCE;
	case I_ifHCOutBroadcastPkts:
		*var_len = sizeof(unsigned long long);
		long_long_ret = 0;
		//return (unsigned char *) (& long_long_ret);
		return NO_MIBINSTANCE;
	case I_ifLinkUpDownTrapEnable:
		if (read_write_flag == 3) {
			*write_method = write_ifLinkUpDownTrapEnable;
		}
		// FIXME: Santosh, not supported in ifnet struct
		//long_ret = netif_entry->if_xentry.if_link_up_down_trap_enable;
		return (unsigned char *) (& long_ret);
	case I_ifHighSpeed:
		long_ret = 100;
		return (unsigned char *) (& long_ret);
	case I_ifPromiscuousMode:
		if (read_write_flag == 3) {
			*write_method = write_ifPromiscuousMode;
		}
		//long_ret = (netif_entry->flags & NETIF_FLAG_PROMISC ? 1 : 2);
		long_ret = (ifnet_entry->if_flags & IFF_PROMISC ? 1 : 2);
		return (unsigned char *) (& long_ret);
	case I_ifConnectorPresent:
		return NO_MIBINSTANCE;
	case I_ifAlias:
		// FIXME: Santosh, not supported in ifnet struct
#if 0
		if (read_write_flag == 3) {
			*write_method = write_ifAlias;
		}
		strcpy(char_ptr_ret, netif_entry->if_xentry.if_alias);
		*var_len = strlen(char_ptr_ret);
		return (unsigned char *) (char_ptr_ret);
#endif //0
		return NO_MIBINSTANCE;
	case I_ifCounterDiscontinuityTime:
		//long_ret = netif_entry->if_xentry.if_counter_discontinuity_time;
		//return (unsigned char *) (& long_ret);
		return NO_MIBINSTANCE;
	default:
		return NO_MIBINSTANCE;
	}
}

/* Information on a particular relationship between two sub-
 * layers, specifying that one sub-layer runs on 'top' of the
 * other sub-layer.  Each sub-layer corresponds to a conceptual
 * row in the ifTable.
 */

unsigned char *
var_ifStackEntry(int *var_len,
        Oid *newoid, Oid *reqoid, int searchType,
        snmp_info_t *mesg, int (**write_method)())
{
	/* Last subOID of COLUMNAR OID is column */
	int	column = newoid->name[(newoid->namelen - 1)];
	int	result;

#if 0
    while (0 /* Test whether the entry has been found */) {
        /* Add indexes for the entry to OID newname */
        /* Determine whether it is the requested OID  */
        result = compare(reqoid, newoid);
        if (((searchType == EXACT) && (result != 0)) ||
                ((searchType == NEXT) && (result >= 0))) {
            return NO_MIBINSTANCE;
        }
    }
    /* Place here the computation functionality */
#endif

#if 1
	register int interface;
	//struct netif *netif_entry;
	struct ifnet *ifnet_entry;
	char Name[16];

	//if_number = cyg_snmp_num_interfaces();
	if_number = Interface_Scan_Get_Count ();

	interface = reqoid->name[(newoid->namelen)];
	result = compare(reqoid, newoid); 

	if (searchType == NEXT) {
		for (interface += 1; interface <= if_number; interface++) {
			if (result <= 0) {
				return NO_MIBINSTANCE;
			}
//		if (searchType == EXACT) {
//			interface--;
//			netif_entry = cyg_snmp_get_if(interface);
//			break; 
//		}
			//if ((netif_entry = cyg_snmp_get_if(interface)) != NULL) {
 			GetIfNameFromIndex (interface, Name);
			if ((ifnet_entry = Get_Ifnet(interface, Name)) != NULL) {
				break; 
			}
		}
	} else {
 		GetIfNameFromIndex (interface, Name);
		ifnet_entry = Get_Ifnet(interface, Name);
		//netif_entry = cyg_snmp_get_if(interface);
	}

	if (interface > if_number) {
		return NO_MIBINSTANCE;
	}

	newoid->name[newoid->namelen] = interface;
	newoid->name[newoid->namelen + 1] = 0;
	newoid->namelen += 1;
#endif

    /* Set write-function */
    *write_method = 0;
    /* Set size (in bytes) and return address of the variable */
    *var_len = sizeof(long);
    switch (column) {
	case I_ifStackHigherLayer:
	    return (unsigned char *) NO_MIBINSTANCE;//not exist
	case I_ifStackLowerLayer:
	    return (unsigned char *) NO_MIBINSTANCE;//not exist
	case I_ifStackStatus:
	    return (unsigned char *) NO_MIBINSTANCE;//no access
        default:
            return NO_MIBINSTANCE;
    }
}

/* The value of sysUpTime at the time of the last change of
 * the (whole) interface stack.  A change of the interface
 * stack is defined to be any creation, deletion, or change in
 * value of any instance of ifStackStatus.  If the interface
 * stack has been unchanged since the last re-initialization of
 * the local network management subsystem, then this object
 * contains a zero value.
 */

unsigned char *
var_ifStackLastChange(int *var_len, snmp_info_t *mesg,
        int (**write_method)())
{
    /* Add value computations */

    /* Set size (in bytes) and return address of the variable */
    *var_len = sizeof(long);
    return (unsigned char *) (& if_stack_last_change);
}

/* A list of objects identifying an address for which the
 * system will accept packets/frames on the particular
 * interface identified by the index value ifIndex.
 */

unsigned char *
var_ifRcvAddressEntry(int *var_len,
        Oid *newoid, Oid *reqoid, int searchType,
        snmp_info_t *mesg, int (**write_method)())
{
	/* Last subOID of COLUMNAR OID is column */
	int	column = newoid->name[(newoid->namelen - 1)];
	int	result;

#if 0
    while (0 /* Test whether the entry has been found */) {
        /* Add indexes for the entry to OID newname */
        /* Determine whether it is the requested OID  */
        result = compare(reqoid, newoid);
        if (((searchType == EXACT) && (result != 0)) ||
                ((searchType == NEXT) && (result >= 0))) {
            return NO_MIBINSTANCE;
        }
    }
    /* Place here the computation functionality */
#endif

    /* Set write-function */
    *write_method = 0;
    /* Set size (in bytes) and return address of the variable */
    *var_len = sizeof(long);
    switch (column) {
	case I_ifRcvAddressAddress:
	    return (unsigned char *) NO_MIBINSTANCE;//not exist
	case I_ifRcvAddressStatus:
	    return (unsigned char *) NO_MIBINSTANCE;//no access
	case I_ifRcvAddressType:
	    return (unsigned char *) NO_MIBINSTANCE;//no access
        default:
            return NO_MIBINSTANCE;
    }
}

/* An entry containing objects for invoking tests on an
 * interface.
 */
/*
int	write_ifTestId(int action,
	unsigned char *var_val, unsigned char varval_type, int var_val_len,
	unsigned char *statP, oid *name, int name_len)
{
    switch (action) {
    case RESERVE1:
    case RESERVE2:
    case COMMIT:
    case FREE:
    }
}
*/

//added by Anath
//int //int was replaced by unsigned char * by imra for testing
int write_ifTestId(int action,
        unsigned char *var_val, unsigned char varval_type, int var_val_len,
        unsigned char *statP, Oid *reqoid)
{
	int i;
	IfTestEntry if_test_entry;
	unsigned long temp_int = 0;
	int interface;

	interface = reqoid->name[reqoid->namelen - 1];

	for(i = 0; i < var_val_len; i++)
	{
		temp_int <<= sizeof(char) * 8;
		temp_int += var_val[i];
	}

        switch (action) {
        case RESERVE1:
                return 0;
        case RESERVE2:
                return 0;
        case COMMIT:
                if_test_entry.if_test_id = ntohl(temp_int);
                return 0;
        case ACTION://added by imran
                return 0;
        case FREE:
                return 0;
        }
}

//added by Anath
//int //int was replaced by unsigned char * by imra for testing
int write_ifTestStatus(int action,
        unsigned char *var_val, unsigned char varval_type, int var_val_len,
        unsigned char *statP, Oid *reqoid)
{
	int i;
	IfTestEntry if_test_entry;
	unsigned long temp_int = 0;
	int interface;

	interface = reqoid->name[reqoid->namelen - 1];

	for(i = 0; i < var_val_len; i++)
	{
		temp_int <<= sizeof(char) * 8;
		temp_int += var_val[i];
	}

        switch (action) {
        case RESERVE1:
                return 0;
        case RESERVE2:
                return 0;
        case COMMIT:
                if_test_entry.if_test_status = ntohl(temp_int);
                return 0;
        case ACTION://added by imran
                return 0;
        case FREE:
                return 0;
        }
}

//added by Anath
//int //int was replaced by unsigned char * by imra for testing
int write_ifTestType(int action,
        unsigned char *var_val, unsigned char varval_type, int var_val_len,
        unsigned char *statP, Oid *reqoid)
{
	int interface;
	IfTestEntry if_test_entry;

	interface = reqoid->name[reqoid->namelen - 1];

        switch (action) {
        case RESERVE1:
                return 0;
        case RESERVE2:
                return 0;
        case COMMIT:
                strncpy(if_test_entry.if_test_type, var_val, var_val_len);
                return 0;
        case ACTION://added by imran
                return 0;
        case FREE:
                return 0;
        }
}

//added by Anath
//int //int was replaced by unsigned char * by imra for testing
int write_ifTestOwner(int action,
        unsigned char *var_val, unsigned char varval_type, int var_val_len,
        unsigned char *statP, Oid *reqoid)
{
	int interface;
	IfTestEntry if_test_entry;

	interface = reqoid->name[reqoid->namelen - 1];

        switch (action) {
        case RESERVE1:
                return 0;
        case RESERVE2:
                return 0;
        case COMMIT:
                strncpy(if_test_entry.if_test_owner, var_val, var_val_len);
                return 0;
        case ACTION://added by imran
                return 0;
        case FREE:
                return 0;
        }
}

unsigned char *
var_ifTestEntry(int *var_len,
        Oid *newoid, Oid *reqoid, int searchType,
        snmp_info_t *mesg, int (**write_method)())
{
	/* Last subOID of COLUMNAR OID is column */
	int	column = newoid->name[(newoid->namelen - 1)];
	int	result;

#if 0
    while (0 /* Test whether the entry has been found */) {
        /* Add indexes for the entry to OID newname */
        /* Determine whether it is the requested OID  */
        result = compare(reqoid, newoid);
        if (((searchType == EXACT) && (result != 0)) ||
                ((searchType == NEXT) && (result >= 0))) {
            return NO_MIBINSTANCE;
        }
    }
    /* Place here the computation functionality */
#endif

#if 1
	register int interface;
	IfTestEntry if_test_entry;
	//struct netif *netif_entry;
	struct ifnet *ifnet_entry;
	char Name[16];

	//if_number = cyg_snmp_num_interfaces();
	if_number = Interface_Scan_Get_Count ();

	interface = reqoid->name[(newoid->namelen)];
	result = compare(reqoid, newoid); 

	if (searchType == NEXT) {
		for (interface += 1; interface <= if_number; interface++) {
			if (result <= 0) {
				return NO_MIBINSTANCE;
			}
//		if (searchType == EXACT) {
//			interface--;
//			netif_entry = cyg_snmp_get_if(interface);
//			break; 
//		}
			//if ((netif_entry = cyg_snmp_get_if(interface)) != NULL) {
 			GetIfNameFromIndex (interface, Name);
			if ((ifnet_entry = Get_Ifnet(interface, Name)) != NULL) {
				break; 
			}
		}
	} else {
 		GetIfNameFromIndex (interface, Name);
		ifnet_entry = Get_Ifnet(interface, Name);
		//netif_entry = cyg_snmp_get_if(interface);
	}

	if (interface > if_number) {
		return NO_MIBINSTANCE;
	}

	newoid->name[newoid->namelen] = interface;
	newoid->name[newoid->namelen + 1] = 0;
	newoid->namelen += 1;
#endif

    /* Set write-function */
    *write_method = 0;
    /* Set size (in bytes) and return address of the variable */
    *var_len = sizeof(long);
    switch (column) {
	case I_ifTestId:
            return NO_MIBINSTANCE;
	    if (read_write_flag == 3) {
	    	*write_method = write_ifTestId;
	    }
	case I_ifTestStatus:
            return NO_MIBINSTANCE;
	    if (read_write_flag == 3) {
	    	*write_method = write_ifTestStatus;
	    }
	case I_ifTestType:
            return NO_MIBINSTANCE;
	    if (read_write_flag == 3) {
	    	*write_method = write_ifTestType;
	    }
	case I_ifTestResult:
            return NO_MIBINSTANCE;
	case I_ifTestCode:
            return NO_MIBINSTANCE;
	case I_ifTestOwner:
            return NO_MIBINSTANCE;
	    if (read_write_flag == 3) {
	    	*write_method = write_ifTestOwner;
	    }
        default:
            return NO_MIBINSTANCE;
    }
}
#endif

static oid ifMIBObjects_oid[] = { O_ifMIBObjects };
static Object ifMIBObjects_variables[] = {
    { SNMP_STRING, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifName }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifInMulticastPkts }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifInBroadcastPkts }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifOutMulticastPkts }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifOutBroadcastPkts }}},
    { SNMP_COUNTER64, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifHCInOctets }}},
    { SNMP_COUNTER64, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifHCInUcastPkts }}},
    { SNMP_COUNTER64, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifHCInMulticastPkts }}},
    { SNMP_COUNTER64, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifHCInBroadcastPkts }}},
    { SNMP_COUNTER64, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifHCOutOctets }}},
    { SNMP_COUNTER64, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifHCOutUcastPkts }}},
    { SNMP_COUNTER64, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifHCOutMulticastPkts }}},
    { SNMP_COUNTER64, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifHCOutBroadcastPkts }}},
    { SNMP_INTEGER, (RWRITE| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifLinkUpDownTrapEnable }}},
    { SNMP_GAUGE, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifHighSpeed }}},
    { SNMP_INTEGER, (RWRITE| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifPromiscuousMode }}},
    { SNMP_INTEGER, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifConnectorPresent }}},
    { SNMP_STRING, (RWRITE| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifAlias }}},
    { SNMP_TIMETICKS, (RONLY| COLUMN), var_ifXEntry,
                {3, { I_ifXTable, I_ifXEntry, I_ifCounterDiscontinuityTime }}},
    { SNMP_INTEGER, (NOACCESS| COLUMN), var_ifStackEntry,
                {3, { I_ifStackTable, I_ifStackEntry, I_ifStackStatus }}},
    { SNMP_INTEGER, (RWRITE| COLUMN), var_ifTestEntry,
                {3, { I_ifTestTable, I_ifTestEntry, I_ifTestId }}},
    { SNMP_INTEGER, (RWRITE| COLUMN), var_ifTestEntry,
                {3, { I_ifTestTable, I_ifTestEntry, I_ifTestStatus }}},
    { SNMP_OBJID, (RWRITE| COLUMN), var_ifTestEntry,
                {3, { I_ifTestTable, I_ifTestEntry, I_ifTestType }}},
    { SNMP_INTEGER, (RONLY| COLUMN), var_ifTestEntry,
                {3, { I_ifTestTable, I_ifTestEntry, I_ifTestResult }}},
    { SNMP_OBJID, (RONLY| COLUMN), var_ifTestEntry,
                {3, { I_ifTestTable, I_ifTestEntry, I_ifTestCode }}},
    { SNMP_STRING, (RWRITE| COLUMN), var_ifTestEntry,
                {3, { I_ifTestTable, I_ifTestEntry, I_ifTestOwner }}},
    { SNMP_INTEGER, (NOACCESS| COLUMN), var_ifRcvAddressEntry,
                {3, { I_ifRcvAddressTable, I_ifRcvAddressEntry, I_ifRcvAddressStatus }}},
    { SNMP_INTEGER, (NOACCESS| COLUMN), var_ifRcvAddressEntry,
                {3, { I_ifRcvAddressTable, I_ifRcvAddressEntry, I_ifRcvAddressType }}},
    { SNMP_TIMETICKS, (RONLY| SCALAR), var_ifTableLastChange,
                 {2, { I_ifTableLastChange, 0 }}},
    { SNMP_TIMETICKS, (RONLY| SCALAR), var_ifStackLastChange,
                 {2, { I_ifStackLastChange, 0 }}},
    { NULL }
    };
static SubTree ifMIBObjects_tree =  { NULL, ifMIBObjects_variables,
	        (sizeof(ifMIBObjects_oid)/sizeof(oid)), ifMIBObjects_oid};

static oid interfaces_oid[] = { O_interfaces };
static Object interfaces_variables[] = {
    { SNMP_INTEGER, (RONLY| SCALAR), var_ifNumber,
                 {2, { I_ifNumber, 0 }}},
    { SNMP_INTEGER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifIndex }}},
    { SNMP_STRING, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifDescr }}},
    { SNMP_INTEGER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifType }}},
    { SNMP_INTEGER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifMtu }}},
    { SNMP_GAUGE, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifSpeed }}},
    { SNMP_STRING, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifPhysAddress }}},
    { SNMP_INTEGER, (RWRITE| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifAdminStatus }}},
    { SNMP_INTEGER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifOperStatus }}},
    { SNMP_TIMETICKS, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifLastChange }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifInOctets }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifInUcastPkts }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifInNUcastPkts }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifInDiscards }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifInErrors }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifInUnknownProtos }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifOutOctets }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifOutUcastPkts }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifOutNUcastPkts }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifOutDiscards }}},
    { SNMP_COUNTER, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifOutErrors }}},
    { SNMP_GAUGE, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifOutQLen }}},
    { SNMP_OBJID, (RONLY| COLUMN), var_ifEntry,
                {3, { I_ifTable, I_ifEntry, I_ifSpecific }}},
    { NULL }
    };
static SubTree interfaces_tree =  { NULL, interfaces_variables,
	        (sizeof(interfaces_oid)/sizeof(oid)), interfaces_oid};

/* This is the MIB registration function. This should be called */
/* within the init_IF_MIB-function */
void register_subtrees_of_IF_MIB()
{
    /* Santosh, we may not need this */
    //insert_group_in_mib(&ifMIBObjects_tree);
    insert_group_in_mib(&interfaces_tree);
}
