


//#include <library/snmp_config.h>
#include <sys/param.h>

/* General includes */
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <sys/utsname.h>
#include <unistd.h>


#include <asn1.h>
#include <snmp.h>
#include <agt_mib.h>
#include <snmpv2-mib.h>

#include <ifx_common.h>
#include <ifx_config.h>
#include <ifx_amazon_cfg.h>
#include <ifx_api_structs.h>

#if 0
#define ifx_snmp_dbg(format, args...) printf(format, ##args)
#else
#define ifx_snmp_dbg(format, args...)
#endif

struct timeval snmp_boottime;
char sysDescr_value[ 128 ];
char sysLocation_value[ 128 ] = "[no location given]";
char sysContact_value[ 128 ] = "[no location given]";


extern int read_write_flag;
#define SMASH_MAX_LEN 255
#define SYSNAME       1
#define SYSLOCATION   2
#define SYSCONTACT    3

unsigned char sysDescr[SMASH_MAX_LEN];
unsigned int  sysObjId[SMASH_MAX_LEN];
unsigned char sysContact[SMASH_MAX_LEN];
unsigned char sysName[SMASH_MAX_LEN];
unsigned char sysLocation[SMASH_MAX_LEN];
unsigned long sysUptime;
unsigned long sysServices;
#if 0
unsigned char sys_descr[] = "My machine";
unsigned char sys_object_id[] = {1, 3, 6, 1, 4, 1, 785, 3};
unsigned long sys_up_time;
unsigned long sys_services = 78;
#endif
unsigned long sys_or_last_change;
typedef struct {
	unsigned long sys_or_id;
	char *sys_or_descr;
	unsigned long sys_or_uptime;
} SysOREntry;

//static SysOREntry sys_or_entry;

/* SNMPv2_MIB initialisation (must also register the MIB module tree) */
void init_SNMPv2_MIB()
{
	gettimeofday(&snmp_boottime, (struct timezone *)0);//test imran
	register_subtrees_of_SNMPv2_MIB();
}

int
init_sysDescr()
{
struct utsname  sysident;

    if (uname(&sysident)) {
	return(-1);
    }
    sprintf(sysDescr_value, "%s %s %s", sysident.sysname, sysident.release,
					sysident.machine);
    return(0);
}

char *
set_sysDescription(char *line)
{
    strncpy(sysDescr_value, line, 127);
    sysDescr_value[127] = '\0';
    return(0);
}

char *
set_sysLocation(char *line)
{
    strncpy(sysLocation_value, line, 128);
    return(0);
}

char *
set_sysContact(char *line)
{
    strncpy(sysContact_value, line, 128);
    return(0);
}

int set_sysmib (char *param_val, int param)
{
    DEVICE_INFO devInfo;
	int ret = 0, flags = 0;
	
    memset(&devInfo, 0x00, sizeof(DEVICE_INFO));
	
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
    /* Get all the DEVICE_INFO parameters using Object API */
    ret = ifx_get_device_info(&devInfo, flags);
	if (ret < 0) {
	    ifx_snmp_dbg ("[%s]:[%d] - error occurred while getting device info\n", __FUNCTION__, __LINE__);
		return -1;
	}

	switch (param)
	{
		case SYSNAME:
			ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
			if ((strlen(param_val) + 1) > SMASH_MAX_LEN) {
	    		ifx_snmp_dbg ("[%s]:[%d] - sysname passed doesnt fit in devInfo\n", __FUNCTION__, __LINE__);
				return -1;
			}
			strcpy (devInfo.sysName, param_val);
			break;
		case SYSCONTACT:
			ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
			if ((strlen(param_val) + 1) > SMASH_MAX_LEN) {
	    		ifx_snmp_dbg ("[%s]:[%d] - syscontact passed doesnt fit in devInfo\n", __FUNCTION__, __LINE__);
				return -1;
			}
			strcpy (devInfo.sysContact, param_val);
			break;
		case SYSLOCATION:
			ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
			if ((strlen(param_val) + 1) > SMASH_MAX_LEN) {
	    		ifx_snmp_dbg ("[%s]:[%d] - syslocation passed doesnt fit in devInfo\n", __FUNCTION__, __LINE__);
				return -1;
			}
			strcpy (devInfo.sysLocation, param_val);
			break;
		default:
			ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
			return -1;
	}
    /* Fill the iid structure in DEVICE_INFO */
    devInfo.iid.config_owner = IFX_SNMP;

	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
    /* Set all the DEVICE_INFO parameters using Object API */
    ret = ifx_set_device_info(IFX_OP_MOD, &devInfo, IFX_F_MODIFY | IFX_F_DONT_CHECKPOINT);
	if (ret < 0) {
	    ifx_snmp_dbg ("[%s]:[%d] - error occurred while setting device info\n", __FUNCTION__, __LINE__);
		return -1;
	}
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	return 0;
}


/* A textual description of the entity.  This value should
 * include the full name and version identification of the
 * system's hardware type, software operating-system, and
 * networking software.
 */

unsigned char *
var_sysDescr(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
	DEVICE_INFO devInfo;
	unsigned int flags = 0;
	int ret = 0;

	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	*write_method = 0;

	memset (&devInfo, 0x00, sizeof(DEVICE_INFO));
	ret = ifx_get_device_info (&devInfo, flags);                                                                    
	if (ret < 0) {
	    ifx_snmp_dbg ("[%s]:[%d] - error occurred while getting device info\n", __FUNCTION__, __LINE__);
		return 0;
	}
	*var_len = strlen(devInfo.description);
	strcpy (sysDescr, devInfo.description);
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	ifx_snmp_dbg ("sysdescr=%s\n", sysDescr);
	return (unsigned char *) (sysDescr);

}

/* The vendor's authoritative identification of the network
 * allocated within the SMI enterprises subtree (1.3.6.1.4.1)
 * and provides an easy and unambiguous means for determining
 * `Flintstones, Inc.' was assigned the subtree
 * 1.3.6.1.4.1.4242, it could assign the identifier
 * 1.3.6.1.4.1.4242.1.1 to its `Fred Router'.
 */

unsigned char *
var_sysObjectID(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
	DEVICE_INFO devInfo;
	char *sPtr = NULL, *saveptr = NULL;
	unsigned int flags = 0;
	int ret = 0, index = 0;

	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	memset (&devInfo, 0x00, sizeof(DEVICE_INFO));
	ret = ifx_get_device_info (&devInfo, flags);                                                                    
	if (ret < 0) {
	    ifx_snmp_dbg ("[%s]:[%d] - error occurred while getting device info\n", __FUNCTION__, __LINE__);
		return 0;
	}
#if 1
	ifx_snmp_dbg ("[%s]:[%d] devInfo.sysObjId:%s\n", __FUNCTION__, __LINE__, devInfo.sysObjId);
	index = 0;
	sysObjId[index] = atoi(strtok(devInfo.sysObjId, "."));
	index++;
	for (;;)
	{
		if ((sPtr = strtok(NULL, ".")) == NULL)
		{
			ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
			break;
		}
		sysObjId[index] = atoi(sPtr);
		index++;
	}
	*var_len = sizeof(unsigned int) * index; 
	ifx_snmp_dbg ("[%s]:[%d] index=%d\n", __FUNCTION__, __LINE__, index);
#endif 

	ifx_snmp_dbg ("[%s]:[%d] - varlen=%d\n", __FUNCTION__, __LINE__, *var_len);
	return (unsigned char *) (sysObjId);
}

unsigned long current_sysUpTime()
{
	struct timeval  now;

	gettimeofday(&now, (struct timezone *)0);//test imran
	return ((now.tv_sec - snmp_boottime.tv_sec) * 100
		+ (now.tv_usec - snmp_boottime.tv_usec) / 10000);
}

/* The time (in hundredths of a second) since the network
 * management portion of the system was last re-initialized.
 */

unsigned char *
var_sysUpTime(int *var_len, Access_rec *access_id,
        int (**write_method)())
{

#if 0
    /* FIXME: Santosh, device info api doesnt show correct time yet */
	DEVICE_INFO devInfo;
	unsigned int flags = 0;
	int ret = 0;

	*write_method = 0;
	*var_len = sizeof(long);

	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	memset (&devInfo, 0x00, sizeof(DEVICE_INFO));
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	ret = ifx_get_device_info (&devInfo, flags);                                                                    
	if (ret < 0) {
	    ifx_snmp_dbg ("[%s]:[%d] - error occurred while getting device info\n", __FUNCTION__, __LINE__);
		return 0;
	}
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	ifx_snmp_dbg (" uptime=%d\n", devInfo.uptime);
	sysUptime = devInfo.uptime;
	return (unsigned char *) (&sysUptime);
#endif //0
	*write_method = 0;
	*var_len = sizeof(long);
	sysUptime = current_sysUpTime();
	return (unsigned char *) (& sysUptime);
}

/* The textual identification of the contact person for this
 * this person.  If no contact information is known, the value
 * is the zero-length string.
 */

//int //int was replaced by unsigned char * by imra for testing	
int write_sysContact(int action,
	unsigned char *var_val, unsigned char varval_type, int var_val_len,
	unsigned char *statP, oid *name, int name_len)
{

	switch (action) {
	case RESERVE1:
		return 0;
	case RESERVE2:
		return 0;
	case COMMIT:
		ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
		strncpy (sysContact, var_val, var_val_len);
		sysContact[var_val_len] = '\0';
		ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
		set_sysmib (sysContact, SYSCONTACT);
		ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
		return 0;
	case ACTION:
		return 0;
	case FREE:
		return 0;
	default:
		return 0;
	}
	return 0;
}


unsigned char *
var_sysContact(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
	DEVICE_INFO devInfo;
	unsigned int flags = 0;
	int ret = 0;

	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	memset (&devInfo, 0x00, sizeof(DEVICE_INFO));
	ret = ifx_get_device_info (&devInfo, flags);                                                                    
	if (ret < 0) {
	    ifx_snmp_dbg ("[%s]:[%d] - error occurred while getting device info\n", __FUNCTION__, __LINE__);
		return 0;
	}

	/* Set write-function (uncomment if you want to implement it)  */
	if (read_write_flag == 3) {
		*write_method = write_sysContact; 
	}

	*var_len = strlen(devInfo.sysContact);
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	strcpy (sysContact, devInfo.sysContact);
	ifx_snmp_dbg ("syscontact=%s\n", sysContact);
	return (unsigned char *) (sysContact); 
}

/* An administratively-assigned name for this managed node.
 * By convention, this is the node's fully-qualified domain
 * name.  If the name is unknown, the value is the zero-length
 * string.
 */

int write_sysName(int action,
	unsigned char *var_val, unsigned char varval_type, int var_val_len,
	unsigned char *statP, oid *name, int name_len)
{
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	switch (action) {
	case RESERVE1:
		return 0;
	case RESERVE2:
		return 0;
	case COMMIT:
		ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
		strncpy (sysName, var_val, var_val_len);
		sysName[var_val_len] = '\0';
		ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
		set_sysmib (sysName, SYSNAME);
		ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
		return 0;
	case ACTION:
		return 0;
	case FREE:
		return 0;
	default:
		return -1;
	}
}

/*
unsigned char *
var_sysName(int *var_len, Access_rec *access_id,
        int (**write_method)(int , unsigned char *, unsigned char , int ,    
		                     unsigned char *, oid *, int )) */
unsigned char *
var_sysName(int *var_len, Access_rec *access_id,
        int (**write_method)())

{
	DEVICE_INFO devInfo;
	unsigned int flags = 0;
	int ret = 0;

	memset (&devInfo, 0x00, sizeof(DEVICE_INFO));
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);

	if (read_write_flag == 3) {
		*write_method = write_sysName;
	}

	ret = ifx_get_device_info (&devInfo, flags);
	if (ret < 0)
	{
	    ifx_snmp_dbg ("[%s]:[%d] - error occurred while getting device info\n", __FUNCTION__, __LINE__);
		return 0;
	}
	*var_len = strlen(devInfo.sysName);
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	strcpy (sysName , devInfo.sysName);
	ifx_snmp_dbg ("sysName=%s\n", sysName);
	return (unsigned char *) (sysName);
}

/* The physical location of this node (e.g., `telephone
 * closet, 3rd floor').  If the location is unknown, the value
 * is the zero-length string.
 */

int write_sysLocation(int action,
	unsigned char *var_val, unsigned char varval_type, int var_val_len,
	unsigned char *statP, oid *name, int name_len)
{

	switch (action) {
	case RESERVE1:
		return 0; 
	case RESERVE2:
		return 0;
	case COMMIT:
		ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
		strncpy (sysLocation, var_val, var_val_len);
		sysLocation[var_val_len] = '\0';
		ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
		set_sysmib (sysLocation, SYSLOCATION);
		ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
		return 0;
	case ACTION:
		return 0;
	case FREE:
		return 0;
	default:
		return 0;
	}
	return 0;
}

unsigned char *
var_sysLocation(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
	DEVICE_INFO devInfo;
	int ret = 0;
	unsigned int flags = 0;

	memset (&devInfo, 0x00, sizeof(DEVICE_INFO));
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);

	/* Set write-function (uncomment if you want to implement it)  */
	if (read_write_flag == 3) {
		*write_method = write_sysLocation; 
	}
	ret = ifx_get_device_info (&devInfo, flags);                                                                     
	if (ret < 0) {
	    ifx_snmp_dbg ("[%s]:[%d] - error occurred while getting device info\n", __FUNCTION__, __LINE__);
		return 0;
	}
	*var_len = strlen(devInfo.sysLocation);
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	strcpy (sysLocation ,devInfo.sysLocation); 
	ifx_snmp_dbg ("sysLocation=%s\n", sysLocation);
	return (unsigned char *) (sysLocation);
}

/* A value which indicates the set of services that this
 * entity may potentially offers.  The value is a sum.  This
 * sum initially takes the value zero, Then, for each layer, L,
 * in the range 1 through 7, that this node performs
 * transactions for, 2 raised to (L - 1) is added to the sum.
 * For example, a node which performs only routing functions
 * would have a value of 4 (2^(3-1)).  In contrast, a node
 * which is a host offering application services would have a
 * value of 72 (2^(4-1) + 2^(7-
 */

unsigned char *
var_sysServices(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
	DEVICE_INFO devInfo;
	unsigned int flags = 0;
	int ret = 0;

	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	memset (&devInfo, 0x00, sizeof(DEVICE_INFO));
	*var_len = sizeof(long);

	ret = ifx_get_device_info (&devInfo, flags);                                                                    
	if (ret < 0) {
	    ifx_snmp_dbg ("[%s]:[%d] - error occurred while getting device info\n", __FUNCTION__, __LINE__);
	    return 0;
	}
	ifx_snmp_dbg ("[%s]:[%d] \n", __FUNCTION__, __LINE__);
	sysServices = devInfo.sysServices;
	return (unsigned char *) (&sysServices);
}

unsigned long sysORLastChange_value = 0;
struct sysOREntry_struct snmpV2_value = { NULL, 1, {7, {O_snmpMIB}}, "The MIB module for SNMPv2 entities", 0};
struct sysOREntry_struct *sysOREntry_root = &snmpV2_value;

int insert_sysOREntry(struct sysOREntry_struct *value)  
{
struct sysOREntry_struct *current = sysOREntry_root;

    while (current->next) {
            current = current->next;
    }
    value->index = current->index + 1;
    current->next = value;
    sysORLastChange_value = current_sysUpTime();
    current->lastChange = sysORLastChange_value;
    return(0); 
}   

/* The value of sysUpTime at the time of the most recent
 * change in state or value of any instance of sysORID.
 */

unsigned char *
var_sysORLastChange(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
    *write_method = 0;
    *var_len = sizeof(sys_or_last_change);
    return (unsigned char *) (& sys_or_last_change);
}

/* An entry (conceptual row) in the sysORTable.
 */

unsigned char *
var_sysOREntry(int *var_len,
        Oid *newoid, Oid *reqoid, int searchType,
        Access_rec *access_id, int (**write_method)())
{

/* Last subOID of COLUMNAR OID is column */
int     column = newoid->name[(newoid->namelen - 1)];
int     sysORIndex = newoid->namelen++;

/* Variables defined */
int     result;
struct sysOREntry_struct *current = sysOREntry_root;

    while (current) { 
        newoid->name[ sysORIndex ] = current->index;
        result = compare(reqoid, newoid);
        if (((searchType == EXACT) && (result == 0)) ||
                ((searchType == NEXT) && (result < 0))) {
           break;
        }
        current = current->next;
    }
    if (current == NULL) return(NULL);

    *write_method = 0;

    switch (column) {
        case I_sysORIndex:
            return (unsigned char *) NO_MIBINSTANCE;
        case I_sysORID:
            *var_len = current->oid_value.namelen * sizeof(oid);
            return (unsigned char *) &(current->oid_value.name);
        case I_sysORDescr:
            *var_len = strlen(current->descr);
            return (unsigned char *) current->descr;
        case I_sysORUpTime:
            *var_len = sizeof(unsigned long);
            return (unsigned char *) &(current->lastChange);
        default:
            return NO_MIBINSTANCE;
    }
}

/* The total number of messages delivered to the SNMP entity
 * from the transport service.
 */

unsigned char *
var_snmpInPkts(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
extern long     snmpInPkts_instance;

    *write_method = 0;
    *var_len = sizeof(long);
    return (unsigned char *) &snmpInPkts_instance;
}

/* The total number of SNMP messages which were delivered to
 * the SNMP entity and were for an unsupported SNMP version.
 */

unsigned char *
var_snmpInBadVersions(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
extern long     snmpInBadVersions_instance;

    *write_method = 0;
    *var_len = sizeof(long);
    return (unsigned char *) &snmpInBadVersions_instance;
}

/* The total number of SNMP messages delivered to the SNMP
 * entity which used a SNMP community name not known to said
 * entity.
 */

unsigned char *
var_snmpInBadCommunityNames(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
extern long     snmpInBadCommunityNames_instance;

    *write_method = 0;
    *var_len = sizeof(long);
    return (unsigned char *) &snmpInBadCommunityNames_instance;
}

/* The total number of SNMP messages delivered to the SNMP
 * entity which represented an SNMP operation which was not
 * allowed by the SNMP community named in the message.
 */

unsigned char *
var_snmpInBadCommunityUses(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
extern long     snmpInBadCommunityUses_instance;

    *write_method = 0;
    *var_len = sizeof(long);
    return (unsigned char *) &snmpInBadCommunityUses_instance;
}

/* The total number of ASN.1 or BER errors encountered by the
 * SNMP entity when decoding received SNMP messages.
 */

unsigned char *
var_snmpInASNParseErrs(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
extern long     snmpInASNParseErrs_instance;

    *write_method = 0;
    *var_len = sizeof(long);
    return (unsigned char *) &snmpInASNParseErrs_instance;
}

/* Indicates whether the SNMP entity is permitted to generate
 * authenticationFailure traps.  The value of this object
 * overrides any configuration information; as such, it
 * provides a means whereby all authenticationFailure traps may
 * be disabled.
 * 
 * Note that it is strongly recommended that this object be
 * stored in non-volatile memory so that it remains constant
 * across re-initializations of the network management system.
 */
/*
int	write_snmpEnableAuthenTraps(int action,
	unsigned char *var_val, unsigned char varval_type, int var_val_len,
	unsigned char *statP, oid *name, int name_len)
{
    switch (action) {
    case RESERVE1:
    case RESERVE2:
    case COMMIT:
    case FREE:
    }
}
*/

unsigned char *
var_snmpEnableAuthenTraps(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
extern long     snmpEnableAuthenTraps_instance;

    /* Set write-function (uncomment if you want to implement it)  */
    /* *write_method = &write_snmpEnableAuthenTraps(); */
    snmpEnableAuthenTraps_instance = 2;
    *write_method = 0;
    *var_len = sizeof(long);
    return (unsigned char *) &snmpEnableAuthenTraps_instance;
}

/* The total number of GetRequest-PDUs, GetNextRequest-PDUs,
 * GetBulkRequest-PDUs, SetRequest-PDUs, and InformRequest-PDUs
 * delivered to the SNMP entity which were silently dropped
 * because the size of a reply containing an alternate
 * Response-PDU with an empty variable-bindings field was
 * greater than either a local constraint or the maximum
 * message size associated with the originator of the request.
 */

unsigned char *
var_snmpSilentDrops(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
 extern long    snmpSilentDrops_instance;
 
    *write_method = 0;
    *var_len = sizeof(long);
    return (unsigned char *) &snmpSilentDrops_instance;
}

/* The total number of GetRequest-PDUs, GetNextRequest-PDUs,
 * GetBulkRequest-PDUs, SetRequest-PDUs, and InformRequest-PDUs
 * delivered to the SNMP entity which were silently dropped
 * because the transmission of the (possibly translated)
 * time-out) such that no Response-PDU could be returned.
 */

unsigned char *
var_snmpProxyDrops(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
extern long     snmpProxyDrops_instance;

    *write_method = 0;
    *var_len = sizeof(long);
    return (unsigned char *) &snmpProxyDrops_instance;
}

/* An advisory lock used to allow several cooperating SNMPv2
 * use of the SNMPv2 set operation.
 * 
 * This object is used for coarse-grain coordination.  To
 * achieve fine-grain coordination, one or more similar objects
 * might be defined within each MIB group, as appropriate.
 */
/*
int	write_snmpSetSerialNo(int action,
	unsigned char *var_val, unsigned char varval_type, int var_val_len,
	unsigned char *statP, oid *name, int name_len)
{
    switch (action) {
    case RESERVE1:
    case RESERVE2:
    case COMMIT:
    case FREE:
    }
}
*/

unsigned char *
var_snmpSetSerialNo(int *var_len, Access_rec *access_id,
        int (**write_method)())
{
    /* Add value computations */

    /* Set write-function (uncomment if you want to implement it)  */
    /* *write_method = &write_snmpSetSerialNo(); */
    /* Set size (in bytes) and return address of the variable */
    *var_len = sizeof(long);
    return (unsigned char *) NO_MIBINSTANCE;
}

static oid system_oid[] = { O_system };
static Object system_variables[] = {
    { SNMP_STRING, (RONLY| SCALAR), var_sysDescr,
                 {2, { I_sysDescr, 0 }}},
    { SNMP_OBJID, (RONLY| SCALAR), var_sysObjectID,
                 {2, { I_sysObjectID, 0 }}},
    { SNMP_TIMETICKS, (RONLY| SCALAR), var_sysUpTime,
                 {2, { I_sysUpTime, 0 }}},
    { SNMP_STRING, (RWRITE| SCALAR), var_sysContact,
                 {2, { I_sysContact, 0 }}},
    { SNMP_STRING, (RWRITE| SCALAR), var_sysName,
                 {2, { I_sysName, 0 }}},
    { SNMP_STRING, (RWRITE| SCALAR), var_sysLocation,
                 {2, { I_sysLocation, 0 }}},
    { SNMP_INTEGER, (RONLY| SCALAR), var_sysServices,
                 {2, { I_sysServices, 0 }}},
    { SNMP_TIMETICKS, (RONLY| SCALAR), var_sysORLastChange,
                 {2, { I_sysORLastChange, 0 }}},
    { SNMP_OBJID, (RONLY| COLUMN), var_sysOREntry,
                {3, { I_sysORTable, I_sysOREntry, I_sysORID }}},
    { SNMP_STRING, (RONLY| COLUMN), var_sysOREntry,
                {3, { I_sysORTable, I_sysOREntry, I_sysORDescr }}},
    { SNMP_TIMETICKS, (RONLY| COLUMN), var_sysOREntry,
                {3, { I_sysORTable, I_sysOREntry, I_sysORUpTime }}},
    { NULL }
    };
static SubTree system_tree =  { NULL, system_variables,
	        (sizeof(system_oid)/sizeof(oid)), system_oid};

static oid snmp_oid[] = { O_snmp };
static Object snmp_variables[] = {
    { SNMP_COUNTER, (RONLY| SCALAR), var_snmpInPkts,
                 {2, { I_snmpInPkts, 0 }}},
    { SNMP_COUNTER, (RONLY| SCALAR), var_snmpInBadVersions,
                 {2, { I_snmpInBadVersions, 0 }}},
    { SNMP_COUNTER, (RONLY| SCALAR), var_snmpInBadCommunityNames,
                 {2, { I_snmpInBadCommunityNames, 0 }}},
    { SNMP_COUNTER, (RONLY| SCALAR), var_snmpInBadCommunityUses,
                 {2, { I_snmpInBadCommunityUses, 0 }}},
    { SNMP_COUNTER, (RONLY| SCALAR), var_snmpInASNParseErrs,
                 {2, { I_snmpInASNParseErrs, 0 }}},
    { SNMP_INTEGER, (RWRITE| SCALAR), var_snmpEnableAuthenTraps,
                 {2, { I_snmpEnableAuthenTraps, 0 }}},
    { SNMP_COUNTER, (RONLY| SCALAR), var_snmpSilentDrops,
                 {2, { I_snmpSilentDrops, 0 }}},
    { SNMP_COUNTER, (RONLY| SCALAR), var_snmpProxyDrops,
                 {2, { I_snmpProxyDrops, 0 }}},
    { NULL }
    };
static SubTree snmp_tree =  { NULL, snmp_variables,
	        (sizeof(snmp_oid)/sizeof(oid)), snmp_oid};

/*
 * static oid snmpSet_oid[] = { O_snmpSet };
 * static Object snmpSet_variables[] = {
 *     { SNMP_INTEGER, (RWRITE| SCALAR), var_snmpSetSerialNo,
 *                  {2, { I_snmpSetSerialNo, 0 }}},
 *     { NULL }
 *     };
 * static SubTree snmpSet_tree =  { NULL, snmpSet_variables,
 *                  (sizeof(snmpSet_oid)/sizeof(oid)), snmpSet_oid};
 */

/* This is the MIB registration function. This should be called */
/* within the init_SNMPv2_MIB-function */
void register_subtrees_of_SNMPv2_MIB()
{
	insert_group_in_mib(&system_tree);
	insert_group_in_mib(&snmp_tree);
}

