#!/bin/sh

. .config
. featureconfig.sh
. modelconfig.sh
parse_args $@

if [ $BUILD_CLEAN -eq 1 ]; then
	if [ ! $BUILD_CONFIGURE -eq 1 ];then 
		rm -rf ${BUILD_ROOTFS_DIR}*
		rm -rf ${BUILD_ROOTFS_DIR}../rootfs.img 
		exit 0
	fi
fi

create_fstab()
{
echo "proc	/proc	proc	defaults	\
	0	0" > ${BUILD_ROOTFS_DIR}/etc/fstab
echo "/dev/mtdblock/1	/firmware	auto	noauto,ro	\
	0	0" >> ${BUILD_ROOTFS_DIR}/etc/fstab
echo "/dev/mtdblock/2	/	auto	defaults,remount,ro	\
	1	1" >> ${BUILD_ROOTFS_DIR}/etc/fstab
}

rm -f ${BUILD_ROOTFS_DIR}rootfs.img
rm -f ${BUILD_ROOTFS_DIR}etc/conf.tar.gz
cp -aRf ${IFX_ROOTFS_DIR}flashdisk/* ${BUILD_ROOTFS_DIR}

if [ "$IFX_ADSL_FIRMWARE_IN_ROOTFS" != "1" ]; then
rm -f ${BUILD_ROOTFS_DIR}/etc/fstab
create_fstab
fi

if [ "$IFX_CONFIG_HTTPD_SSL_SUPPORT" = "1" ]; then
    cp -a "$ROOTFS_SOURCE_DIR"/etc/services.https "$BUILD_ROOTFS_DIR"/etc/services
fi

cd ${BUILD_ROOTFS_DIR}/ramdisk_copy/
rm -rf dev/ ../dev/*
tar jxvf dev.tar.bz2
rm -rf dev.tar.bz2
cp -af dev/console ../dev/.
cd -
cp -af ${IFX_ROOTFS_DIR}sbin/modprobe.sh ${BUILD_ROOTFS_DIR}sbin

#805212:<IFTW-fchang>.added for PTM support
if [ "$IFX_ASE_CONFIG_PTM" = "1" ]; then
cp -af ${IFX_ROOTFS_DIR}sbin/autoboot.sh ${BUILD_ROOTFS_DIR}root
fi

cp -af ${BUILD_TOOLS_DIR}featureconfig.sh ${BUILD_ROOTFS_DIR}etc/rc.d/config.sh
cp -af ${BUILD_TOOLS_DIR}modelconfig.sh ${BUILD_ROOTFS_DIR}etc/rc.d/model_config.sh
cp -af ${IFX_ROOTFS_DIR}versioninfo.txt ${BUILD_ROOTFS_DIR}etc/versioninfo.txt
cp -af ${IFX_ROOTFS_DIR}sysinfo.txt ${BUILD_ROOTFS_DIR}etc/sysinfo.txt
. ${BUILD_TOOLS_DIR}featureconfig.sh
IFX_PACKAGE_VERSION=`cat .ifx_package_version`
echo $IFX_PACKAGE_VERSION > ${BUILD_ROOTFS_DIR}etc/version

#Sumedh - Add time stamp
TS=`date +%k%M-%d%b%y`
if [ -w ""$ROOTFS_SOURCE_DIR"/etc/timestamp" ]; then
	echo $TS > "$ROOTFS_SOURCE_DIR"/etc/timestamp
	echo "Adding TS ($TS) to $ROOTFS_SOURCE_DIR"
fi
if [ -e "$BUILD_ROOTFS_DIR" ]; then
	echo $TS > "$BUILD_ROOTFS_DIR"/etc/timestamp
	echo "Adding TS ($TS) to $BUILD_ROOTFS_DIR"
fi

# Pramod : if device mode is enabled, copy rcS for device mode as rcS in root filesystem
if [ -n "$IFX_CONFIG_TR69_DEVICE" -a "$IFX_CONFIG_TR69_DEVICE" = "1" ]; then
	cd ${IFX_ROOTFS_DIR}
	cp -a ./flashdisk/etc/rc.d/init.d/rcS.device ${BUILD_ROOTFS_DIR}/etc/rc.d/init.d/rcS
	cp -a ./flashdisk/etc/sys.conf.device ${BUILD_ROOTFS_DIR}/etc/sys.conf
	cd - 
fi

# create system setting
if [ $IFX_CONFIG_TR69 ]; then
	cd ${USER_IFXSOURCE_DIR}tr69
	gzip < rc.conf.tr69 > ${BUILD_ROOTFS_DIR}etc/rc.conf.gz
	cd -
else
	cd ${IFX_ROOTFS_DIR}
	# Pramod : if device mode is enabled, copy rc.conf for device mode as rc.conf in root filesystem
	if [ -n "$IFX_CONFIG_TR69_DEVICE" -a "$IFX_CONFIG_TR69_DEVICE" = "1" ]; then
		gzip < flash/rc.conf.device > rc.conf.gz
	else
		gzip < flash/rc.conf > rc.conf.gz
	fi
	mv rc.conf.gz ${BUILD_ROOTFS_DIR}etc
	cd - 
fi
# Strip as much info as possible
${IFX_STRIP} -R.note -R.comment ${BUILD_ROOTFS_DIR}bin/*
${IFX_STRIP} -R.note -R.comment ${BUILD_ROOTFS_DIR}sbin/*
${IFX_STRIP} -R.note -R.comment ${BUILD_ROOTFS_DIR}usr/bin/*
${IFX_STRIP} -R.note -R.comment ${BUILD_ROOTFS_DIR}usr/sbin/*
${IFX_STRIP} -R.note -R.comment ${BUILD_ROOTFS_DIR}root/*
${IFX_STRIP} -x -R.note -R.comment ${BUILD_ROOTFS_DIR}lib/*
${IFX_STRIP} -x -R.note -R.comment ${BUILD_ROOTFS_DIR}usr/lib/*
find ${BUILD_ROOTFS_DIR}lib/modules -type f -name "*.o" -exec ${IFX_STRIP} -x -R.note -R.comment {} \;
find ${BUILD_ROOTFS_DIR}usr/lib/ -type f -name "*.so*" -exec ${IFX_STRIP} -x -R.note -R.comment {} \;
find ${BUILD_ROOTFS_DIR}lib/ -type f -name "*.so*" -exec chmod 555 {} \;
find ${BUILD_ROOTFS_DIR}usr/lib/ -type f -name "*.so*" -exec chmod 555 {} \;

# Create symbolic link
cd ${BUILD_ROOTFS_DIR}
ln -sf ramdisk/var var
ln -sf ramdisk/tmp tmp
ln -sf ramdisk/flash flash
ln -sf ../proc/mounts etc/mtab
ln -sf /tmp/log ramdisk_copy/dev/log
ln -sf ../flash/passwd etc/passwd
ln -sf ../ramdisk/etc/hosts etc/hosts
ln -sf ../ramdisk/etc/resolv.conf etc/resolv.conf
ln -sf ../ramdisk/flash/rc.conf etc/rc.conf

if [ "$IFX_CONFIG_FTP_SERVER" = "1" ]; then
	ln -sf ../ramdisk/etc/ftpusers etc/ftpusers
fi

# TR69 Configuration directory
if [ "$IFX_CONFIG_TR69" = "1" ]; then
	ln -sf ../../ramdisk/etc/tr69/CAcertSrv.pem etc/tr69/CAcertSrv.pem
	if [ "$CONFIG_MATRIX" != "1" ]; then
		ln -sf ../../ramdisk/etc/tr69/stunnel.conf etc/tr69/stunnel.conf
		ln -sf ../../ramdisk/etc/tr69/stunnel.pem etc/tr69/stunnel.pem
	fi
fi

if [ "$IFX_CONFIG_SNMPv1" = "1" -o "$IFX_CONFIG_SNMPv3" = "1" ]; then
	ln -sf ../ramdisk/etc/snmp etc/snmp
fi

if [ "$IFX_CONFIG_DHCP_SERVER" = "1" ]; then
	ln -sf ../ramdisk/etc/udhcpd.conf etc/udhcpd.conf
fi

if [ "$IFX_CONFIG_DNSRELAY" = "1" ]; then
	ln -sf ../ramdisk/etc/dnrd/ etc/dnrd
fi

if [ "$IFX_CONFIG_RIP" = "1" ]; then
	ln -sf ../ramdisk/etc/ripd.conf etc/ripd.conf
	ln -sf ../ramdisk/etc/zebra.conf etc/zebra.conf
fi

if [ "$IFX_CONFIG_INETD" = "1" ]; then
	ln -sf ../ramdisk/etc/inetd.conf etc/inetd.conf
fi

if  [ "$IFX_CONFIG_TR037" = "1" ]; then
	ln -sf ../ramdisk/etc/ilmid etc/ilmid
fi

if [ "$IFX_CONFIG_PPPOE" = "1" -o "$IFX_CONFIG_PPPOA" = "1" ]; then
	ln -sf ../../ramdisk/etc/ppp/resolv.conf etc/ppp/resolv.conf
	ln -sf ../../ramdisk/etc/ppp/peers etc/ppp/peers
	ln -sf ../../ramdisk/etc/ppp/options etc/ppp/options
fi

if [ "$IFX_CONFIG_OPENSWAN" = "1" ]; then
	ln -sf ../ramdisk/etc/ipsec.conf etc/ipsec.conf
	ln -sf ../ramdisk/etc/ipsec.secrets etc/ipsec.secrets
fi
	
if [ "$IFX_CONFIG_WIRELESS_ATH" = "1" ]; then
	ln -sf ../ramdisk/etc/hostapd.conf etc/hostapd.conf
fi


cd -

# Remove unneeded files and shell script comments
./rm_unneeded_files.sh $@
./rm_comment.sh $@
find "$BUILD_ROOTFS_DIR" -type f -name "*dos*" | xargs rm -rf
find "$BUILD_ROOTFS_DIR/usr/web" -type f -name "napt_alg*" | xargs rm -rf
find "$BUILD_ROOTFS_DIR" -type f | xargs file | grep 'Bourne shell' | awk -F ':' '{print $1}' | xargs chmod a+x
#cp -a "$BUILD_ROOTFS_DIR" "$TOOLS_DIR"/rootfs.unstripped
find "$BUILD_ROOTFS_DIR" | xargs sstrip &>/dev/null
#create rootfs.img
cd ${BUILD_DIR}
find . -type d | xargs chmod 755
chown 0.0 root_filesystem/ -R
grep -iq "CONFIG_SQUASHFS_LZMA=y" ${TOPDIR}/config/kernel_config

find root_filesystem -name '.svn' | xargs rm -rf

if [ $? -eq 0 ];then
	mksquashfs-lzma root_filesystem/ tmp_rootfs.img -be -noappend
	mkimage -A MIPS -O Linux -C lzma -T filesystem -e 0x00 -a 0x00 -n "${PLATFORM_NAME} RootFS" -d tmp_rootfs.img rootfs.img
else
	mksquashfs root_filesystem/ tmp_rootfs.img -be -noappend
	mkimage -A MIPS -O Linux -C gzip -T filesystem -e 0x00 -a 0x00 -n "${PLATFORM_NAME} RootFS" -d tmp_rootfs.img rootfs.img
fi

rm -f tmp_rootfs.img
chmod 644 rootfs.img
#rm -rf "$BUILD_ROOTFS_DIR"
#cp -a "$TOOLS_DIR"/rootfs.unstripped "$BUILD_ROOTFS_DIR"
#rm -rf "$TOOLS_DIR"/rootfs.unstripped

[ -n "${TFTP_DIR}" ] && cp -af rootfs.img ${TFTP_DIR}
cd -

