#!/bin/sh

if [ -e ./.config ]; then
	. ./.config
	. ./featureconfig.sh
else
	. .gen_funcs
	if [ "$1" != "help" -a "$1" != "" ]; then
		display_error "\nPlease run menuconfig before building !!\n"
	fi
fi

show_help()
{
	display_info "\nUsage :\n"
	display_info "./build.sh help"
	display_info "./build.sh kernel {clean / configure}"
	display_info "./build.sh apps [opensource/ifx/\"app_name\"] {clean / configure}"
	display_info "./build.sh images"
	display_info "./build.sh rootfs\n"
	display_info "For e.g ./build.sh apps opensource busybox-1.00 configure"
	display_info "	./build.sh all clean\n"
}
show_detailed_help()
{
	display_info "\n"
	display_info "#############################################################"
	display_info "#                                                           #"
	display_info "# Information for first time users                          #"
	display_info "# ================================                          #"
	display_info "#                                                           #"
	display_info "# If you are building for the fist time, follow these steps #"
	display_info "# 1. setup.sh -p path/to/config                             #"
	display_info "# 2. make menuconfig                                        #"
	display_info "# 3. make configure                                         #"
	display_info "#                                                           #"
	display_info "#############################################################"
	display_info
	display_info
	display_info "#############################################################"
	display_info "#                                                           #"
	display_info "# Additional information on build targets                   #"
	display_info "# =======================================                   #"
	display_info "#                                                           #"
	display_info "# Following targets are supported by build                  #"
	display_info "#                                                           #"
	display_info "# configure : configures and compiles kernel and application#"
	display_info "# as selected in menuconfig. You also need to run this if   #"
	display_info "# default configuration for kernel (pointed to by arch/mips/#"
	display_info "# defconfig) or busybox (pointed to by sysdeps/linux/defcon-#" 
	display_info "# fig) is changed or for any other application which uses   #" 
	display_info "# GNU autoconf/automake system.                             #" 
	display_info "#                                                           #" 
	display_info "# clean : does a distclean on kernel and all applications   #" 
	display_info "# selected in menuconfig. You should run this if you change #" 
	display_info "# the build from big endian to little endian or vice-a-versa#" 
	display_info "#                                                           #" 
	display_info "# kernel: builds only kernel. You also could pass ARGS to  #" 
	display_info "# it. E.g :                                                 #" 
	display_info "# make kernel ARGS=configure or                            #" 
	display_info "# make kernel ARGS='clean configure'                       #" 
	display_info "#                                                           #" 
	display_info "# apps: builds opensource and ifx applications chosen in    #" 
	display_info "# menuconfig. You also could pass ARGS to it. Eg.          #" 
	display_info "# make apps ARGS=opensource will build all opensource apps #" 
	display_info "# make ifx ARGS=ifx will build all ifx apps                #" 
	display_info "# make apps ARGS='busybox-1.00 clean' will make distclean  #"
	display_info "# only on busybox. In this case directory name should be    #" 
	display_info "# specified which happens to be busybox-1.00 for busybox.   #" 
	display_info "#                                                           #" 
	display_info "# images : builds kernel and rootfs images without doing    #"
	display_info "# clean or configure.                                       #"
	display_info "#                                                           #"
	display_info "# rootfs : builds only rootfs image without clean or config #"
	display_info "#                                                           #"
	display_info "# Just executing make will do build all without clean or    #"
	display_info "# configure.                                                #"
	display_info "#                                                           #"
	display_info "#############################################################"
	display_info "\n"
}
show_apps()
{
	display_info -e "\nYou may use any one of the following applications\n"
	cd "$USER_OPENSOURCE_DIR"
	for dir in `ls "$USER_OPENSOURCE_DIR" "$USER_IFXSOURCE_DIR"` 
	do
		if [ -e "$dir"/config.sh ]; then
			echo -e "$dir\t" 
		fi
	done
}


build_all()
{
	parse_args $@

	if [ $BUILD_CLEAN -eq 1 ]; then
		rm -rf "$BUILD_DIR"/* 2> /dev/null
		build_kernel "$clean"

		if  [ "$IFX_CONFIG_ADSL_UTILITIES" = "1" ]; then
			build_kernel configure
			# We do "configure" first time here, in case dsl_driver build clean fail.
			cd ${USER_IFXSOURCE_DIR}dsl_api/
			./build.sh configure
			cd -
		fi
		if [ "$CONFIG_LIBTUNE" = "y" ]; then
			if [ -d "$USER_LIB" ]&& [ "$clean" == "distclean" ];then
				rm -rf $USER_LIB
			fi
		fi

		build_ifx_apps "$clean"
		build_opensource_apps "$clean"
		build_uboot "$clean"
		parse_args $@
		if [ $BUILD_CONFIGURE -ne 1 ]; then
			exit 0;
		fi
	fi
	if [ $BUILD_CLEAN -eq 1 ];then
		ARGS=`echo $@ | sed "s,$clean,,"`
	else
		ARGS="$@"
	fi

	create_rootfs "$ARGS"
	if  [ "$IFX_CONFIG_ADSL_UTILITIES" = "1" ]; then
		build_kernel
	else
		build_kernel "$ARGS"
	fi
	build_ifx_apps "$ARGS"
	build_opensource_apps "$ARGS"
	build_kernel image
	build_kernel modules
	build_rootfs "$ARGS"
	if [ "$IFX_ADSL_FIRMWARE_IN_ROOTFS" != "1" ]; then
		build_firmware "$ARGS"
	fi
	build_uboot "$ARGS" 
	build_fullimage "$ARGS"
}

build_single_app ()
{
	dir=( $(find "$USER_SOURCE_DIR" -maxdepth 2 -name "$1" -printf "%f\n") )
	if [ "$dir" = "" ];then
		dir=( $(find "$USER_SOURCE_DIR" -maxdepth 2 -iname "*$1*" -printf "%f\n") )
		if [ ${#dir[@]} -gt 1 ];then	
			display_error "There are more than 1 applications with names matching with '$1'\nPlease choose one of the following:\n `echo ${dir[@]}`"
		fi
	fi
	if [ -e "$USER_OPENSOURCE_DIR"/"$dir" ]; then
		cd "$USER_OPENSOURCE_DIR"/"$dir"
		shift
		./build.sh $@
		ifx_error_check $?
	elif [ -e "$USER_IFXSOURCE_DIR"/"$dir" ]; then
		cd "$USER_IFXSOURCE_DIR"/"$dir"
		shift
		./build.sh $@
		ifx_error_check $?
	else
		show_apps
		display_error "Application "$dir" not configured"
	fi
}

build () 
{
	cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.generic ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
	cp -rf ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf.generic ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf

	if  [ "$IFX_CONFIG_INNOVE" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.innova ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
		cp -rf ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf.innova ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf
		#cp -rf ../../platforms/Amazon_SE/source/rootfs/flashdisk/etc/rc.d/init.d/rcS_Globe ../../platforms/Amazon_SE/source/rootfs/flashdisk/etc/rc.d/init.d/rcS
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flashdisk/etc/rc.d/init.d/xdslrc_Globe.sh ../../platforms/Amazon_SE/source/rootfs/flashdisk/etc/rc.d/init.d/xdslrc.sh
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flashdisk/etc/rc.d/rc.getIfStats_Globe ../../platforms/Amazon_SE/source/rootfs/flashdisk/etc/rc.d/rc.getIfStats
		cp -rf ../../platforms/Amazon_SE/tools/build_tools/rm_unneeded_files_Globe.sh ../../platforms/Amazon_SE/tools/build_tools/rm_unneeded_files.sh
	fi

	if  [ "$IFX_CONFIG_GVT" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.gvtbrazil ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
	fi

	if  [ "$IFX_CONFIG_CWPANAMA" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.cwp ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
	fi

	if  [ "$IFX_CONFIG_DIGITEL" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.digitel ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
	fi

	if  [ "$IFX_CONFIG_AZTECH_WEBPAGE" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.aztech ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
	fi
	
	if  [ "$IFX_CONFIG_MSO" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.mso ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
		cp -rf ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf.mso ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf
	fi

	if  [ "$IFX_CONFIG_ALLNET" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.allnet ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
		cp -rf ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf.allnet ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf
	fi
	if  [ "$IFX_CONFIG_WILTEL" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.wiltel ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
		cp -rf ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf.wiltel ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf
	fi
	if  [ "$IFX_CONFIG_TNN" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.tnn ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
		cp -rf ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf.tnn ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf
	fi
	if  [ "$IFX_CONFIG_PORTUGESE" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.portugese ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
		cp -rf ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf.portugese ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf
	fi
	if  [ "$IFX_CONFIG_BLUECOM" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.bluecom ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
	fi
	if  [ "$IFX_CONFIG_HARADA" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.harada ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
		cp -rf ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf.harada ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf
	fi

	if  [ "$IFX_CONFIG_AMPER" = "1" ]; then
		cp -rf ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf.amper ../../platforms/Amazon_SE/source/rootfs/flash/rc.conf
		cp -rf ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf.amper ../../source/user/ifx/tr69/Miscellaneous/AutoGenConf/tr69.conf
	fi

case $1 in
	kernel)
		shift
		build_kernel $@;;
	apps)
		shift
		case $1 in
			opensource)
				shift
				build_opensource_apps $@;;
			ifx)
				shift
				build_ifx_apps $@;;
			*)
				if [ "$1" = "clean" -o "$1" = "configure" -o "$1" = "" ];then
					build_ifx_apps $@
					ifx_error_check $?
					build_opensource_apps $@
					ifx_error_check $?
				else
					build_single_app $@
				fi
		esac;;
	rootfs)	
		shift
		build_rootfs $@;;
	images)	
		shift
		rm -rf "$BUILD_DIR"/*
		build_all;;
	firmware)
		shift
		build_firmware $@;;
	fullimage)
		shift
		build_fullimage $@;;
	uboot)
		shift
		build_uboot $@;;
	all)
		shift
		build_all $@;;

	help)
		shift
		PERUSE=`which less`
		if [ "$PERUSE" = "" ];then
			PERUSE=`which more`
		fi 
		show_detailed_help ;;#| "$PERUSE" ;;

	distclean)
		shift
		find "$TOOLS_DIR" -maxdepth 1 -type l | xargs rm -f
# Shouldn't touch. Found by Jeffrey.
#		rm -rf "$TOOLCHAIN_DIR"/hostutils
#		rm -f "$TOOLCHAIN_DIR"/.configured
		rm -f .config .vob .libs.old;;
	*) show_help;;
esac
}

if [ "$1" = "targets" ];then
	if [ buildconfig.sh -nt .for_check ]; then                                                                                     
	BUILD_TARGETS=( $(grep 'BUILD_' buildconfig.sh | sed "/is not set/d" |  sed "s/=y//" | sed "s/BUILD_//" | tr '[:upper:]' '[:lower:]') )
	ARGS=( $(grep 'ARGS_' buildconfig.sh | sed "/is not set/d" |  sed "s/=y//" | sed "s/ARGS_//" | tr '[:upper:]' '[:lower:]') )
	for target in ${BUILD_TARGETS[@]} 
	do
		case "$target" in
			all)
			build all "${ARGS[@]}" ;;
			kernel | uboot | rootfs | firmware | fullimage)
			build "$target" "${ARGS[@]}";;
			apps) ;;
#			drv_tapi_build | drv_vmmc_build)
#				build_tapi "$target" "${ARGS[@]}";;
			*)
			dir="`echo $target | tr '_' '?'`"
			dir="`find "$USER_SOURCE_DIR" -maxdepth 2 -iname "$dir" -printf "%f\n"`"
			build apps "$dir" "${ARGS[@]}";;
		esac	
	done
		
	fi
else
	build $@
fi
