#!/bin/bash

if [ -e ./.config ]; then
        . ./.config
	. ./featureconfig.sh
else
        . .gen_funcs
fi


DEBUG=0
show_lib_help()
{
        display_info "\nUsage :\n"
        display_info "./libtune.sh help"
	display_info "./libtune.sh prepare [DLSYM_FILE]"
        display_info "./libtune.sh build LIBNAME LIBARGS"
        display_info "./libtune.sh done"
	display_info "./libtune.sh { clean | distclean }\n"
	display_info "For e.g., ./libtune.sh build libc \$C_LIB_ARGS"
	display_info "\nFor additional information, please see"
	display_info "the documentation for this software"
}


sort_and_uniq()
{
	FILE=$1
	sort $FILE > $FILE-tmp
	mv $FILE-tmp $FILE
	uniq $FILE > $FILE-tmp
	mv $FILE-tmp $FILE
}

get_undefined_syms()
{
	DLSYM_FILE=$1
	cd $USER_BUILD_LIB
        rm -f unresolved
        touch unresolved
	if [ $DEBUG = 1 ];then
	        display_info "Getting unresolved symbols ..."
	fi
        for sym in `find "$BUILD_ROOTFS_DIR" -type f | xargs file | \
                egrep -i '(MSB executable|MSB shared object)' | awk -F ':' '{print $1}' | \
                xargs "$IFX_READELF" -s -W | awk '{ if ( $7 == "UND" ) print $8}'`
        do
                if [ "$sym" != "" ];then
                        echo "$sym" >> unresolved
                fi
        done
	if [ "$DLSYM_FILE" != "" ];then
                if [ ! -e $DLSYM_FILE ];then
                        display_info "File $DLSYM_FILE doesnt exist !!\n"
			return
                fi
		for dlsymbol in `cat $DLSYM_FILE`
	        do
        	        if [ "$dlsymbol" != "" ];then
                        	echo "$dlsymbol" >> unresolved
                	fi
        	done
        fi
	sort_and_uniq unresolved
}

get_needed_objs()
{
	cd $USER_BUILD_LIB
        rm -f needed
        touch needed
	if [ $DEBUG = 1 ];then
        	display_info "Generating list of needed objects ..."
        fi
	for sym in `cat unresolved`
        do
		obj=`awk -v pat="$sym" \
                '{ if ( $3 !~ /V|W/ \
                && $4 == pat) printf "%s/%s\n",$1,$2 }' \
                "$USER_LIB_OBJECTS"/defined`
                if [ "${obj[@]}" != "" ];then
			echo "${obj[@]}" >> needed
                else
                        obj=`awk -v pat1=libc -v pat2="$sym" \
                        '{ if ( $1 == pat1 && $3 ~ /V|W/ \
                        && $4 == pat2) printf "%s/%s\n",$1,$2 }' \
                        "$USER_LIB_OBJECTS"/defined`
                        if [ "${obj[@]}" != "" ];then
				echo "${obj[@]}" >> needed
                        fi
                fi
        done
	
	sort_and_uniq needed
        
	index=0
	LINES=1
	if [ $DEBUG = 1 ];then
        	display_info "Generating list of dependencies ..."
	fi
	while [ $index -le $LINES ]
        do
                file=( $( cat needed ) )
                file="${file[$index]}"
                if [ -e "$USER_LIB_OBJECTS"/"$file".depend ]; then
                        write_to_file needed \
                        `cat "$USER_LIB_OBJECTS"/"$file".depend`
                fi
                index=`expr $index + 1`
		LINES=`cat needed | wc -l `
        done
}

check_dep()
{
	rm -f dep_lib
	"$IFX_READELF" -a "$USER_LIB_PATH/$1" | \
	grep 'Shared library'|awk -F ':' '{print $2 }'| \
	sed 's/\[//'| sed 's/\]//' |sed 's/lib//'| \
	awk -F '.so' '{print $1}'  > dep_lib
	if [ -e dep_lib ];then
        	for library in `cat dep_lib`
        	do
			
			if [ "$library" != "" ] && [ ! -e "lib$library.so" ];then
				display_error " The needed library lib$library.so for rebuilding $1  is not built/selected."
			fi
        	done
	fi
}


check_rebuild_lib()
{
	`ls $USER_LIB_OBJECTS |grep -xq $1`
        if [ $? -ne 0 ]; then
                display_error "You can not rebuild $LIBRARY_NAME library.\nPlease run mklibdep on this library first."
        fi
}

build()
{
	cd $USER_BUILD_LIB
	LIBRARY_NAME=$1	
	shift
	ARGS=$@
	if [ "$LIBRARY_NAME" = "" ];then
		display_error "No library name was specified. Please see the help for this software"
	
	else 
		if [ "$ARGS" = "" ];then
			display_error "No arguments were specified to rebuild $LIBRARY_NAME library"
		fi
	fi	
	ARGS_LIBRARY_NAME=`echo  $ARGS | awk -F '-o ' '{print $2}'  | awk '{print $1}'`
	if [ "$LIBRARY_NAME" != "libc" ];then
		check_dep $ARGS_LIBRARY_NAME
	fi
	`ls $TOOLCHAIN_OBJECTS |grep -qx $LIBRARY_NAME`
	if [ $? -eq 1 ];then
		check_rebuild_lib $LIBRARY_NAME
		if [ -e $USER_LIB_PATH/old/$LIBRARY_NAME.so ];then                                                            
			diff  $USER_LIB_PATH/old/$LIBRARY_NAME.so $USER_LIB_PATH/$LIBRARY_NAME.so                                  
			if [ $? -eq 0 ] && [ -e $BUILT_LIB/$ARGS_LIBRARY_NAME ];then
				cp -f $BUILT_LIB/$ARGS_LIBRARY_NAME .
				return
			else
				cp -f "$USER_LIB_PATH"/"$LIBRARY_NAME".so "$USER_LIB_PATH"/old/"$LIBRARY_NAME".so
			fi
		fi
	else
		if [ -e $BUILT_LIB/$ARGS_LIBRARY_NAME ];then
			cp -af "$BUILT_LIB"/"$ARGS_LIBRARY_NAME" .
			return
		fi
	fi
	rm -f *.a
	if [ $DEBUG = 1 ];then 
		display_info "Tuning $LIBRARY_NAME library .."
	fi
	for obj in `awk '{print $3}' "$USER_LIB_OBJECTS"/"$LIBRARY_NAME"/order`
        do
                grep -xq "$LIBRARY_NAME/$obj" needed #&>/dev/null
                if [ $? -eq 0 ];then
                        "$IFX_AR" cr "$LIBRARY_NAME".a "$USER_LIB_OBJECTS"/$LIBRARY_NAME/$obj
                fi
        done
	if [ -e $LIBRARY_NAME.a ];then
		NEW_ARGS=$ARGS
		if [ "$LIBRARY_NAME" != "libc" ];then
			NEW_ARGS="$NEW_ARGS -lc"
			for dep in `cat dep_lib`
			do
				NEW_ARGS=" $NEW_ARGS -l$dep "
			done
		fi
		rm -f $ARGS_LIBRARY_NAME
                "$IFX_RANLIB" $LIBRARY_NAME.a
                "$IFX_LD" -L. $NEW_ARGS
		"$IFX_STRIP" "$STRIP_ARGS" "$ARGS_LIBRARY_NAME"
		cp -af "$ARGS_LIBRARY_NAME" "$BUILT_LIB"/
        fi
	cd $BUILD_TOOLS_DIR
}	

do_rebuild()
{
	cd $USER_BUILD_LIB
	rm -f .no_rebuild
        rm -rf "$BUILT_LIB"/*.so*
        get_needed_objs
	cd $BUILD_TOOLS_DIR
}
	
prepare()
{
	DLSYM_FILE=$1
        get_undefined_syms $DLSYM_FILE
	cd $USER_BUILD_LIB
	if [ -e $BUILD_ROOTFS_DIR/lib/ ];then
		cp -af "$BUILD_ROOTFS_DIR"/lib/*.so* .
	fi
	for lib in `cat "$TOOLS_DIR"/.libs.old`
        do
                cp -af "$USER_LIB_PATH"/"$lib" .
	done
	if [ -e unresolved_old ]; then
		sort_and_uniq unresolved_old
		diff -bru unresolved unresolved_old &>/dev/null
		if [ $? -ne 0 ]; then
			do_rebuild
			return
		fi
		touch .no_rebuild
		cp needed_old needed
		cd $BUILD_TOOLS_DIR
		return
	fi 
	do_rebuild
}

main()
{
	WHOAMI=`whoami`
        if [ "$WHOAMI" = "root" ];then
                display_error "Running this software as root is strongly discouraged.\nPlease run it as a normal user."
        fi
	
	case $1 in
		prepare)
			rm_libs
			shift
			prepare $@
			exit;;
		build) 
			shift
			build $@ 
			exit;;
		done)	
			cp_libs 
			cd $USER_BUILD_LIB
			mv unresolved unresolved_old
			if [ -e needed ];then
				mv needed needed_old
			fi
			rm -rf *.a .no_rebuild tmp dep_lib
			mv -f *.so* "$BUILD_ROOTFS_DIR"/lib/
			cd $BUILD_TOOLS_DIR 
			exit;;
		clean)
			cd $USER_BUILD_LIB
			rm -rf $BUILT_LIB/* *
			cd $BUILD_TOOLS_DIR
			exit;;
		distclean) 
			cd $BUILD_TOOLS_DIR
			rm -rf $USER_LIB 
			exit;;	 
		help) show_lib_help 
			exit;;
		*) show_lib_help 
			exit;;
	esac		
}

main $@
