#!/bin/sh

if [ -e ./.config ]; then
	. ./.config
else
	. .gen_funcs
fi

CONFIG_PATH=""
PLATFORM=

write_to_config()
{
	echo -e "$@" >> "$CONFIG_PATH"/Config.model."$PLATFORM"
}
unset_config()
{
	for model in $@
	do
		write_to_config "unset CONFIG_"$model""
	done
}

set_config()
{
	rm -f model_list
	models=( $(echo $@) )
	write_to_config "choice 'Model' \\"
	maxcount=`echo ${models[@]} | wc -w`
	for ((count=0; count < maxcount; count++))
	do
		echo -e "${models[$count]}\n" >> model_list
	done
	models=( $(cat model_list | sort -d) )
	for ((count=0; count < maxcount; count++))
	do
		if [ $count -eq 0 ];then
			write_to_config "\" \\"
		fi
		model="${models[$count]}"
		config="CONFIG_$model"
		write_to_config "$model\t$config \\"
		if (( count==maxcount-1 ));then
			write_to_config "\" $model"
		fi
	done
	rm -f model_list

}
get_models()
{
	platforms=( $(find ../../platforms -maxdepth 1 -type d -printf "%P\n" 2>/dev/null) )
	maxcount=`echo ${platforms[@]} | wc -w`
	for ((ii=0; ii < maxcount; ii++))
	do
                PLATFORM="${platforms[$ii]}"
                echo "# Supported "$PLATFORM" Models" > \
                        "$CONFIG_PATH"/Config.model."$PLATFORM"
                for ((jj=0; jj < maxcount; jj++))
                do 
                        if [ "$jj" != "$ii" ];then
                                models=( $(find ../../platforms/"${platforms[$jj]}"/config -maxdepth 1 -type d -printf "%P\n" 2>/dev/null) )
                                unset_config ${models[@]}
                        fi
                done
                models=( $(find ../../platforms/$PLATFORM/config -maxdepth 1 -type d -printf "%P\n" 2>/dev/null) )
		if [ ${#models[@]} -gt 0 ];then
			set_config ${models[@]}
			write_to_config "define_string PLATFORM_DIR \"`cd ../../platforms/$PLATFORM && pwd`"\"
			write_to_config "define_string MODEL \"\$(for name in ${models[@]}; do val=\$(eval echo \\\$\$\"CONFIG_\$name\"); if [ \"\$val\" = \"y\" ]; then echo \$name ; fi ; done)\""
		else
			rm -f "$CONFIG_PATH"/Config.model."$PLATFORM"
		fi
        done
}
tools_setup()
{
	setup_link "$1"/Config.in ./Config.board
	setup_link "$1"/Config.help .
	setup_link "$1"/Config.libs .
	setup_link "$1"/build_functions.sh ./.board_funcs
	setup_link "$1"/device_table.txt .
	setup_link "$1"/Config.default .
	CONFIG_PATH="`cd "$1" && pwd`"
	get_models
	for file in `ls "$1"/Config.model.*`
	do
		setup_link "$file" .
	done
	display_success "\nLinks to \"$1\" setup successfully\n"
}

env_setup()
{
        CC=`which cleartool 2>/dev/null`
        if [ "$CC" = "" ]; then 
                echo "n" > .vob
        else
                "$CC" pwv -short | grep NONE &> /dev/null
		if [ $? -eq 0 ]; then
			echo "n" > .vob
		else
			echo "y" > .vob
		fi
		
        fi
}

toolchain_setup()
{
	if [ -e "$TOOLCHAIN_DIR" ]; then
		cd "$TOOLCHAIN_DIR"
		if [ ! -e .configured ];then
			./setup.sh all
		fi
		if [ $? -ne 0 ]; then
			display_error "Unable to configure toolchain.\nCannot proceed with build."
		fi
		cd "$BUILD_TOOLS_DIR"
	else
		display_error "Invalid toolchain path $TOOLCHAIN_DIR.\nCannot proceed with build."
	fi
}

reconfig()
{
	RC=0
	cd "$1"
	if [ "$2" -nt ./.config ];then	
		cp -f "$2" .config
	fi
	touch -am .for_check
	sleep 1
	make menuconfig
	if [ ./.config -nt .for_check -a "$NEW_DEFAULTS" == "y" ]; then
		cp -a .config "$2"
		RC=$?
	fi
	rm .for_check
	if [ $RC -ne 0 ]; then
		display_error "You do not have write permissions for $2\nUnable to make new configuration as default."
	fi
	cd - &> /dev/null
}

reconfig_libs()
{
	if [ "$CONFIG_LIB_OPT" = "y" ]; then
		build_uclibc optimized
	elif [ "$CONFIG_LIB_CUST" = "y" ]; then
		build_uclibc custom
	elif [ "$CONFIG_LIB_DEF" = "y" ]; then
		build_uclibc default
	else
		display_error "Invalid library configuration"
	fi 
}

check_reconfig()
{
	if [ "$RECONFIG" = "y" ]; then
		display_info "\nChange kernel configuation ...\n"
		sleep 2 
		reconfig "$KERNEL_SOURCE_DIR" \
		"$CONFIG_DIR"/kernel_config
		display_info "\nChange busybox configuation ...\n"
		sleep 2 
		reconfig "$USER_OPENSOURCE_DIR"/busybox-1.00 \
		"$CONFIG_DIR"/busybox_config
	fi
	if [ "$RECONFIG_LIBS" = "y" ]; then
		reconfig_libs 
	fi
	if [ "$NEW_MODEL" = "y" ];then
		cp -f "$KERNEL_SOURCE_DIR"/.config \
		"$NEW_MODEL_PATH"/kernel_config	
		cp -f "$USER_OPENSOURCE_DIR"/busybox-1.00/.config \
		"$NEW_MODEL_PATH"/busybox_config
	fi
}
get_bool_y ()
{
	LINECOUNT="$1"
	FILE="$2"
	for count in  `seq 1 1 $LINECOUNT`
	do
		line="`awk "NR==$count" "$FILE"`"
		set=`echo $line | grep '=y' | sed "s/=y//"`
		if [ "$set" != "" ];then
			sed -i "/$set=y/c \define_bool $set y" "$FILE"
		fi
	done
}
get_bool_n ()
{
	LINECOUNT="$1"
	FILE="$2"
	for count in  `seq 1 1 $LINECOUNT`
	do
		line="`awk "NR==$count" "$FILE"`"
		unset=`echo $line | grep 'is not set' | awk '{print $2}'`
		if [ "$unset" != "" ];then
			sed -i "/# $unset is not set/c \define_bool $unset n" "$FILE"
		fi
	done
}
get_int ()
{
	LINECOUNT="$1"
	FILE="$2"
	for count in  `seq 1 1 $LINECOUNT`
	do
		line="`awk "NR==$count" "$FILE"`"
		option=`echo $line | grep '=' | sed "s/=/\ /" | awk '{print $1}'`
		value=`echo $line | grep '=' | sed "s/=/\ /" | awk '{print $2}'`
		isint=`echo "$value" | tr -d '[:digit:]'`
		if [ "$option" != "" -a "$isint" = "" ];then
			sed -i "/$option=$value/c \define_int $option $value" "$FILE"
		fi
	done
}
get_hex ()
{
	LINECOUNT="$1"
	FILE="$2"
	for count in  `seq 1 1 $LINECOUNT`
	do
		line="`awk "NR==$count" "$FILE"`"
		option=`echo $line | grep '=' | sed "s/=/\ /" | awk '{print $1}'`
		value=`echo $line | grep '=' | sed "s/=/\ /" | awk '{print $2}'`
		ishex=`echo "$value" | tr -d '[:xdigit:]'`
		if [ "$option" != "" -a "$ishex" == "x" ];then
			sed -i "/$option=$value/c \define_hex $option $value" "$FILE"
		fi
	done
}
get_string ()
{
	LINECOUNT="$1"
	FILE="$2"
	for count in  `seq 1 1 $LINECOUNT`
	do
		line="`awk "NR==$count" "$FILE"`"
		option=`echo $line | grep '=' | sed "s/=/\ /" | awk '{print $1}'`
		value=`echo $line | grep '=' | sed "s/=/\ /" | awk '{print $2}'`
		if [ "$option" != "" ];then
			sed -i "/$option=$value/c \define_string $option $value" "$FILE"
		fi
	done
}
convert_config()
{
	INFILE="$1"
	if [ -e "$INFILE" ];then
		cp -f "$INFILE" "$INFILE".new
		LINECOUNT=`cat featureconfig.sh | wc -l`
		get_bool_y "$LINECOUNT" "$INFILE".new
		get_bool_n "$LINECOUNT" "$INFILE".new
		get_int "$LINECOUNT" "$INFILE".new
		get_hex "$LINECOUNT" "$INFILE".new
		get_string "$LINECOUNT" "$INFILE".new
	else
		display_error "Incomplete configuration information"
	fi
}
convert_and_save_configs()
{
	convert_config featureconfig.sh
	convert_config modelconfig.sh
	if [ "$NEW_DEFAULTS" = "y" ];then
		cp -f featureconfig.sh.new \
		"$CONFIG_DIR"/apps_config
		if [ $? -ne 0 ];then
			display_error "You do not have write permission for "$PLATFORM_DIR"/config/"$MODEL"/apps_config"
		fi
		cp -f modelconfig.sh.new \
		"$CONFIG_DIR"/model_config
		if [ $? -ne 0 ];then
			display_error "You do not have write permission for "$PLATFORM_DIR"/config/"$MODEL"/model_config"
		fi
	fi
	if [ "$NEW_MODEL" = "y" ];then
		mkdir -p "$NEW_MODEL_PATH"
		cp -f featureconfig.sh.new "$NEW_MODEL_PATH"/apps_config
		cp -f modelconfig.sh.new "$NEW_MODEL_PATH"/model_config
	fi
	rm -f featureconfig.sh.new modelconfig.sh.new
}
config_setup()
{
	#CONFIG_DIR=$(cd `readlink Config.board | xargs dirname` && pwd)
	#sed -i "s,CONFIG_DIR=.*,CONFIG_DIR=\"$CONFIG_DIR\"," .config
	rm -f featureconfig modelconfig
	sed -i "s/=\"1\"/=y/" featureconfig.sh &>/dev/null
	if [ $model_changed -eq 1 -o \
	"$RESTORE" = y ];then
		cp -af "$PLATFORM_DIR"/config/"$MODEL"/* \
		"$CONFIG_DIR"
		chmod ug+rw "$CONFIG_DIR"/* &>/dev/null
		touch -am "$CONFIG_DIR"/* &>/dev/null
		setup_link "$PLATFORM_DIR"/config/"$MODEL"/apps_config featureconfig
		make features <<< E &>/dev/null
		setup_link "$PLATFORM_DIR"/config/"$MODEL"/model_config modelconfig
		make model <<< E &>/dev/null
	fi
	if [ "$RECONFIG" = "y" -o \
	"$NEW_MODEL" = "y" ];then
		display_info "Change feature configuration ..."
		sleep 2
		setup_link "$CONFIG_DIR"/Config.features featureconfig
		make features 
		display_info "Change model configuration ..."
		sleep 2
		setup_link "$CONFIG_DIR"/Config.model modelconfig
		make model 
		if [ "$NEW_DEFAULTS" = "y" -o \
		"$NEW_MODEL" = "y" ];then
			convert_and_save_configs
			if [ "$NEW_MODEL" = "y" ];then
				"$BUILD_TOOLS_DIR"/setup.sh -p "$CONFIG_DIR" &>/dev/null
				cp "$PLATFORM_DIR"/config/"$MODEL"/model.h \
				"$NEW_MODEL_PATH"
			fi
		fi
	fi
	rm -f featureconfig modelconfig
	sed -i "s/=y/=\"1\"/" featureconfig.sh
}
uboot_setup()
{
	if [ -e "$UBOOT_SOURCE_DIR" ]; then
		cd "$UBOOT_SOURCE_DIR"
		rm -f sysconfig.sh config.sh model_config.sh
		setup_link "$BUILD_TOOLS_DIR"/.config sysconfig.sh
		setup_link "$BUILD_TOOLS_DIR"/featureconfig.sh config.sh
		setup_link "$BUILD_TOOLS_DIR"/modelconfig.sh model_config.sh
		cd - > /dev/null
	else
		display_error ""$UBOOT_SOURCE_DIR" does not exist."
	fi	

}
rest_setup()
{
	display_info "Setting up links. Please wait ..."
	#save_oldconfigs
	config_setup
	toolchain_setup
	if [ "$CONFIG_LIBTUNE" = "y" ]; then
		toolchain_rebuild_setup
	fi
	plat_setup
	check_reconfig
	uboot_setup
}

case "$1" in
	-p | -P | -path)
		shift
		if [ -n "$1" -a -e "$1" ]; then
			tools_setup "$1"
			env_setup
		else
			display_error "\nInvalid Path\n"
		fi;;

	rest)
		if [ .config -nt .for_check ]; then
			has_model_changed
			model_changed=$?
			if [ "$RESTORE" = y ];then
				display_info "!! IMP !! Restoring default configuration for Model "$MODEL".\nThis will overwrite any changes that you have made.\nDo you want to continue ? [y/n]"
				read choice
				if [ "$choice" != y ];then
					exit 0;
				fi
			fi
			
			if [ $model_changed -eq 1 -o \
			"$RESTORE" = y -o \
			"$RECONFIG" = y ];then
				shift
				rest_setup
			fi
		fi;;
	*)
		display_error "\nUsage [ ./setup.sh -p /path/to/config ]\n"
esac
