#!/bin/sh 

## ============================================================================
## Copyright (C) 2003[- 2004] ? Infineon Technologies AG.
##
## All rights reserved.
## ============================================================================
##
## ============================================================================
##
## This document contains proprietary information belonging to Infineon 
## Technologies AG. Passing on and copying of this document, and communication
## of its contents is not permitted without prior written authorisation.
## 
## ============================================================================
##
## File Name: site_prep.sh 
## Author   : Mars Lin (mars.lin@infineon.com)
## Date     : 2005/05/13
##

## 507141:linmars: fix root user build problem
## 603091:fchang 2006/3/9 Modify the tool chain path for LXDB 1.1
## 603092:fchang 2006/3/9 Correct the tool-chain path for LXDB 1.1, the TOOLCHAIN_BIN_DIR will be specified in toolchain-env.sh instead
## 607101:jelly 2006/3/9 add platform selector
## 607111:leejack 2006/07/11 Integrate build environment
## 607131:linmars 2006/07/13 build mkimage fixed
## 607191:linmars 2006/07/19 add additional mechanism for prepare environment without user interactive
## 610121:linmars 2006/10/12 add relative path solution

##
## CONFIGURATIONS
##
SP_DEBUG_MODE=NO
SP_AUTO=0
SP_AUTO_ARGV=$@
SP_AUTO_ARGC=$#
SP_AUTO_RELATIVE_PATH=0
SP_AUTO_PLATFORM_NAME=
SP_AUTO_MODEL_NAME=

SP_CFG_FILENAME=.site_config
SP_CFGTMP_FILENAME=.site_config.tmp

SP_USERNAME=`whoami`

SP_PLATFORM_NAME=
SP_PACKAGE_VERSION=

SP_DEFAULT_CWD=/opt/infineon/amazon/tools/build_tools/
SP_DEFAULT_TOOLCHAIN_DIR=/opt/uclibc-toolchain/ifx-lxdb-1-1/gcc-3.3.6/toolchain-mips/bin

SP_DEFAULT_CROSS_COMPILER_PREFIX=mips-linux-
SP_DEFAULT_BUILD_DIR="`getent passwd | grep $SP_USERNAME`"
SP_DEFAULT_TFTP_DIR=/tftpboot
SP_DEFAULT_TARGET=mips-linux
SP_DEFAULT_HOST=i386-redhat-linux
SP_DEFAULT_BUILD=i386-pc-linux-gnu

SP_DEFAULT_FAKEROOT_DIR=/usr/local/bin/fakeroot

##
## SUB-ROUTINE
##

. ./common_funcs.sh

# check CWD
sp_check_dir() {
	CURR_DIR=`pwd`
	if [ "`basename $CURR_DIR`" != "build_tools" ] ; then
		prompt_nl "$SP_USERNAME: Your working directory is not an site preparsion directory"
		prompt_nl "    (e.q. $SP_DEFAULT_CWD)"
		exit 1
	fi
}

sp_check_default_build_dir()
{
	for i in $SP_DEFAULT_BUILD_DIR
	do
		if [ "`echo $i|cut -d: -f1`" == "$SP_USERNAME" ]; then
			PlatformName=`echo $SP_PLATFORM_NAME|tr [A-Z] [a-z]`
			SP_DEFAULT_BUILD_DIR="`echo $i|cut -d: -f6`/"$PlatformName"_build"
			break
		fi
	done
}

sp_select_platform () {
	if [ "$SP_AUTO" = "1" ]; then
		SP_PLATFORM_NAME=$SP_AUTO_PLATFORM_NAME
	else
		PlatFormID=0
		for PlatFormName in  $(ls ../../platforms/)
		do
			PlatFormID=$(($PlatFormID+1))
			echo "$PlatFormID) $PlatFormName"
		done
		PlatformSelecedID=`prompt_string "Please select the platform: " "1"`
	
		PlatFormID=0
		for PlatFormName in  $(ls ../../platforms/)
		do
			PlatFormID=$(($PlatFormID+1))
			if [ "$PlatformSelecedID" -eq "$PlatFormID" ]
			then
				SP_PLATFORM_NAME=$PlatFormName;
			fi
		done
	fi
}

sp_make_link () {
	for i in `cd $1;find .`
	do
		if [ ! -e $i ]
		then
			if [ ${SP_AUTO_RELATIVE_PATH} -eq 0 ]; then
				ln -sf $1$i $i
			else
				SML_TARGET=$i
				SML_RPATH=""
				while [ "`dirname $SML_TARGET`" != "." ]; do
					SML_RPATH=${SML_RPATH}../
					SML_TARGET=`dirname $SML_TARGET`
				done
				ln -sf ${SML_RPATH}$1$i $i
			fi
		fi
	done
}

sp_remove_link () {
	for i in `cd $1;find . `
	do
		if [ -L $i  ]
		then
			rm -fr $i
		fi
	done

}

sp_remove_previous_link() {
	if [ -e .ifx_platform_name ];
	then
		echo "Removing previous links, please wait..."

		KernelIfxDir=`cd ../../;pwd`/platforms/`cat .ifx_platform_name`/source/kernel/

		cd $KERNEL_SOURCE_DIR
		TargetDir=${KERNEL_IFX_DIR}os/
		sp_remove_link $TargetDir
		TargetDir=${KernelIfxDir}ifx/bsp/
		sp_remove_link $TargetDir
		TargetDir=${KernelIfxDir}ifx/os/
		sp_remove_link $TargetDir
		cd -

		TargetDir=${TOPDIR}platforms/`cat .ifx_platform_name`/
		cd $TOPDIR
        	sp_remove_link $TargetDir
		cd -

		rm .ifx_platform_name -rf
		rm mkimage -rf	
		echo "done."
	else
		echo "No previous link exist."
	fi
}

sp_make_working_link() {
	echo "Processing working links, please wait..."
	cd $TOPDIR
	if [ ${SP_AUTO_RELATIVE_PATH} -eq 0 ]; then
		TargetDir=${TOPDIR}platforms/$SP_PLATFORM_NAME/
	else
		TargetDir=./platforms/$SP_PLATFORM_NAME/
	fi
	sp_make_link $TargetDir
	cd -

	KernelIfxDir=`cd ../../;pwd`/platforms/$SP_PLATFORM_NAME/source/kernel/

	cd $KERNEL_SOURCE_DIR
	if [ ${SP_AUTO_RELATIVE_PATH} -eq 0 ]; then
		TargetDir=${KERNEL_IFX_DIR}os/
	else
		TargetDir=../../ifx/os/
	fi
	sp_make_link $TargetDir
	if [ ${SP_AUTO_RELATIVE_PATH} -eq 0 ]; then
		TargetDir=${KernelIfxDir}ifx/bsp/
	else
		TargetDir=../../../../platforms/$SP_PLATFORM_NAME/source/kernel/ifx/bsp/
	fi
	sp_make_link $TargetDir
	if [ ${SP_AUTO_RELATIVE_PATH} -eq 0 ]; then
		TargetDir=${KernelIfxDir}ifx/os/
	else
		TargetDir=../../../../platforms/$SP_PLATFORM_NAME/source/kernel/ifx/os/
	fi
	sp_make_link $TargetDir
	cd -

	echo $SP_PLATFORM_NAME > .ifx_platform_name	
	echo "done."
}

# basic configuration
sp_basic_config() {
	sp_select_platform
	TOPDIR=`cd ../../;pwd`/
	sp_save_parameter "TOPDIR" "$TOPDIR"
	KERNEL_SOURCE_DIR=${TOPDIR}source/kernel/opensource/linux-2.4.31/
	sp_save_parameter "KERNEL_SOURCE_DIR" "$KERNEL_SOURCE_DIR"
	KERNEL_INCLUDE_PATH=${KERNEL_SOURCE_DIR}inlcude/
	sp_save_parameter "KERNEL_INCLUDE_PATH" "$KERNEL_INCLUDE_PATH"
	sp_save_parameter "IFX_PLATFORM_NAME" "$SP_PLATFORM_NAME"
	TOOLCHAIN_DIR=`prompt_string "Please enter toolchain base directory: " "$SP_DEFAULT_TOOLCHAIN_DIR"`
	TOOLCHAIN_DIR=`absolute_path $TOOLCHAIN_DIR`
	sp_save_parameter "TOOLCHAIN_DIR" "$TOOLCHAIN_DIR"
	CROSS_COMPILER_PREFIX=`prompt_string "Please enter toolchain compiler prefix: " "$SP_DEFAULT_CROSS_COMPILER_PREFIX"`
	sp_save_parameter "CROSS_COMPILER_PREFIX" "$CROSS_COMPILER_PREFIX"
	sp_check_default_build_dir
	BUILD_DIR=`prompt_string "Please enter temporary directory for building system: " "$SP_DEFAULT_BUILD_DIR"`
	[ ! -e $BUILD_DIR ] && install -d $BUILD_DIR
	BUILD_DIR=`absolute_path "$BUILD_DIR"`
	sp_save_parameter "BUILD_DIR" "$BUILD_DIR"
	TFTP_DIR=`prompt_string "Would you want to copy all the images to TFTP server working directory automatically? [YES/NO]" "YES"`
	if [ "`echo $TFTP_DIR | tr a-z A-Z`" = "YES" ] ; then
		TFTP_DIR=`prompt_string "Please enter TFTP server working directory: " "$SP_DEFAULT_TFTP_DIR"`
		if [ ! -e $TFTP_DIR ]; then
			install -d $TFTP_DIR
		fi
		TFTP_DIR=`absolute_path "$TFTP_DIR"`
	else
		TFTP_DIR=""
	fi
	sp_save_parameter "TFTP_DIR" "$TFTP_DIR"

	FAKEROOT_DIR=`prompt_string "Please enter the directory for installed fakeroot: " "$SP_DEFAULT_FAKEROOT_DIR"`
        if [ ! -e $FAKEROOT_DIR ]; then
                prompt_nl "ERROR!! fakeroot cannot be found! Install fakeroot first!"
                exit 1
        fi
        sp_save_parameter "FAKEROOT_DIR" "$FAKEROOT_DIR"

	BUILD_ROOTFS_DIR=${BUILD_DIR}root_filesystem/
	if [ -e $BUILD_ROOTFS_DIR ]; then
		rm -rf $BUILD_ROOTFS_DIR
	fi
	install -d $BUILD_ROOTFS_DIR
	sp_save_parameter "BUILD_ROOTFS_DIR" "$BUILD_ROOTFS_DIR"
	
	UBOOT_SOURCE_DIR=${TOPDIR}source/u-boot/
	
	sp_save_parameter "UBOOT_SOURCE_DIR" "$UBOOT_SOURCE_DIR"
	USER_SOURCE_DIR=${TOPDIR}source/user/
	sp_save_parameter "USER_SOURCE_DIR" "$USER_SOURCE_DIR"
	USER_IFXSOURCE_DIR=${USER_SOURCE_DIR}ifx/
	sp_save_parameter "USER_IFXSOURCE_DIR" "$USER_IFXSOURCE_DIR"
	USER_OPENSOURCE_DIR=${USER_SOURCE_DIR}opensource/
	sp_save_parameter "USER_OPENSOURCE_DIR" "$USER_OPENSOURCE_DIR"
	IFX_APIS_DIR=${USER_IFXSOURCE_DIR}IFXAPIs/
	sp_save_parameter "IFX_APIS_DIR" "$IFX_APIS_DIR"
	KERNEL_IFX_DIR=${TOPDIR}source/kernel/ifx/
	sp_save_parameter "KERNEL_IFX_DIR" "$KERNEL_IFX_DIR"
	BUILD_TOOLS_DIR=${TOPDIR}tools/build_tools/
	sp_save_parameter "BUILD_TOOLS_DIR" "$BUILD_TOOLS_DIR"
	IFX_ROOTFS_DIR=${TOPDIR}source/rootfs/
	sp_save_parameter "IFX_ROOTFS_DIR" "$IFX_ROOTFS_DIR"
	TOOLCHAIN_BIN_DIR=`absolute_path "${TOOLCHAIN_DIR}../bin"`
	
	PATH=$BUILD_TOOLS_DIR:$PATH
        sp_save_parameter "PATH" "$PATH"

	IFX_CC=${CROSS_COMPILER_PREFIX}gcc
	sp_save_parameter "IFX_CC" "$IFX_CC"
	IFX_AR=${CROSS_COMPILER_PREFIX}ar
	sp_save_parameter "IFX_AR" "$IFX_AR"
	IFX_AS=${CROSS_COMPILER_PREFIX}as
	sp_save_parameter "IFX_AS" "$IFX_AS"
	IFX_LD=${CROSS_COMPILER_PREFIX}ld
	sp_save_parameter "IFX_LD" "$IFX_LD"
	IFX_NM=${CROSS_COMPILER_PREFIX}nm
	sp_save_parameter "IFX_NM" "$IFX_NM"
	IFX_STRIP=${CROSS_COMPILER_PREFIX}strip
	sp_save_parameter "IFX_STRIP" "$IFX_STRIP"
	IFX_RANLIB=${CROSS_COMPILER_PREFIX}ranlib
	sp_save_parameter "IFX_RANLIB" "$IFX_RANLIB"
	IFX_CXX=${CROSS_COMPILER_PREFIX}g++ 
	sp_save_parameter "IFX_CXX" "$IFX_CXX"
	IFX_CPP=${CROSS_COMPILER_PREFIX}cpp
	sp_save_parameter "IFX_CPP" "$IFX_CPP"
	IFX_OBJCOPY=${CROSS_COMPILER_PREFIX}objcopy
	sp_save_parameter "IFX_OBJCOPY" "$IFX_OBJCOPY"
	IFX_OBJDUMP=${CROSS_COMPILER_PREFIX}objdump
	sp_save_parameter "IFX_OBJDUMP" "$IFX_OBJDUMP"
	IFX_HOSTCC=gcc
	sp_save_parameter "IFX_HOSTCC" "$IFX_HOSTCC"
	SP_DEFAULT_IFX_CFLAGS="\"-Os -mips32 -mtune=4kc -I${IFX_APIS_DIR}include\""
	IFX_CFLAGS="$SP_DEFAULT_IFX_CFLAGS"
	sp_save_parameter "IFX_CFLAGS" "$IFX_CFLAGS"
	SP_DEFAULT_IFX_LDFLAGS="\"-L${BUILD_ROOTFS_DIR}lib -Wl,-rpath-link,${BUILD_ROOTFS_DIR}lib\""
	IFX_LDFLAGS="$SP_DEFAULT_IFX_LDFLAGS"
	sp_save_parameter "IFX_LDFLAGS" "$IFX_LDFLAGS"
	TARGET=$SP_DEFAULT_TARGET
	sp_save_parameter "TARGET" "$TARGET"
	HOST=$SP_DEFAULT_HOST
	sp_save_parameter "HOST" "$HOST"
	BUILD=$SP_DEFAULT_BUILD
	sp_save_parameter "BUILD" "$BUILD"
	sp_remove_previous_link
	sp_make_working_link
	SP_PACKAGE_VERSION=`cat .ifx_package_version`
	sp_save_parameter "IFX_PACKAGE_VERSION" "$SP_PACKAGE_VERSION"
}

sp_utility_prepare() {
	if [ "$SP_AUTO" = "1" ]; then
		./build_config.sh -n $SP_AUTO_MODEL_NAME
	else
	        ./build_config.sh
	fi
	if [ ! -x squashfs2.2/squashfs-tools/mksquashfs ] ; then
		cd squashfs2.2/squashfs-tools/
		make
		cd -
	fi
	if [ ! -x squashfs-lzma/squashfs/squashfs2.2/squashfs-tools/mksquashfs-lzma ]; then
		cd squashfs-lzma/lzma/lzma406/SRC/7zip/Compress/LZMA_Lib
		make
		cd -	
		cd squashfs-lzma/squashfs/squashfs2.2/squashfs-tools
		make
		cd -
	fi
	if [ ! -x mkimage ] ; then
		cd $UBOOT_SOURCE_DIR
		. ${TOPDIR}tools/build_tools/Path.sh
		. ${TOPDIR}tools/build_tools/model_config.sh
		CROSS_COMPILE="mips-linux-" make ifx_mips_config
		TBVALUE1=`cat board/ifx/config.mk |grep ^TEXT_BASE |cut -d'=' -f2 | cut -d' ' -f1`
		TBVALUE2=`cat board/ifx/config.mk |grep ^TEXT_BASE |cut -d'=' -f2 | cut -d' ' -f2`
		if [ "${TBVALUE2}" != "" ]; then
			TBVALUE=${TBVALUE2}
		elif [ "${TBVALUE1}" != "" ]; then
			TBVALUE=${TBVALUE1}
		else
			TBVALUE=0xB0000000
		fi
		OBJTREE="`pwd`" CURDIR="`pwd`" SRCTREE="`pwd`" TOPDIR="`pwd`" TEXT_BASE="${TBVALUE}" CROSS_COMPILE="mips-linux-" make IFX_CFLAGS="-DIFX_CONFIG_FLASH_SIZE=${IFX_CONFIG_FLASH_SIZE} -DIFX_CONFIG_MEMORY_SIZE=${IFX_CONFIG_MEMORY_SIZE}" -C tools/ mkimage
		cd -
		cp -af ${UBOOT_SOURCE_DIR}tools/mkimage .
	fi
}

sp_post_operation () {
	echo "#!/bin/sh" > Path.sh
	cat "$SP_CFG_FILENAME" >> Path.sh
	echo ". ${BUILD_TOOLS_DIR}common_funcs.sh" >> Path.sh

	echo ". ${TOOLCHAIN_BIN_DIR}toolchain-env.sh" >> Path.sh

	chmod 755 Path.sh
}

site_prepare() {
	touch "$SP_CFGTMP_FILENAME"
	sp_check_dir
	sp_basic_config
	mv "$SP_CFGTMP_FILENAME" "$SP_CFG_FILENAME"
	sp_post_operation
	sp_utility_prepare

}

rm -f $SP_CFG_FILENAME

if [ ! -e "$SP_CFG_FILENAME" ] ; then
	if [ "$SP_AUTO_ARGV" != "" ]; then
		sp_i=0
		while [ $sp_i -lt $SP_AUTO_ARGC ]; do
			sp_i=`expr $sp_i + 1`
			eval sp_argv='$'$sp_i
			case "$sp_argv" in
			-r)
				SP_AUTO_RELATIVE_PATH=1
				echo "SP_AUTO_RELATIVE_PATH=$SP_AUTO_RELATIVE_PATH"
				;;
			-a)
				if [ `expr $SP_AUTO_ARGC - $sp_i` -lt 2 ]; then
					echo "./site_prep.sh [-a PLATFORM MODEL]"
					exit 1
				fi
				SP_AUTO=1
				eval SP_AUTO_PLATFORM_NAME='$'`expr $sp_i + 1`
				eval SP_AUTO_MODEL_NAME='$'`expr $sp_i + 2`
				SP_AUTO_TARGET=../../platforms/$SP_AUTO_PLATFORM_NAME/config/$SP_AUTO_MODEL_NAME
				if [ ! -e $SP_AUTO_TARGET -o ! -e $SP_AUTO_TARGET ]; then
					echo "Model $SP_AUTO_MODEL_NAME in platform $SP_AUTO_PLATFORM_NAME is not existed!!"
					exit 1
				fi
				echo "SP_AUTO_PLATFORM_NAME=$SP_AUTO_PLATFORM_NAME, SP_AUTO_MODEL_NAME=$SP_AUTO_MODEL_NAME"
				sp_i=`expr $sp_i + 2`
				;;
			*)
				echo "Unknow argv: $sp_argv"
				exit 1
				;;
			esac
		done
	fi
	site_prepare
fi

